unit GTI2Util;

interface

uses
  SysUtils, Windows, Messages, Classes, Dialogs, comctrls,
  Graphics, Forms, printers, controls, consts, inifiles,
  math, Maths, StdCtrls, gti2Const, uniteKer, registry;

const
        Digits : array[0..$F] of Char = '0123456789ABCDEF';
        HorlogeMaxPWM = 16000000; { 16 MHz }
        TraitMagnum = 0.0018;  { demi pas }
        ZeroMagnum = 30000;
        MaxiHardGBF = 16383;
        ZeroGBF = 8192;
        GammeGBF = 20; // -10 +10 V

        PrecisionGTI2 = 5;
        PrecisionTime = 7;
        SymbReg : char = '';
        SymbReg2 : char = '&';
        RegistreGTI2 = 'Software\GTI2';

type
    EGetFloatError = class(Exception);
    ChaineAzt = array[0..255] of char;
    pseudoWord = record case byte of
             0 : (octets : array[0..9] of char);
             1 : (mot : word);
             2 : (entier : smallInt);
             3 : (longMot : longWord);
             4 : (octet : byte);
             5 : (index : array[1..4] of byte;valeur : array[1..4] of word);
             6 : (Entre,Sortie,phase,Freq,Nombre : word);
    end;
    TvaleurMagnum = record
            case boolean of
                 false : (octets : array[0..5] of char);
                 true : (DateM : longWord;posM : word);
            end;
     Tcalibre = (c10m,c20m,c50m,c100m,c200m,c500m,c1,c2,c5,c10,c20);

        TsortiePWM = class(Tobject)
            protected
            Ffrequence : integer; { en Hz }
            Fduty100 : integer; { en % }
            Fperiode : integer; { 0..65535 }
            Fhorloge : integer; { 0=16 MHz ... 4=2 MHz }
            FcodeDuty : integer; { 0..Fperiode }
            FperiodeOK : boolean;
            Procedure setFrequence(Avalue : integer);
            Procedure setDuty(Avalue : integer);
            Procedure setPeriode;
            public
            PWMi : String[1];
            property Frequence : integer read Ffrequence write setFrequence;
            property Duty100 : integer read Fduty100 write setDuty;
            constructor create;
            Function GetNomVoieOrphy : shortString;
            Function CodePWM : shortString;
            procedure load(Avoie : shortString;Fichier : tMemIniFile);
            procedure store(Avoie : shortString;Fichier : tMemIniFile);
        end;

        Tmagnum = class(Tobject)
            private
               FmodeAngulaire,FmodeRadian,Fchute : boolean;
               procedure setModeAngulaire(value : boolean);
               procedure setChute(value : boolean);
               function getNomUnite : shortString;
            public
            increment : double;
            OscilMin : integer;
            DureeOscillation : double; // s
            isCodeurInc,codeurActif,Negatif : boolean;
            TraitMagnumE : double; // E=effective
            EFi,EBi : byte;
            property modeAngulaire : boolean read FmodeAngulaire write setModeAngulaire;
            property modeRadian : boolean read FmodeRadian write FModeRadian;
            property nomUnite : shortString read getNomUnite;
            property chute : boolean read Fchute write setChute;
            constructor create;
            Function ValeurReal(V : integer) : double;
            procedure load(Avoie : shortString;Fichier : tMemIniFile);
            procedure store(Avoie : shortString;Fichier : tMemIniFile);
            Function codeMagnum : shortString;
            Function stopMagnum : shortString;
            function codeCodeurInc : shortString;
        end;

Const
        crSupprArr = char(vk_back);
        crSuppr = char(vk_clear);
        crHaut = char(vk_up);
        crBas = char(vk_down);
        crGauche = char(vk_left);
        crDroite = char(vk_right);
        crHome = char(vk_home);
        crEnd = char(vk_end);
        crCR = #13;
        crLF = #10;
        crESC = #27;
        crTab = char(vk_tab);
        Separateur : TCharSet = [' ',crTab,','];
        ToutCarac   : TcharSet = [#032..#151,#224..#250];
        ChiffreEtc : TcharSet = ['0'..'9','+','-','e','E','.'];
        ChiffreWin : TcharSet = ['0'..'9','+','-','e','E','.',','];
        ToucheEdition : TcharSet = [crSupprArr,crGauche,crDroite,crHome,crEnd,crSuppr];

        CalibreStr : array[Tcalibre] of string[6] =
             ('10 mV','20 mV','50 mV','100 mV','200 mV','500 mV',
              '1 V','2 V','5 V','10 V','20 V');

Function OKReg(NumOK : shortString) : boolean;
Procedure Info(NumInfo : shortString);
Procedure afficheErreur(Erreur : shortString);
Procedure afficheErreurFormat(Erreur : shortString;args : array of const);
Function ToucheValidation(key : word) : boolean;
Procedure TrimComplet(var S : shortString);
Function Pad(S : shortString;Len : integer) : shortString;
Function ConvertitChaine(nombre : shortString;var valeur : double) : boolean;
Function FormatDefaut(nombre : double) : shortString;
Function CompareStr(s1,s2 : shortString) : boolean;
Function FloatToStrWin(valeur : double) : shortString;
Function FloatToStrReg(valeur : double) : shortString;
Function StrToFloatWin(nombre : shortString) : double;
Function EncodeDuree(duree : LongInt) : TdateTime;
Function DureeEcoulee(InstantDebut : TdateTime) : double;
function PosSeparateur(C : char;S : string;N : Integer): Integer;
Function existeOption(opt : char;defaut : boolean) : boolean;
Function GetFloat(Nombre : ShortString) : double;
Function FormatGTI2(nombre : double) : shortString;
Function FormatTime(nombre : double) : shortString;
Function FloatToStrPoint(valeur : double;precision : integer) : shortString;
Function DateTimeToFloat(nombre : shortString) : double;
function valeur125(valeur : double) : double;

{-------- Numeric conversion -----------}
function BinaryB(B : Byte) : shortString;
function BinaryW(W : Word) : shortstring;
function HexB(B : Byte) : shortstring;
function HexW(W : Word) : shortstring;

var
     sortiePWM : TsortiePWM;
     magnum : Tmagnum;

implementation

Procedure UpCaseStr(var s : shortstring);
var i : integer;
begin
     for i := 1 to length(s) do s[i] := UpCase(S[i])
end;

Procedure TrimComplet(var s : shortstring);
var longueur,posCourant : integer;
begin
	longueur := Length(s);
	posCourant := 1;
	while (posCourant<=longueur) do
		if ord(s[posCourant])<=32
			then begin
			   Delete(s,posCourant,1);
			   dec(longueur);
			end
			else inc(posCourant);
end;

Function OKReg(NumOK : shortString) : boolean;
begin
     OKreg := MessageDlg(NumOK,mtConfirmation,[mbYes,mbNo],0)=idYes
end;

Procedure Info(NumInfo : shortString);
begin
     MessageDlg(NumInfo,mtInformation,[mbOK],0)
end;

Procedure ConvertitPrefixe(var nombre : shortString);
var u : coefUnite;
    c : char;
begin
   c := nombre[length(nombre)];
   if c in caracPrefixe then begin
        u := zero;
        repeat inc(u) until (c=prefixeUnite[u]);
        delete(nombre,length(nombre),1);
        nombre := nombre+expUnite[u];
   end;
end;

Function StrToFloatWin(nombre : shortString) : double;
var posSeparator : integer;
begin
   if decimalSeparator='.' then begin
      posSeparator := pos(',',nombre);
      if posSeparator>0 then Nombre[posSeparator] := '.'
   end;
   if decimalSeparator=',' then begin
      posSeparator := pos('.',nombre);
      if posSeparator>0 then Nombre[posSeparator] := ','
   end;
   ConvertitPrefixe(nombre);
   try
   StrToFloatWin :=  strToFloat(nombre);
   except
   StrToFloatWin := sansSignif;    
   end;
end;

Function FloatToStrWin(valeur : double) : shortString;
begin
   result := FloatToStrF(valeur,ffGeneral,5,2)
end;

Function FloatToStrReg(valeur : double) : shortString;
var posSeparator : integer;
begin
   result := FloatToStrF(valeur,ffGeneral,5,2);
   if decimalSeparator=',' then begin
      posSeparator := pos(',',result);
      if posSeparator>0 then result[posSeparator] := '.';
   end;
end;

Function ConvertitChaine(nombre : shortString;var valeur : double) : boolean;
begin
   if (upcase(nombre[1])='E')
	  then nombre := '1'+nombre
	  else if (nombre[1]='.') or (nombre[1]=',') 
		   then nombre := '0'+nombre;
   valeur := StrToFloatWin(nombre);
   convertitChaine := valeur<>sansSignif;
end;

Function FormatDefaut(Nombre : double) : shortString;
var Nexp : integer;
begin
     if Nombre=sansSignif then begin
        result := ' ';
        exit;
     end;
     if abs(Nombre)<1e-10
         then Nexp := 2
         else if abs(Nombre)<1e+10
              then Nexp := 1
              else Nexp := 2;
      result := FloatToStrF(nombre,ffGeneral,5,Nexp)
end;

procedure afficheErreur(Erreur : shortString);
var SauveCursor : Tcursor;
begin
     SauveCursor := Screen.cursor;
     Screen.cursor := crDefault;
     MessageDlg(Erreur,mtError,[mbOK],0);
     Screen.cursor := SauveCursor;
end;

procedure afficheErreurFormat(Erreur : shortString;args : array of const);
var SauveCursor : Tcursor;
begin
     SauveCursor := Screen.cursor;
     Screen.cursor := crDefault;
     MessageDlg(Format(Erreur,args),mtError,[mbOK],0);
     Screen.cursor := SauveCursor;
end;

Function CompareStr(s1,s2 : shortstring) : boolean;
begin
    upCaseStr(s1);
    upCaseStr(s2);
    compareStr := s1=s2;
end;

Function ToucheValidation(key : word) : boolean;
begin
    ToucheValidation := key in [vk_tab,vk_left,vk_right,vk_down,vk_up,vk_return]
end;

Procedure ValidateDouble(Sender : Tedit;var z : double);
begin
     z := StrToFloatWin(sender.text);
     if z=sansSignif then sender.text := FloatToStr(z);
end;

function Pad(S : shortString;Len : integer) : shortstring;
{-Return a string right-padded to length len with ' '}
var
  o : shortstring;
  Slen : integer;
begin
   Slen := length(S);
   if SLen>Len then S := copy(S,1,Len);
   setLength(o,len);
   Move(S[1], o[1], SLen);
   FillChar(o[Succ(SLen)], Len-SLen,' ');
   Pad := o;
end;

Function EncodeDuree(duree : LongInt) : TdateTime;
var h,m,s,ms : word;
begin
    h := duree div 3600;
    duree := duree-h*3600;
    m := duree div 60;
    duree := duree-m*60;
    s := duree;
    ms := 0;
    EncodeDuree := EncodeTime(h,m,s,ms);
end;

Function DureeEcoulee(InstantDebut : TdateTime) : double;
var h,m,s,ms : word;
begin
      decodeTime(now-instantDebut,h,m,s,ms);
      DureeEcoulee := h*3600.0+m*60.0+s*1.0+ms*0.001;
end;

function PosSeparateur(C : char;S : string;N : Integer): Integer;
var
  j,i : Integer;
begin
  j := 0;
  for i := 1 to length(S) do
      if S[i]=C then begin
         inc(j);
         if j=N then begin
            result := i;
            exit;
         end;
      end;   
  result := 0;
end;

Function existeOption(opt : char;defaut : boolean) : boolean;
var str : shortstring;
    i : integer;
begin
    result := defaut;
    for i := 1 to ParamCount do begin
       str := ParamStr(i);
       if (length(str)>1) and (str[1]='/') and (upcase(str[2])=opt) then begin
          if length(str)=2
             then result := true
             else result := (str[3]='+');
          exit;
       end;
    end;
end;

Function GetFloat(Nombre : ShortString) : double;
Const
    MaxPile = 10;
    CaracCte : TcharSet = [piMin,piMaj,'P','I','p','i'];
type
    IndicePile  = 1..MaxPile;
Var
    Pile : Array[IndicePile] of double;
    IndicePileCourant : 0..MaxPile;
    CaracCourant   : char;
    PointeurExp : byte;

Procedure suivant;
{Lit le prochain caractre et l'assigne  caracCourant}
{Le caracCourant est toujours extrait avant appel d'un analyseur}
begin
    inc(pointeurExp);
    if pointeurExp<=length(Nombre)
       then caracCourant := Nombre[pointeurExp]
       else caracCourant := #0;
end;

Procedure InitFonctor;
Begin
  pointeurExp := 0;
  indicePileCourant := 0;
  suivant;
end;{ InitFonctor }

{Les procdures LireXXX extraient un XXX de la chaine Nombre  partir du
 pointeur courant }

Function LireNombre : double;
    Var PointeurChiffre : byte;
        Valeur : double;
        NombreLoc : ShortString;

    Procedure lireEntier;
    begin
       While ( caracCourant in chiffre ) do suivant
    end;

    Begin
        PointeurChiffre := PointeurExp;{pointe le premier chiffre du nombre}
        lireEntier;
        If ( caracCourant in ['.',','] ) {C'est un nombre dcimal}
           Then begin
              suivant;{on saute le .}
              lireEntier
           end;
        If ( upcase(caracCourant) = 'E' ) {puissance de 10}
           then begin
              suivant;{on saute le E}
              if (caracCourant='+') or (caracCourant='-')  then suivant;
              lireEntier
           end
           else if caracCourant in caracPrefixe then suivant {coeff unit};
        NombreLoc := copy(nombre,pointeurChiffre,pointeurExp-PointeurChiffre);
        if not ConvertitChaine(nombreLoc,valeur)
           then raise EGetFloatError.Create('Erreur nombre');
        LireNombre := valeur;
   end; { lireNombre }

Function LireIdEnt : ShortString;
Var PointeurIdent : byte;
Begin
  PointeurIdent := pointeurExp;
  While (caracCourant in CaracCte) do suivant;
  if (pointeurExp-pointeurIdent)>longUnite then raise EGetFloatError.Create('Identificateur trop long (>8)');
  LireIdent := copy(nombre,pointeurIdent,pointeurExp-pointeurIdent)
End;{LireIdEnt}

Procedure Depile(Var P : double);
Begin
  if indicePileCourant=0
    then raise EGetFloatError.Create('Erreur interne')
    else begin
      P := Pile[indicePileCourant];
      dec(indicePileCourant);
   end;
End;

Procedure Empile(P : double);
Begin
  if indicePileCourant=MaxPile
    then raise EGetFloatError.Create('Trop complexe')
    else begin
      inc(indicePileCourant);
      Pile[indicePileCourant] := P;
   end;
End;

Function CalcOperateur(C : char;PG,PD : double) : double;
begin
   case C of
         '+'  : calcOperateur := PG+PD;
         '-'  : calcOperateur := PG-PD;
         '/'  : calcOperateur := PG/PD;
         '*'  : calcOperateur := PG*PD;
         else CalcOperateur := 0;
   end;
end;

Procedure AnalyseExpression;Forward;

Procedure E;
{E=(.Expression.)}
Begin
  If caracCourant='('
    Then Begin
        suivant;
        AnalyseExpression;
        case caracCourant of
             ')'  : suivant;
             #0 : ; { fin => on ajoute ) manquante }
             else raise EGetFloatError.Create('Erreur )');
        end;{case}
    End
    Else raise EGetFloatError.Create('Erreur (')
End;{E}

Procedure AnalyseElement;
{Element = E/nombre}
Var
  U : ShortString;
Begin
  If (caracCourant in [piMin,piMaj,'P','p'])
    Then Begin
        U := LireIdent;
        U := AnsiUpperCase(U);
        if (U=piMin) or (U='PI')or (U=piMaj)
           then Empile(pi)
           else raise EGetFloatError.Create('Erreur nombre')
    End {Then}
    Else If caracCourant in Chiffre+['-','+','.']
        Then Empile(LireNombre)
        Else E
End;{analyseElement}

Procedure AnalyseTerme;
{terme = facteur.[* ou /].element}
Var
  C        : Char;
  T1,T2    : double;
Begin
  AnalyseElement;
  While (caracCourant='*') or (caracCourant='/') do Begin
       C:=caracCourant;
       suivant;
       AnalyseElement;
       depile(T2);
       depile(T1);
       Empile(CalcOperateur(C,T1,T2));
    End {while}
End; { AnalyseTerme }

Procedure AnalyseExpression;
{Expression = [+ ou - ou rien].Terme.[+ ou -].terme}
Var
  C      : Char;
  T1,T2  : double;
Begin
  If (caracCourant='+') or (caracCourant='-')
     Then Begin
        C:=caracCourant;
        suivant
     end
     Else C:='+';
  AnalyseTerme;
  If C='-' Then begin
         depile(T1);
         Empile(-T1)
  end;
  While (caracCourant='+') or (caracCourant='-') do Begin
      C:=caracCourant;
      suivant;
      AnalyseTerme;
      depile(T2);
      depile(T1);
      Empile(CalcOperateur(C,T1,T2));
  End {while}
End;{AnalyseExpression}

var posVirgule : integer;
Begin {getFloat}
  trimComplet(Nombre);
  if nombre='' then begin
     result := sansSignif;
     exit;
  end;
  posVirgule := pos(',',nombre);
  while posVirgule>0 do begin
      Nombre[posVirgule] := '.';
      posVirgule := pos(',',nombre);      
  end;
  InitFonctor;
  AnalyseExpression;
  if pred(pointeurExp)<>length(nombre) then
     raise EGetFloatError.Create('Erreur nombre');
  result := Pile[1];
end;{getFloat}

Function FormatGeneral(Nombre : double;Nchiffres : integer) : shortString;
var Nexp : integer;
begin
     if Nombre=sansSignif then begin
        FormatGeneral := ' ';
        exit;
     end;
     if abs(Nombre)<1e-10
         then Nexp := 2
         else if abs(Nombre)<1e+10
              then Nexp := 1
              else Nexp := 2;
      FormatGeneral := FloatToStrF(nombre,ffGeneral,Nchiffres,Nexp)
end;


Function FormatGTI2(Nombre : double) : shortString;
begin
     result := FormatGeneral(Nombre,PrecisionGTI2)
end;

Function FormatTime(Nombre : double) : shortString;
begin
     FormatTime := FormatGeneral(Nombre,PrecisionTime)
end;

function PosN(const C : shortString;S : shortString;const K : Integer): Integer;
var
  P : Integer;
begin
  Result := 0;
  if K>0 then Delete(S,1,K);
  P := Pos(C, S);
  if P>0 then result := K+P;
end;



Function DateTimeToFloat(nombre : shortString) : double;
var i,Njour : integer;
begin
     result := 0;
     trimComplet(nombre);
     i:=pos(':',nombre);
     if i>0
        then begin
           i:=pos('j',nombre);
           Njour := 0;
           if i>0
               then begin
                 if i>1 then begin
                   try
                   Njour := StrToInt(copy(nombre,1,pred(i)));
                   except
                   end;
                 end; { i>1 }
                 delete(nombre,1,i);
               end { i>0 }
               else begin
                   i := posN(':',nombre,2);
                   if i=0 then nombre := '0:'+nombre;
               end;
           try
           result := (StrToTime(nombre)+Njour)*60*60*24;
           except
           end;
        end {i>0}
        else begin
        i:=pos('j',nombre);
        if i>0 then begin
           if i>1 then begin
              try
              result := StrToInt(copy(nombre,1,pred(i)))*60*24*60;
              except
              end;
           end;
           delete(nombre,1,i);
        end;
        i:=pos('h',nombre);
        if i>0 then begin
           if i>1 then begin
              try
              result := result+StrToInt(copy(nombre,1,pred(i)))*60*60;
              except
              end;
           end;
           delete(nombre,1,i);
        end;
        i:=pos('min',nombre);
        if i>0 then begin
           if i>1 then begin
              try
              result := result+StrToFloatWin(copy(nombre,1,pred(i)))*60;
              except
              end;
           end;
           delete(nombre,1,i+2);
        end;
        i:=pos('m',nombre);
        if i>0 then begin
           if i>1 then begin
              try
              result := result+StrToFloatWin(copy(nombre,1,pred(i)))*1e-3;
              except
              end;
           end;
           delete(nombre,1,i);
        end;
        i:=pos('',nombre);
        if i>0 then begin
           if i>1 then begin
               try
               result := result+StrToFloatWin(copy(nombre,1,pred(i)))*1e-6;
               except
               end;
           end;
           delete(nombre,1,i);
        end;
        i:=pos('s',nombre);
        if i>0 then delete(nombre,i,1);
        if nombre<>'' then begin
            try
            result := result+strToFloatWin(nombre);
            except
            end;
        end;
        if result = 0 then result := 0.1;
     end;
end;

function valeur125(valeur : double) : double;
var mantisse,exposant,coeff : double;
begin
     exposant := log10(valeur);
     coeff := Int(exposant);
     if (exposant<0) and (coeff<>exposant) then coeff := coeff-1;
     coeff := power(10,coeff);
     mantisse := Int(valeur/coeff);
     if mantisse>2 then if mantisse<6
           then mantisse := 5
           else mantisse := 10;
     result := mantisse*coeff;
end;

Function FloatToStrPoint(valeur : double;precision : integer) : shortString;
var nombre : shortString;
    posSeparator : byte;
begin
   nombre := FloatToStrF(valeur,ffGeneral,precision,2);
   if decimalSeparator<>'.' then begin
      posSeparator := pos(decimalSeparator,nombre);
      if posSeparator>0 then Nombre[posSeparator] := '.';
   end;
   result := nombre;
end;

  function HexB(B : Byte) : shortString;
    {-Return hex string for byte}
  begin
    setLength(result,2);
    HexB[1] := Digits[B shr 4];
    HexB[2] := Digits[B and $F];
  end;

  function HexW(W : Word) : shortString;
    {-Return hex string for word}
  begin
    setLength(result,4);
    HexW[1] := Digits[hi(W) shr 4];
    HexW[2] := Digits[hi(W) and $F];
    HexW[3] := Digits[lo(W) shr 4];
    HexW[4] := Digits[lo(W) and $F];
  end;

  function BinaryB(B : Byte) : shortString;
    {-Return binary string for byte}
  var
    I, N : Word;
  begin
    N := 1;
    setlength(result,8);
    for I := 7 downto 0 do begin
      BinaryB[N] := Digits[Ord(B and (1 shl I) <> 0)]; {0 or 1}
      Inc(N);
    end;
  end;

  function BinaryW(W : Word) : shortString;
    {-Return binary string for word}
  var
    I, N : Word;
  begin
    N := 1;
    setlength(result,16);
    for I := 15 downto 0 do begin
      BinaryW[N] := Digits[Ord(W and (1 shl I) <> 0)]; {0 or 1}
      Inc(N);
    end;
  end;

constructor TsortiePWM.create;
begin
//     inherited create('Vsa','V',trTensionSA,voieSA);
     PWMi := '0';
     Ffrequence := 1000;
     Fduty100 := 50;
     setPeriode;
end;

Procedure TsortiePWM.setFrequence(Avalue : integer);
begin
    if (Avalue>50) and (Avalue<50000) then begin
       Ffrequence := Avalue;
       setPeriode;
    end;
end;

Procedure TsortiePWM.setDuty(Avalue : integer);
begin
    if (Avalue>0) and (Avalue<100) then begin
       Fduty100 := Avalue;
       setPeriode;
    end;
end;

Function TsortiePWM.GetNomVoieOrphy : shortString;
begin
     result := 'PWM'+PWMi
end;

Procedure TsortiePWM.Load(Avoie : shortString;fichier : tMemIniFile);
begin
   with fichier do begin
        Ffrequence := ReadInteger(Avoie,'Frequence',1000);
        Fduty100 := ReadInteger(Avoie,'Duty',50);
        PWMi := ReadString(Avoie,'Code','0');
   end;
end;

Procedure TsortiePWM.Store(Avoie : shortString;fichier : tMemIniFile);
begin
   with fichier do begin
        WriteInteger(Avoie,'Frequence',Ffrequence);
        WriteInteger(Avoie,'Duty',Fduty100);
        WriteString(Avoie,'Code',PWMi);
   end
end;

Function TsortiePWM.CodePWM : shortString;
begin
   setPeriode;
   if not FperiodeOK
      then result := 'WPWM'
      else result := 'WPWM '+PWMi+
       ' '+intTostr(FcodeDuty)+
       ' '+intTostr(Fperiode)+
       ' '+intTostr(Fhorloge)
end;

Procedure TsortiePWM.setPeriode;
var i : integer;
begin
   Fperiode := HorlogeMaxPWM div Ffrequence;
   Fhorloge := 0;
   while (Fperiode>65535) and (Fhorloge<4) do begin
         inc(Fhorloge);
         Fperiode := Fperiode div 2;
   end;
   Fduty100 := (Fduty100*Fperiode) div 100;
   FperiodeOK := Fperiode<65535;
   Ffrequence := HorlogeMaxPWM div Fperiode;
   for i := 0 to pred(Fhorloge) do
       Ffrequence := Ffrequence div 2;
   Fduty100 := FcodeDuty*100 div Fperiode;
end;

constructor Tmagnum.create;
begin
     modeAngulaire := false;
     modeRadian := false;
     EFi := 0;
     EBi := 5;
     increment := traitMagnum;
     isCodeurInc := false;
     OscilMin := 8;
     DureeOscillation := 2;
     TraitMagnumE := 0.0018;
end;

Function Tmagnum.ValeurReal(V : integer) : double;
begin
    if IsCodeurInc
       then Result := (V-zeroMagnum)*increment
       else Result := V*traitMagnumE
end;

Function Tmagnum.codeMagnum : shortString;
begin
    if chute
       then result := 'WDEF '+intToStr(EFi)
       else result := 'WCODINC '+intToStr(EFi)+' '+intToStr(EBi);
end;

Function Tmagnum.stopMagnum : shortString;
begin
     if isCodeurInc
         then if CodeurActif
             then begin
                result := 'WPOSITION'; { dsactive }
                codeurActif := false;
             end
             else
         else if chute
             then result := 'WSDEF'
             else result := 'WSCODINC'
end;

Function Tmagnum.codeCodeurInc : shortString;
begin
     result := 'WPOSITION '+intToStr(EFi)+' '+intToStr(EBi);
     codeurActif := true
end;

Procedure Tmagnum.Load(Avoie : shortString;fichier : tMemIniFile);
begin
   with fichier do begin
         modeAngulaire := ReadBool(Avoie,'angle',false);
         modeRadian := ReadBool(Avoie,'radian',false);
         isCodeurInc := ReadBool(Avoie,'codinc',false);
         increment := ReadFloat(Avoie,'increment',TraitMagnum);
         traitMagnumE := ReadFloat(Avoie,'taille',TraitMagnum);
         OscilMin := ReadInteger(Avoie,'OscilMin',oscilMin);
         Chute := ReadBool(Avoie,'Chute',false);
         Negatif := ReadBool(Avoie,'Negatif',true);
         DureeOscillation := ReadFloat(Avoie,'DureeOscil',dureeOscillation);
         EFi := ReadInteger(Avoie,'Front',0);
         EBi := ReadInteger(Avoie,'EBi',5);
   end;
end;

Procedure Tmagnum.Store(Avoie : shortString;fichier : tMemIniFile);
begin
   with fichier do begin
         WriteBool(Avoie,'radian',FModeRadian);
         WriteBool(Avoie,'angle',FModeAngulaire);
         WriteBool(Avoie,'Chute',FChute);
         WriteBool(Avoie,'Negatif',Negatif);
         WriteBool(Avoie,'codinc',IsCodeurInc);
         WriteFloat(Avoie,'Taille',traitMagnumE);
         WriteFloat(Avoie,'Increment',increment);
         WriteInteger(Avoie,'OscilMin',oscilMin);
         WriteFloat(Avoie,'DureeOscil',DureeOscillation);
         writeInteger(Avoie,'Front',EFi);
         writeInteger(Avoie,'EBi',EBi);
   end
end;

procedure Tmagnum.setModeAngulaire(value : boolean);
begin
     if FmodeAngulaire<>value then begin
        FmodeAngulaire := value;
        if not value then TraitMagnumE := traitMagnum;
     end;
end;

function Tmagnum.getNomUnite : shortString;
begin
    if FmodeAngulaire
        then if modeRadian
               then result := 'rad'
               else result := ''
        else result := 'm';
end;

procedure Tmagnum.setChute(value : boolean);
begin
     if Fchute<>value then begin
        Fchute := value;
        if value
           then FmodeAngulaire := false
           else if not FmodeAngulaire
              then TraitMagnumE := traitMagnum;
     end;
end;

var WindirAzt : chaineAzt;
Initialization
     GetWindowsDirectory(winDirAzt,sizeOf(winDirAzt)-1);
     SortiePWM := TsortiePWM.create;
     Magnum := Tmagnum.create;

{
CSIDL_BITBUCKET	Recycle bin  file system directory containing file objects in the user's recycle bin. The location of this directory is not in the registry; it is marked with the hidden and system attributes to prevent the user from moving or deleting it.
CSIDL_CONTROLS	Control Panel  virtual folder containing icons for the control panel applications.
CSIDL_DESKTOP	Windows desktop  virtual folder at the root of the name space.
CSIDL_DESKTOPDIRECTORY	File system directory used to physically store file objects on the desktop (not to be confused with the desktop folder itself).
CSIDL_DRIVES	My Computer  virtual folder containing everything on the local computer: storage devices, printers, and Control Panel. The folder may also contain mapped network drives.
CSIDL_FONTS	Virtual folder containing fonts.
CSIDL_NETHOOD	File system directory containing objects that appear in the network neighborhood.
CSIDL_NETWORK	Network Neighborhood  virtual folder representing the top level of the network hierarchy.
CSIDL_PERSONAL	File system directory that serves as a common respository for documents.
CSIDL_PRINTERS	Printers folder  virtual folder containing installed printers.
CSIDL_PROGRAMS	File system directory that contains the user's program groups (which are also file system directories).
CSIDL_RECENT	File system directory that contains the user's most recently used documents.
CSIDL_SENDTO	File system directory that contains Send To menu items.
CSIDL_STARTMENU	File system directory containing Start menu items.
CSIDL_STARTUP	File system directory that corresponds to the user's Startup program group.
CSIDL_TEMPLATES	File system directory that serves as a common repository for document templates.
}

finalization
     sortiePWM.Free;
     Magnum.Free;
end.

