Unit HiResTmr;

Interface

uses  WinTypes, WinProcs;

{ Timer definitions }

{ Definitions relating to the performance counter in use }
  const
  { The max value before rollover. Default is -1 but
    MSDOS timer counter rolls over each midnight and will
    reprogram this value }
  r_CounterMaxValue : Int64 = -1;

  { The resolution which defaults in DOS and Win3 to about 0.8us,
    Delphi 2 32-bit will load this value too, in case
    it changes in the future }
  r_CountsPerSec    : Int64 = 1193180;
  r_CountsPerMilliSec    : Int64 = 1193;

  { The number of counter counts taken to call the performance
    counter }
  r_CounterCallOverhead : Int64 = 0;

  procedure DelayS( const AValue : extended );
  { Delays this value using the performance counter }

  function  GetElapsedSec(const Debut, Fin : int64) : extended;

  function  GetElapsed(const AValue, AStart : int64) : int64;

Implementation

  function  GetElapsed(const AValue, AStart : int64) : int64;
  { Returns the duration between AValue and AStart including any rollover }
  begin
    If AValue < AStart
       then GetElapsed := AValue + (r_CounterMaxValue - AStart)
      { Has rolled over }
       else GetElapsed := AValue - AStart
  end;

  function GetElapsedSec(const Debut, Fin : int64) : extended;
  begin
       Result := GetElapsed(Fin,Debut)/r_CountsPerSec
  end;

  procedure WaitForElapsed( const AValue : Int64);
  { Hangs waiting for counter to reach this value }
  var
   R, r_Start : Int64;
  begin
   QueryPerformanceCounter( r_Start );
   Repeat
     QueryPerformanceCounter( R )
   until GetElapsed( R, r_Start ) >= AValue;
  end;

  procedure CalibratePerformanceCounterOverhead;
  { Calls the performance counter to determine the time overhead }
  var
   I : word;
   r_Start, r_End : Int64;
  begin
   QueryPerformanceCounter( r_Start );
   For I := 1 to 1000 do QueryPerformanceCounter( r_End );
   { Get the difference }
   r_CounterCallOverhead := GetElapsed( r_End, r_Start);
   r_CounterCallOverhead := r_CounterCallOverhead div 1000;
  end;

  procedure DelayS( const AValue : extended );
  { Delays this value using the performance counter }
  var
    r_Elapsed : Int64;
    z : extended;
  begin
    z := (AValue * r_CountsPerSec)
       - (r_CounterCallOverhead * 2);
    r_Elapsed := round(z);
    If r_Elapsed < 0 then  r_Elapsed := 0;
    WaitForElapsed( r_Elapsed );
  end;

Initialization
     QueryPerformanceFrequency(r_CountsPerSec);
     CalibratePerformanceCounterOverhead;
end.
