unit uniteKer;

interface

uses sysUtils, math, classes;


const
        LongUnite = 32;

        UnJour : double = 24*3600.0;
        UneHeure : double = 3600.0;
        QuartdHeure : double = 15*60.0;
        UneMinute : double = 60.0;

        alpha = #128;
        beta = #129;
        chi = #130;
        delta = #131;
        deltaMaj = #150;
        epsilon = #132;
        phi = #133; phiMaj = #151;
        gamma = #134; gammaMaj = #152;
        eta = #135;
        lambda = #136; lambdaMaj = #153;
        nu = #138;
        piMin = #139;piMaj = #154;
        theta = #140;thetaMaj = #155;
        rho = #141;
        sigma = #142;sigmaMaj = #156;
        tau = #143;
        omega = #144;omegaMaj = #157;
        xi = #147;xiMaj = #158;
        psi = #148;psiMaj = #159;
        zeta = #149;
        mu = #181;
        vide = #248;videMaj = #216;


Type
  TcharSet = set of char;
  strUnite = string[LongUnite];
  CoefUnite = (zero,yocto,zepto,atto,femto,pico,nano,micro,milli,
               nulle,kilo,mega,giga,tera,peta,exa,zetta,yotta,infini);
	TuniteDeBase = (_metre,_kilogramme,_seconde,_ampere,_kelvin,_mole,_candela);

  EUniteError = class(Exception);
  TuniteSansDim = (tuVide,tuDegre,tuRadian,tuDecibel,tuPourCent);

	Tunite = class
                 private
                     FnomUnite : strUnite;
                     Correct,Adaptable,UniteDonnee,Indifferent : boolean;
                     SansDim : TuniteSansDim;
                     Dimension : array[TuniteDeBase] of shortInt;
                     Puissance : integer;
                     exposantU : double;
                     nbreAvantVirgule : integer;
                     Procedure SetNomUnite(const nu : strUnite);
                     Procedure MulUnite(op : char;U0 : Tunite;puiss : shortInt);
                     Procedure PlusUnite(U0 : Tunite);
                     Procedure InverseUnite;
                     Procedure setErreurUnite;
                 public
                     nom : strUnite;
                 Property nomUnite : strUnite read FnomUnite write SetNomUnite;
                 Constructor Create;
                 Procedure setUniteComplet(const nu : strUnite;p : shortInt;d1,d2,d3,d4,d5,d6,d7 : shortInt);
                 procedure Assign(U : Tunite);
                 Procedure Init;
                 Procedure RecopieUnite(U : Tunite);
                 Function  NomAff(puissValeur : shortInt) : strUnite;
                 Function  NomAxe(exposant : double) : strUnite;
                 function formatUnite(nombre : double) : strUnite;
                 function formatSimple(nombre : double) : strUnite;                 
                 function formatUniteDouble(mini,maxi : double) : strUnite;                 
                 Function FormatUniteExp(Nombre,Exposant : double;
                      NbreAvantVirgule : integer) : strUnite;
                 function NomValeurUnite(nombre : double) : strUnite;virtual;
                 procedure setExposant(maxi : double;expo : integer);
                 Function FormatUniteDB15(Nombre : double;
                      avecUnite : boolean) : strUnite;
	end;

Const
    NbreUniteConnue = 32;
    NbreUniteToleree = 10;
    IndexRadian = 2;
    IndexCelsius = 3;
    caracUnite : TcharSet =
           ['A'..'Z','a'..'z',OmegaMaj,'1'..'3','-','.','/',''];

	  caracPrefixe : TcharSet =
	   ['a','f','p','n','','m','d','c','h','k','M','G','T','P','E'];
   	NomBase : array[TuniteDeBase] of string[3] =
	('m','kg','s','A','K','mol','cd');
    NomSansDim : array[TuniteSansDim] of string[3] =
        ('','','rad','dB','%');
    prefixeUnite : array[coefunite] of char =
          ('0','y','z','a','f','p','n','','m',
            ' ','k','M','G','T','P','E','Z','Y','');
    valUnite : array[coefunite] of double =
	       (1E-27,1E-24,1E-21,1E-18,1E-15,1E-12,1E-9,1E-6,1E-3,
                1,1E+3,1E+6,1E+9,1E+12,1E+15,1E+18,1E+21,1E+24,1E+27);
    puissanceUnite : array[coefunite] of shortInt =
	       (-27,-24,-21,-18,-15,-12,-9,-6,-3,
                0,+3,+6,+9,+12,+15,+18,+21,+24,+27);
    expUnite : array[coefUnite] of string[4] =
	       ('E-27','E-24','E-21','E-18','E-15','E-12','E-9','E-6','E-3',
                '','E+3','E+6','E+9','E+12','E+15','E+18','E+21','E+24','E+27');

    Chiffre     : TcharSet = ['0'..'9'];
    Majuscule   : TcharSet = ['A'..'Z'];
    Minuscule   : TcharSet = ['a'..'z'];
    Grecque  : TcharSet = [alpha,beta,gamma,gammaMaj,delta,deltaMaj,chi,
                           epsilon,phi,phiMaj,eta,psi,psiMaj,lambda,lambdaMaj,mu,nu,
                           piMin,piMaj,theta,thetaMaj,rho,sigma,sigmaMaj,tau,omega,omegaMaj,
                           xi,xiMaj,zeta];

var UniteConnue : array[1..NbreUniteConnue] of Tunite;
    UniteToleree : array[1..NbreUniteToleree] of Tunite;
    UniteNulle,UTemps : Tunite;

Function ChaineDuree(t : double) : strUnite;
function duree125(duree : double) : double;
Function CaracToChiffre(carac : char) : integer;
Function PuissToStr(puiss : integer) : strUnite;

implementation

Procedure Tunite.InverseUnite;
var i : TuniteDeBase;
begin
  if not correct then exit;
  Puissance := -puissance;
  for i:=_Metre to _Candela do
	dimension[i] := -dimension[i];
  sansDim := tuVide;
end; {InverseUnite}

Procedure Tunite.PlusUnite(U0 : Tunite);
var i : TuniteDeBase;
begin
  correct := correct and U0.correct;
  if not correct then exit;
  Puissance := puissance+U0.puissance;
  for i:=_Metre to _Candela do
	dimension[i] := dimension[i]+U0.dimension[i];
  if U0.sansDim<>tuVide then if sansDim=tuVide
     then sansDim := U0.sansDim
     else sansDim := tuVide;
  Adaptable := Adaptable or U0.Adaptable;
  Indifferent := Indifferent and U0.Indifferent;
end; {PlusUnite}

Procedure Tunite.MulUnite(op : char;U0 : Tunite;puiss : shortInt);
var i : TuniteDeBase;
begin
   correct := correct and U0.correct;
   if not(correct) then exit;
   if puiss<>1 then sansDim := tuVide;
   case op of
	'*' : begin
	   Puissance := U0.Puissance*puiss;
	   for i:=_Metre to _Candela do
		   dimension[i] := U0.dimension[i]*puiss;
	end;
	'/' : begin
		if (U0.puissance mod puiss <> 0) then setErreurUnite;
		Puissance := U0.Puissance div puiss;
		for i:=_Metre to _Candela do begin
		   if (U0.dimension[i] mod puiss <> 0) then setErreurUnite;
		   dimension[i] := U0.dimension[i] div puiss;
		end;
        end;
   end; { case }
   adaptable := adaptable or U0.adaptable;
   indifferent := indifferent and U0.indifferent;
end;{MulUnite}

Procedure Tunite.SetNomUnite(const nu : strUnite);
{ Pour les units dont le nomUnite est impos par l'utilisateur }
var Ucourant : Tunite;
    Inverse : boolean;
    LongUnite : byte;

Procedure AnalysePrefixe(var index : byte);
begin
   if FnomUnite[index]='/' then begin
	  inc(index);
	  inverse := true;
   end;
   if (index<LongUnite) and
      (FnomUnite[index] in caracPrefixe) and
      (FnomUnite[succ(index)] in majuscule+minuscule+[''])
	then begin
		case FnomUnite[index] of
		  'a' : Ucourant.puissance := -18;
		  'f' : Ucourant.puissance := -15;
		  'p' : Ucourant.puissance := -12;
		  'n' : Ucourant.puissance := -9;
		  '' : Ucourant.puissance := -6;
		  'm' : Ucourant.puissance := -3;
		  'c' : Ucourant.puissance := -2;
                  'd' : Ucourant.puissance := -1;
		  'h' : Ucourant.puissance := +2;
		  'k' : Ucourant.puissance := +3;
		  'M' : Ucourant.puissance := +6;
		  'G' : Ucourant.puissance := +9;
		  'T' : Ucourant.puissance := +12;
		  'P' : Ucourant.puissance := +15;
		  'E' : Ucourant.puissance := +18;
	end;
	inc(index);
 end;
end;{AnalysePrefixe}

Procedure AnalyseCode(var index : byte);
var trouve : boolean;
    j : integer;
    nomCourant : strUnite;
    indexInit : byte;

Procedure OubliSeparateur;
var j : integer;
    UU : Tunite;
begin
    index := indexInit;
    nomCourant := '';
    repeat
    	nomCourant := nomCourant+FnomUnite[index];
        inc(index);
    	for j := 1 to NbreUniteConnue do begin
    		UU := UniteConnue[j];
    		trouve := nomCourant=UU.nomUnite;
    		if trouve then begin
    		   Ucourant.PlusUnite(UU);
    		   exit
    		end;
    	end;
    	for j := 1 to NbreUniteToleree do begin
    		UU := UniteToleree[j];
    		trouve := nomCourant=UU.nomUnite;
              	if trouve then begin
		   Ucourant.PlusUnite(UU);
		   exit
		end;
	end;
  until (index>LongUnite) or not (FnomUnite[index] in Majuscule+Minuscule);
end; { OubliSeparateur }

var UU : Tunite;
begin { AnalyseCode }
	nomCourant  := '';
	indexInit := index;
	repeat
        nomCourant := nomCourant+FnomUnite[index];
	      inc(index);
	until (index>longUnite) or
	  not (FnomUnite[index] in Majuscule+Minuscule);
	for j := 1 to NbreUniteConnue do begin
	     UU := UniteConnue[j];
	     trouve := nomCourant=UU.nomUnite;
	     if trouve then begin
	     	   Ucourant.PlusUnite(UU);
	     	   exit
	     end;
	 end;
	 for j := 1 to NbreUniteToleree do begin
	     UU := UniteToleree[j];
	     trouve := nomCourant=UU.nomUnite;
	     if trouve then begin
		   Ucourant.PlusUnite(UU);
		   exit;
	     end;
	 end;
	 oubliSeparateur;
	 if not Trouve then setErreurUnite;
end; { AnalyseCode }

Procedure AnalyseSuffixe(var index : byte);
var negatif,kilogramme : boolean;
    z : shortInt;
    i : TuniteDeBase;
begin
     if index>longUnite then begin
        if inverse then Ucourant.inverseUnite;
        exit;
     end;
     if FnomUnite[index]='^' then begin
	   inc(index);
	   if (index>LongUnite) then setErreurUnite;
     end;
     negatif := false;
     if FnomUnite[index] in ['-','+'] then begin
        negatif := FnomUnite[index]='-';
        inc(index);
        if (index>LongUnite) then setErreurUnite;
     end;
     kilogramme := true;
     for i:=_Metre to _Candela do if i=_kilogramme
     	then kilogramme := kilogramme and (Ucourant.dimension[_kilogramme]<>0)
     	else kilogramme := kilogramme and (Ucourant.dimension[i]=0);
{ cas  faire }
     if FnomUnite[index] in chiffre
	      then begin
           	z := caracToChiffre(FnomUnite[index]);
		        if negatif then z := -z;
      		  inc(index);
	      end
        else z := 1;
   if inverse then z := -z;
   Ucourant.MulUnite('*',Ucourant,z);
end; { AnalyseSuffixe }

var i : TuniteDeBase;
    index,j : byte;
    stValeur : strUnite;
begin { SetNomUnite }
    try { Erreur Unite }
    FnomUnite := nu;
    sansDim := tuVide;
    puissance := 0;
    correct := true;
    for i:=_Metre to _Candela do dimension[i] := 0;
    longUnite := Length(FnomUnite);
    UniteDonnee := longUnite>0;
    if not UniteDonnee then exit;
    for j := 1 to NbreUniteConnue do begin
       	Ucourant := UniteConnue[j];
       	if FnomUnite=Ucourant.nomUnite then begin
	         RecopieUnite(Ucourant);
    	      exit
       	end;
    end;
    for j := 1 to NbreUniteToleree do begin
	Ucourant := UniteToleree[j];
	if FnomUnite=Ucourant.nomUnite then begin
	   RecopieUnite(Ucourant);
	   exit;
	end;
    end;
    index := 1;
    inverse := false;
    if Pos('1E',FNomUnite)=1 then begin
	  index := 3;
          while (index<LongUnite) and
    		(FnomUnite[index] in Chiffre+['-','+']) do
    	               inc(index);
	  if index>3 then begin
		 stValeur := copy(FnomUnite,1,pred(index));
		 puissance := strToInt(StValeur);
	  end;
	  if FnomUnite[index]=' ' then inc(index);
   end; { 1E+xx }
   Ucourant := Tunite.create;
   while (index<=LongUnite) do begin
	   Ucourant.init;
	   AnalysePrefixe(index);
	   AnalyseCode(index);
	   AnalyseSuffixe(index);
	   PlusUnite(Ucourant);
	   if (index<LongUnite) and (FnomUnite[index]='.') then
	  	inc(index); { sparateur }
   end;
   Ucourant.free;
   except
         on EUniteError do begin
            correct := false;
         end;
   end;
end; { SetNomUnite }

Function AjoutePuissDix(nom : strUnite;puiss : shortInt) : strUnite;
var dest : strUnite;
begin
   if puiss=0
      then ajoutePuissDix := nom
      else begin
         dest := '10^('+puissToStr(puiss)+')';
         if length(dest)+Length(nom) > LongUnite
            then ajoutePuissDix := dest+'S.I.'
            else ajoutePuissDix := dest+nom
      end;
end;

Function Tunite.NomAff(puissValeur : shortInt) : strUnite;
{ unit dont on connait le dveloppement en puissance }

Function PrefixeConnu(p : shortInt) : boolean;
begin
 PrefixeConnu := (p>=-18) and
    (p<=18) and ( (p mod 3=0) or
    (p=-2) or (p=-1) or (p=2))
end;

Function AjoutePrefixe(Nom : strUnite;P : shortInt) : strUnite;
const
    lettreUnite : TcharSet = ['A','C','F','H','J','K','L','N','P','S','T','V','W',
                              'b','c','d','g','l','m','r','s',OmegaMaj];
var prefixe : char;
begin
     if nom='' then begin
       	ajoutePrefixe := AjoutePuissDix('',P);
        exit;
     end;
     if (length(nom)>1) and
        (nom[1] in caracPrefixe) and
        (nom[2] in lettreUnite) then begin
     	 case FnomUnite[1] of
		  'a' : P := P-18;
		  'f' : P := P-15;
		  'p' : P := P-12;
		  'n' : P := P-9;
		  '' : P := P-6;
		  'm' : P := P-3;
		  'c' : P := P-2;
                  'd' : P := P-1;
		  'h' : P := P+2;
		  'k' : P := P+3;
		  'M' : P := P+6;
		  'G' : P := P+9;
		  'T' : P := P+12;
		  'P' : P := P+15;
		  'E' : P := P+18;
         end;
         nom := copy(nom,2,length(nom));
     end;
     case P of
	  -18 : prefixe := 'a';
	  -15 : prefixe := 'f';
	  -12 : prefixe := 'p';
	  -9 : prefixe := 'n';
	  -6 : prefixe := '';
	  -3 : prefixe := 'm';
	  -2 : prefixe := 'c';
	  -1 : prefixe := 'd';
	  0  : begin
             ajoutePrefixe := nom;
             exit;
          end;
	  +2 : prefixe := 'h';
	  +3 : prefixe := 'k';
	  +6 : prefixe := 'M';
	  +9 : prefixe := 'G';
	  +12 : prefixe := 'T';
	  +15 : prefixe := 'P';
	  +18 : prefixe := 'E';
	  else begin
	        ajoutePrefixe := AjoutePuissDix(nom,P);
	        exit;
	  end;
	end;
	ajoutePrefixe := prefixe+nom;
end;

var i : TuniteDeBase;
    OKnulle,OKconnue : boolean;
    j : integer;
    UU : Tunite;
    NomPositif,NomNegatif : strUnite;
    CasDuMetre : boolean;
begin
   if UniteDonnee and (puissValeur=0) then begin
      NomAff := FnomUnite;
      exit;
   end;
   if not correct then begin
      if not UniteDonnee then FnomUnite := '?';
      NomAff := AjoutePuissDix(FnomUnite,puissValeur);
      exit;
   end;
   OKnulle := true;
   for i :=_Metre to _Candela do
       OKnulle := OKnulle and (dimension[i]=0);
   if OKnulle then begin
        if not UniteDonnee then FnomUnite := nomSansDim[sansDim];
        NomAff := AjoutePrefixe(FnomUnite,PuissValeur);
        exit;
   end;
   puissValeur := puissance+puissValeur;
   CasDuMetre := Correct and (dimension[_metre]<>0) and (dimension[_metre]<>1);
   for i:=_KiloGramme to _Candela do CasDuMetre := CasDuMetre and (dimension[i]=0);
   if CasDuMetre and  { 10^-6 m^2 = (mm)^2}
      (puissValeur mod dimension[_metre]=0) and
      prefixeConnu(puissValeur div dimension[_metre])then begin
	            NomPositif := AjoutePrefixe('m',puissValeur div dimension[_metre]);
                             { Pb des units donnes en mL ? }
              NomPositif := NomPositif+PuissToStr(dimension[_metre]);
              NomAff := NomPositif;
	      exit;
   end;
   for j := 1 to NbreUniteToleree do
       if FnomUnite=UniteToleree[j].nomUnite then begin
           Result := AjoutePrefixe(FnomUnite,puissValeur-puissance);
           exit;
       end;
   for j:=1 to NbreUniteConnue do begin
	      UU := UniteConnue[j];
        for i :=_Metre to _Candela do begin
            OKconnue := dimension[i]=UU.dimension[i];
            if not OKconnue then break;
        end;
	      if OKconnue and (sansDim=tuVide) then begin
            Result := AjoutePrefixe(UU.nomUnite,puissValeur-UU.puissance);
	          exit;
  	    end;
   end;
   for j:=1 to NbreUniteConnue do begin
	    UU := UniteConnue[j];
            for i :=_Metre to _Candela do begin
                if i=_Seconde
                     then OKconnue := dimension[_seconde]=UU.dimension[_seconde]-1
                     else OKconnue := dimension[i]=UU.dimension[i];
                if not OKconnue then break;
            end;
	    if OKconnue and (sansDim=tuVide) and (UU.nomUnite<>'Hz') then begin (* = ou <> ? *)
               Result := AjoutePrefixe(UU.nomUnite,puissValeur-UU.puissance);
               Result := Result+'/s';
	       exit;
	    end;
   end;
   if UniteDonnee then begin
      NomAff := AjoutePuissDix(FnomUnite,puissValeur);
      exit;
   end;
   NomPositif := '';
   NomNegatif := '';
   for i:=_Metre to _Candela do
       if dimension[i]>0
           then begin
                if NomPositif<>'' then NomPositif := NomPositif + '.';
                NomPositif := NomPositif + NomBase[i];
                if dimension[i]>1 then
                   NomPositif := NomPositif + PuissToStr(dimension[i]);
           end
           else if dimension[i]<0 then begin
                if NomNegatif<>'' then NomNegatif := NomNegatif+'.';
                NomNegatif := NomNegatif+NomBase[i]+PuissToStr(dimension[i]);
           end;
   if (NomPositif<>'') and (NomNegatif<>'') then
        NomPositif := NomPositif+'.';
   FNomUnite := NomPositif + NomNegatif;
   NomAff := AjoutePuissDix(FNomUnite,puissValeur);
end; { NomAff }

Function Tunite.NomAxe(exposant : double) : strUnite;
begin
    result := nomAff(round(log10(exposant)));
    if result<>''
       then result := nom+' ('+result+')'
       {if pos('/',result)>0
          then result := nom+'('+result+')'
          else result := nom+'/'+result
          }
       else nomAxe := nom;
end;

constructor Tunite.create;
begin
   Init
end;

Procedure InitUnite;
begin
          	UniteConnue[1] := Tunite.create;
	UniteConnue[1].setUniteComplet('m',0,1,0,0,0,0,0,0);
        	UniteConnue[2] := Tunite.create;
	UniteConnue[2].setUniteComplet('A',0,0,0,0,1,0,0,0);
        	UniteConnue[3] := Tunite.create;
	UniteConnue[3].setUniteComplet('g',-3,0,1,0,0,0,0,0);
        	UniteConnue[4] := Tunite.create;
	UniteConnue[4].setUniteComplet('K',0,0,0,0,0,1,0,0);
        	UniteConnue[5] := Tunite.create;
	UniteConnue[5].setUniteComplet('mol', 0,0,0,0,0,0,1,0);
        	UniteConnue[6] := Tunite.create;
	UniteConnue[6].setUniteComplet('cd', 0,0,0,0,0,0,0,1);
        	UniteConnue[7] := Tunite.create;
	UniteConnue[7].setUniteComplet('H', 0,2,1,-2,-2,0,0,0);
        	UniteConnue[8] := Tunite.create;
	UniteConnue[8].setUniteComplet('mol/l', +3,-3,0,0,0,0,1,0);
        	UniteConnue[9] := Tunite.create;
	UniteConnue[9].setUniteComplet('F', 0,-2,-1,4,2,0,0,0);
        	UniteConnue[10] := Tunite.create;
	UniteConnue[10].setUniteComplet('N', 0,1,1,-2,0,0,0,0);
        	UniteConnue[11] := Tunite.create;
	UniteConnue[11].setUniteComplet(omegaMaj, 0,2,1,-3,-2,0,0,0);
        	UniteConnue[12] := Tunite.create;
	UniteConnue[12].setUniteComplet('S', 0,-2,-1,3,2,0,0,0);
        	UniteConnue[13] := Tunite.create;
	UniteConnue[13].setUniteComplet('T', 0,0,1,-2,-1,0,0,0);
        	UniteConnue[14] := Tunite.create;
	UniteConnue[14].setUniteComplet('V', 0,2,1,-3,-1,0,0,0);
        	UniteConnue[15] := Tunite.create;
	UniteConnue[15].setUniteComplet('Hz', 0,0,0,-1,0,0,0,0);
        	UniteConnue[16] := Tunite.create;
	UniteConnue[16].setUniteComplet('s', 0,0,0,1,0,0,0,0);
                UTemps := UniteConnue[16];
        	UniteConnue[17] := Tunite.create;
	UniteConnue[17].setUniteComplet('Pa', 0,-1,1,-2,0,0,0,0);
        	UniteConnue[18] := Tunite.create;
	UniteConnue[18].setUniteComplet('J', 0,2,1,-2,0,0,0,0);
        	UniteConnue[19] := Tunite.create;
	UniteConnue[19].setUniteComplet('W', 0,2,1,-3,0,0,0,0);
        	UniteConnue[20] := Tunite.create;
	UniteConnue[20].setUniteComplet('C', 0,0,0,1,1,0,0,0);
        	UniteConnue[21] := Tunite.create;
	UniteConnue[21].setUniteComplet('Wb', 0,2,1,-2,-1,0,0,0);
        	UniteConnue[22] := Tunite.create;
	UniteConnue[22].setUniteComplet('lm', 0,0,-2,0,0,0,0,1);
        	UniteConnue[23] := Tunite.create;
	UniteConnue[23].setUniteComplet('N/m', 0,0,1,-2,0,0,0,0);
        	UniteConnue[24] := Tunite.create;
	UniteConnue[24].setUniteComplet('J/K', 0,2,1,-2,0,-1,0,0);
        	UniteConnue[25] := Tunite.create;
	UniteConnue[25].setUniteComplet('J/(kg.K)', 0,2,0,-2,0,-1,0,0);
        	UniteConnue[26] := Tunite.create;
	UniteConnue[26].setUniteComplet('V/m', 0,1,1,-3,-1,0,0,0);
        	UniteConnue[27] := Tunite.create;
	UniteConnue[27].setUniteComplet('S/m', 0,-3,-1,3,2,0,0,0);
        	UniteConnue[28] := Tunite.create;
	UniteConnue[28].setUniteComplet('W/(m.K))', 0,1,1,-3,0,-1,0,0);
        	UniteConnue[29] := Tunite.create;
	UniteConnue[29].setUniteComplet('C.m', 0,1,0,0,0,0,0,0);
        	UniteConnue[30] := Tunite.create;
	UniteConnue[30].setUniteComplet(OmegaMaj+'m', 0,3,1,-3,-2,0,0,0);
        	UniteConnue[31] := Tunite.create;
	UniteConnue[31].setUniteComplet('N.m', 0,1,0,1,1,0,0,0);
        	UniteConnue[32] := Tunite.create;
	UniteConnue[32].setUniteComplet('lx', 0,-2,0,0,0,0,0,1);


        	UniteToleree[1] := Tunite.create; { litre }
	UniteToleree[1].setUniteComplet('L',-3,3,0,0,0,0,0,0);
        	UniteToleree[2] := Tunite.create;
	UniteToleree[2].setUniteComplet('rad',0,0,0,0,0,0,0,0);
        	UniteToleree[3] := Tunite.create;
	UniteToleree[2].sansDim := tuRadian;
	UniteToleree[3].setUniteComplet('C',0,0,0,0,0,1,0,0);
        	UniteToleree[4] := Tunite.create;
	UniteToleree[4].setUniteComplet('bar',5,-1,1,-2,0,0,0,0);
          UniteToleree[5] := Tunite.create;
	UniteToleree[5].setUniteComplet('dB',0,0,0,0,0,0,0,0);
      	UniteToleree[5].sansDim := tuDecibel;
        UniteToleree[6] := Tunite.create;
  UniteToleree[6].setUniteComplet('',0,0,0,0,0,0,0,0);
        UniteToleree[6].sansDim := tuDegre;
        UniteToleree[7] := Tunite.create;
 UniteToleree[7].setUniteComplet('%',0,0,0,0,0,0,0,0);
       	UniteToleree[7].sansDim := tuPourCent;
        UniteToleree[8] := Tunite.create;
 UniteToleree[8].setUniteComplet('lux',0,-2,0,0,0,0,0,1);
         UniteToleree[9] := Tunite.create;
 UniteToleree[9].setUniteComplet('%',0,0,0,0,0,0,0,0);
       	UniteToleree[10] := Tunite.create; { mole/litre }
	UniteToleree[10].setUniteComplet('mol/L',3,-3,0,0,0,0,1,0);

 	UniteNulle := Tunite.create;
	UniteNulle.setUniteComplet('',0,0,0,0,0,0,0,0);
end;

Procedure Tunite.SetUniteComplet(const nu : strUnite;p : shortInt;d1,d2,d3,d4,d5,d6,d7 : shortInt);
begin
	 FnomUnite := nu;
   puissance := p;
	 Dimension[_metre] := d1;
	 Dimension[_kilogramme] := d2;
	 Dimension[_seconde] := d3;
	 Dimension[_ampere] := d4;
	 Dimension[_kelvin] := d5;
	 Dimension[_mole] := d6;
	 Dimension[_Candela] := d7;
end;

Procedure Tunite.setErreurUnite;
begin
   correct := false;
   raise EUniteError.Create('Erreur unit')
end;

Procedure Tunite.Init;
var i : TuniteDeBase;
begin
   FnomUnite := '';
   Puissance := 0;
   sansDim := tuVide;
   for i:=_Metre to _Candela do Dimension[i] := 0;
   Correct := true;
   Adaptable := false;
   UniteDonnee := false;
   Indifferent := false;
   nom := '';
end;

Procedure Tunite.RecopieUnite(U : Tunite);
var i : TuniteDeBase;
begin
   FnomUnite := U.nomUnite;
   Puissance := U.puissance;
   sansDim := U.sansDim;
   for i:=_Metre to _Candela do
      Dimension[i] := U.dimension[i];
   Correct := U.correct;
   Adaptable := U.adaptable;
   Indifferent := U.indifferent;
   UniteDonnee := U.UniteDonnee;
end;

Procedure Tunite.Assign(U : Tunite);
begin
   RecopieUnite(U);
   Nom := U.Nom
end;

Function Tunite.FormatUnite(Nombre : double) : strUnite;
var negatif : boolean;
    tampon : strUnite;
    unite : coefUnite;
    chifAv,chifAp : integer;
begin
  if abs(nombre)<1e-12 then result := '0 '+nomAff(0)
  else if abs(nombre)>1e+12 then result := '???'
  else if nomUnite='hPa' then result := IntToStr(round(nombre))+' hPa'
  else begin
      negatif := nombre<0;
      nombre := abs(nombre);
      unite := nulle;
      while ( nombre>=999 ) and (unite<infini) do begin
          nombre := nombre/1000;
          inc(unite)
      end; { nombre<1000 ou infini }
      while ( nombre<0.999 ) and (unite>zero) do begin
          nombre := nombre*1000;
          Dec(unite)
      end; { nombre>=1 ou zero }
      if Nombre<10
          then chifAp := 3
          else if Nombre<100
              then chifAp := 2
              else chifAp := 1;
      chifAv := 6-chifAp;
      tampon := FloatToStrF(nombre,ffFixed,chifAv,chifAp);
      if negatif then tampon := '-' + tampon;
      result := tampon+' '+nomAff(puissanceUnite[unite]);
    end
end;

Function Tunite.FormatSimple(Nombre : double) : strUnite;
var chifAv,chifAp : integer;
begin
   if Nombre<1
      then chifAp := 3
      else chifAp := 2;
   chifAv := 6-chifAp;
   result := FloatToStrF(nombre,ffFixed,chifAv,chifAp);
end;

Function Tunite.FormatUniteDouble(mini,maxi : double) : strUnite;
var unite : coefUnite;
    chifAv,chifAp : integer;

procedure ChercheFormat(nombre : double);
begin
      nombre := abs(nombre);
      unite := nulle;
      while ( nombre>=999 ) and (unite<infini) do begin
          nombre := nombre/1000;
          inc(unite)
      end; { nombre<1000 ou infini }
      while ( nombre<0.999 ) and (unite>zero) do begin
          nombre := nombre*1000;
          Dec(unite)
      end; { nombre>=1 ou zero }
      if Nombre<10
            then chifAp := 1
            else chifAp := 0;
      chifAv := 5-chifAp;
end;

begin
   if nomUnite='hPa'
      then result := IntToStr(round(mini))+'..'+IntToStr(round(maxi))+' hPa'
      else begin
      if abs(maxi)>abs(mini)
      then ChercheFormat(maxi)
      else ChercheFormat(mini);
      result := FloatToStrF(mini,ffFixed,chifAv,chifAp)+
                ' +'+
                FloatToStrF(maxi,ffFixed,chifAv,chifAp)+
                ' '+
                nomAff(puissanceUnite[unite]);
      end;
end;

Function Tunite.FormatUniteExp(Nombre,Exposant : double;
                   NbreAvantVirgule : integer) : strUnite;
var tampon : strUnite;
    unite : coefUnite;
    exponent : integer;
begin
  if nomUnite='hPa'
  then result := IntToStr(round(nombre))+' hPa'
  else begin
     Exponent := floor(Log10(Exposant)) div 3;
     unite := coefUnite(exponent+ord(nulle));
     if unite=zero
        then result := '0 '+nomAff(0)
        else begin
             tampon := FloatToStrF(nombre/exposant,ffFixed,3,5-NbreAvantVirgule);
             result := tampon+' '+nomAff(puissanceUnite[unite]);
        end;
  end
end;

Function Tunite.NomValeurUnite(Nombre : double) : strUnite;
begin
    NomValeurUnite := nom+'='+formatUnite(nombre)
end;

Function ChaineDuree(t : double) : strUnite;
var Njour : integer;
begin
   if t<5*60 { 5 minutes }
      then result := Utemps.formatUnite(t)
      else begin
         if t>2*UneHeure then t := round(t/15)*15; { > 2 h  ; arrondi 15 s }
         if t>12*UneHeure then t := round(t/60)*60; { > 12 h ; arrondi 1 min}
         Njour := trunc(t/UnJour);
         if Njour>0
            then result := intToStr(Njour)+'j '+FormatDateTime('hh"h"',frac(t/UnJour))
            else if t>=UneHeure
                then result := FormatDateTime('hh"h"mm',t/UnJour)
                else result := FormatDateTime('h:mm:ss',t/UnJour)
                { h obligatoire sinon n'importe quoi merci Bill }
      end;
end;

Procedure LibereUnite;
var i : integer;
begin
    for i := 1 to NbreUniteConnue do
       	UniteConnue[i].free;
    for i := 1 to NbreUniteToleree do
      	UniteToleree[i].free;
    UniteNulle.free;
end;

function duree125(duree : double) : double;
var coeff : double;
    exposant,mantisse : integer;
    negatif : boolean;
begin
     negatif := duree<0;
     duree := abs(duree);
     if duree<1e-6 then begin
        result := 0;
        exit;
     end;
     exposant := floor(log10(duree));
     coeff := power(10,exposant);
     mantisse := round(duree/coeff);
     if mantisse>2
        then if mantisse<6
             then mantisse := 5
             else mantisse := 10;
     coeff := mantisse*coeff;
     if negatif
        then result := -coeff
        else result := coeff
end;

Function CaracToChiffre(carac : char) : integer;
begin
      if carac in chiffre
         then result := ord(carac)-ord('0')
         else case carac of
              #186 : result := 0;
              #185 : result := 1;
              #178 : result := 2;
              #179 : result := 3;
              #165 : result := 4;
              #169 : result := 5;
              #172 : result := 6;
              #173 : result := 7;
              #174 : result := 8;
              #188 : result := 9;
              else result := 0;
         end;
end;

Function PuissToStr(puiss : integer) : strUnite;
var dest : strUnite;
begin
      if puiss=0
         then dest := ''
         else dest := IntToStr(puiss);
      PuissToStr := dest
end;

procedure Tunite.setExposant(maxi : double;expo : integer);
begin
   if nomUnite='hPa'
      then begin
         exposantU := 1;
         NbreAvantVirgule := 0;
     end
     else begin
         exposantU := power(10,3*Expo);
         NbreAvantVirgule := floor(Log10(Maxi/exposantU))+2;
     end;
end;

Function Tunite.FormatUniteDB15(Nombre : double;avecUnite : boolean) : strUnite;

Function MyFormat : strUnite;
var tampon : strUnite;
    unite : coefUnite;
    exponent : integer;
begin
  if nomUnite='hPa'
  then result := IntToStr(round(nombre))+' hPa'
  else begin
     Exponent := floor(Log10(exposantU)) div 3;
     unite := coefUnite(exponent+ord(nulle));
     if unite=zero
        then result := '0 '+nomAff(0)
        else begin
              tampon := FloatToStrF(nombre/exposantU,ffFixed,3,5-NbreAvantVirgule);
              result := tampon+' '+nomAff(puissanceUnite[unite]);
        end;
  end
end;

var posVirgule,i : integer;
begin
    if (length(nomUnite)<=3) and avecUnite
       then result := MyFormat
       else result := FloatToStrF(nombre/exposantU,ffFixed,3,5-NbreAvantVirgule);
    posVirgule := pos('.',result);
    for i := 0 to (NbreAvantVirgule-posVirgule) do
        result := ' '+result;
end;

initialization
     InitUnite
finalization
    LibereUnite
end.