unit Graphe;

interface

uses Windows, SysUtils, Graphics, Classes,
   Forms, Controls, Buttons, StdCtrls, ExtCtrls, Dialogs, inifiles,
   maths, gtsUtil, gti2comp, gti2_dynamique,
   uniteKer, gtsdef, constgts, Math, clipBrd;

const
   IntSansSignif     = -1;
   dimPoint: integer = 2;
   scaleCurseur      = 10;
   LongNombreAxe     = 4;
   NbreGraduations   = 10;
   NbreGradMin       = 5;
   Fleche: array[boolean] of char = (#193, #192);

type
   TCourbe = class(Tvoie)
   private
      a, b: double; { entier = rel*a+b }
   public
      deltaAxe, exposant, pointDebut: double;
      { deltaAxe = cart entre graduations sur les axes }
      axeDateTime: boolean;
      NbreChiffre: byte;
      courbeBottom, courbeTop: integer;
      LabelVoie:   Tlabel;
      GBvoie:      TgroupBox;
      constructor Create(num : byte);
      procedure setDelta(grandAxe: boolean; min, max: double);
      procedure chercheMinmax(Nbre: integer; var min, max: double);
      function FormatAxe(x: double): strUnite;
      function XWindow(r: double): integer;
      function XMonde(xi: integer): double;
   end;

   TbaseDeTemps = class(Tcourbe)
   private
      FDureeBalayage: double;
      procedure setDureeBalayage(Avalue: double);
   public
      Deltat:   double;
      ModeLent: boolean;
      procedure setMinMax; override;
      property DureeBalayage: double Read FdureeBalayage Write setDureeBalayage;
      constructor Create;
      procedure load(Avoie: strUnite; Fichier: tMemIniFile); override;
      procedure store(Avoie: strUnite; Fichier: tMemIniFile); override;
   end;

   TgrapheGTS = class
   private
      FNbreInter: integer;
      procedure setNbreInter(Avalue: integer);
   public
      Canvas:        Tcanvas;
      oTraceLigne:   boolean;
      oTracePoints:  boolean;
      codeErreur:    integer;
      avecAxeX, avecAxeY, avecUnAxe: boolean;
      grandAxeX, grandAxeY: boolean;
      voieOrphy:     array[indiceVoieGTS2] of Tcourbe;
      voieGTI2:     array[indiceVoieYD] of Tcourbe;      
      LimiteCourbe, LimiteFenetre: Trect;
      voieTemps : Tcourbe;
      LargCarac, HautCarac, Marge: integer;
      NbreVoie:      integer;
      NbreAff, NbreCourant: integer;
      quadrillage, AnalyseurLogique: boolean;
      monocoup:      boolean;
      loupeAuto:     boolean;
      CoeffInterp:   double; { interpolation lie  WPE }
      ReticuleTrace: boolean;
      property NbreInter: integer Read FnbreInter Write setNbreInter;
      constructor Create;
      destructor Destroy; override;
      procedure segment(x1, y1, x2, y2: integer);
      procedure Draw;
      procedure TraceCourbes;
      procedure TraceReticule(voie: integer);
      procedure SetHauteurFonte(H: integer);
      procedure CourantVersAff;
      procedure TraceMotif(xi, yi: integer; motif: Tmotif);
      procedure PenDefault;
   end;

var
   grapheGts:   TgrapheGTS;
   BaseDeTemps: TbaseDeTemps;

implementation

function Tcourbe.XMonde(xi: integer): double;
begin
   Result := (xi - B) / A;
end;

function Tcourbe.XWindow(r: double): integer;
begin
   if r = sansSignif then
      Result := IntSansSignif
   else
      Result := round(A * r + B);
end;

procedure TgrapheGTS.segment(x1, y1, x2, y2: integer);
   begin
      Canvas.moveTo(x1, y1);
      Canvas.lineTo(x2, y2);
   end;

procedure TgrapheGTS.TraceMotif(xi, yi: integer; motif: Tmotif);
var
   PolyLigne: array[1..5] of Tpoint;
begin
   case motif of
      mCroix:
      begin
         Segment(xi - dimPoint, yi, xi + dimPoint, yi);
         Segment(xi, yi - dimPoint, xi, yi + dimPoint);
      end;
      mCroixDiag:
      begin
         segment(xi - dimPoint, yi - dimPoint, xi + dimPoint, yi + dimPoint);
         segment(xi + dimPoint, yi - dimPoint, xi - dimPoint, yi + dimPoint);
      end;
      mCercle: canvas.ellipse(xi - dimPoint, yi - dimPoint, xi +
            dimPoint + 1, yi + dimPoint + 1);
      mCarre: canvas.rectangle(xi - dimPoint, yi - dimPoint, xi +
            dimPoint, yi + dimPoint);
      mLosange:
      begin
         PolyLigne[1] := Point(xi - dimPoint, yi);
         PolyLigne[2] := Point(xi, yi - dimPoint);
         PolyLigne[3] := Point(xi + dimPoint, yi);
         PolyLigne[4] := Point(xi, yi + dimPoint);
         PolyLigne[5] := Point(xi - dimPoint, yi);
         canvas.polyLine(PolyLigne);
      end;
   end;
end;

procedure TgrapheGTS.TraceReticule(voie: integer);
var
   xi, yi: integer;
begin
   with voieOrphy[voie] do
   begin
      xi := voieTemps.XWindow(voieTemps.valeurAff);
      yi := XWindow(valeurAff);
      if (xi <> IntSansSignif) and (yi <> IntSansSignif) then
      begin
         canvas.pen.color := couleur;
         segment(xi - scaleCurseur, yi,xi + scaleCurseur, yi);
         segment(xi, yi - scaleCurseur,xi, yi + scaleCurseur);
      end;
   end;
end;{ TraceReticule }

procedure TgrapheGts.SetHauteurFonte(H: integer);
begin
   if H <= 13 then
      H := 13 { MS sans serif 8 points = 13 pixels }
   else if H <= 16 then
      H := 16 { MS sans serif 10 points = 16 pixels }
   else if H <= 24 then
      H := 20 { MS sans serif 12 points = 20 pixels }
   else
      H := 2 * (H div 2);
   if (H <= 20) then
      canvas.font.Name := 'Regressi'
   else
      canvas.font.Name := 'Arial Regressi';
   canvas.font.Height := H;
   HautCarac := canvas.TextHeight('A');
   LargCarac := canvas.TextWidth('A');
   Marge := largCarac div 2;
end;

procedure TgrapheGTS.Draw;

   procedure ChercheMonde;
   var
      voie: indiceVoieGTS2;
      N, Nc, hauteurCourbe: integer;
      MaxLoc, MinLoc: double;
   begin { ChercheMonde }
      NbreVoie := 0;
      for voie := voieGTS2Min to voieGTS2Max do
         if voieOrphy[voie].active then
            Inc(NbreVoie);
      if AnalyseurLogique then
         N := NbreVoie
      else
         N := 1;
      with limiteFenetre do
      begin
         avecAxeY  := ((bottom - top) div 5 > NbreGradMin * hautCarac * N div 4) and
            (AnalyseurLogique or AvecUnAxe);
         avecAxeX  := (right - left) > NbreGradMin * 4 * largCarac;
         grandAxeY := (bottom - top) div 5 > NbreGraduations * HautCarac * N div 4;
         grandAxeX := (right - left) > NbreGraduations * 4 * largCarac;
      end;
      limiteCourbe := limiteFenetre;
      Inc(limiteCourbe.left, Marge);
      Dec(limiteCourbe.bottom, Marge);
      Inc(limiteCourbe.top, Marge);
      if avecAxeY then
         Inc(limiteCourbe.left, LongNombreAxe * largCarac);
      if avecAxeX then
         Dec(limiteCourbe.bottom, hautCarac);
      for voie := voieGTS2Min to voieGTS2Max do
         with voieOrphy[voie] do
         begin
            CourbeTop := LimiteCourbe.top;
            CourbeBottom := LimiteCourbe.bottom;
         end;
      if AnalyseurLogique then
      begin
         hauteurCourbe := ((LimiteCourbe.bottom - LimiteCourbe.top) div N) - marge;
         Nc := 0;
         for voie := voieGTS2Min to voieGts2Max do
            with voieOrphy[voie] do
               if active then
               begin
                  CourbeTop := LimiteCourbe.top + Nc * hauteurCourbe + succ(Nc) * marge;
                  CourbeBottom := CourbeTop + hauteurCourbe;
                  Inc(Nc);
               end;
      end;
      with voieTemps do
      begin
         A := (limiteCourbe.right - limiteCourbe.left) / (Maxi - Mini);
         B := limiteCourbe.left - Mini * A;
         setDelta(grandAxeX, mini, maxi);
      end;
      for voie := voieGts2min to voieGTS2Max do
         with voieOrphy[voie] do
            if active then
            begin
               MaxLoc := Maxi;
               MinLoc := Mini;
               if loupeAuto and (NbreAff > 16) then
                  chercheMinMax(NbreAff, MinLoc, MaxLoc);
               A := (CourbeTop - CourbeBottom) / (MaxLoc - MinLoc);
               B := CourbeBottom - MinLoc * A;
               setDelta(grandAxeY, mini, maxi);
            end;
   end; { ChercheMonde }

   procedure DrawAxis;
   var
      yAxeX: integer;
      xNombreMax: integer;
      PointCourant: double;

      function XcorrectNombre(x: integer): boolean;
      begin
         XcorrectNombre := (x > (limiteCourbe.left + marge)) and
            (x < xNombreMax) and (abs(x - limiteCourbe.left) > marge);
      end;

      function XcorrectGrad(x: integer): boolean;
      begin
         XcorrectGrad := (x > limiteCourbe.left) and (x < limiteCourbe.right);
      end;

      procedure graduationX(x: integer);
      begin
         if quadrillage then
            segment(x, limiteCourbe.top, x, limiteCourbe.bottom)
         else
            segment(x, yAxeX, x, yAxeX + marge);
      end;

      procedure initAxeX;
      var
         x1: integer;
      begin
         with voieTemps do
         begin
            PointCourant := PointDebut;
            x1 := XWindow(0);
            canvas.pen.color := couleur;
            if (x1 > limiteCourbe.left) and (x1 < limiteCourbe.right) then
            begin
               canvas.pen.style := psDash;
               canvas.pen.Width := 1;
               segment(x1, limiteCourbe.bottom, x1, limiteCourbe.top);
               penDefault;
            end;
            xNombreMax := limiteFenetre.right - largCarac * longNombreAxe div 2;
            with limiteFenetre do
            begin
               segment(left, yAxeX, right, yAxeX);
               { flches }
               segment(right - marge * 3, yAxeX + marge, right, yAxeX);
               segment(right - marge * 3, yAxeX - marge, right, yAxeX);
            end;
            SetTextAlign(canvas.handle, TA_center + TA_top);
         end;
      end; // initAxeX

      procedure traceAxeX;
      var
         xs: integer;
         NombreAff: strUnite;
      begin
         with voieTemps do
         begin
            if Quadrillage then
            begin
               canvas.pen.style := psDot;
               canvas.pen.Width := 1;
            end;
            repeat
               xs := Xwindow(PointCourant);
               if XcorrectGrad(xs) then
               begin
                  graduationX(xs);
                  if XcorrectNombre(xs) then
                  begin
                     if abs(PointCourant) < (Maxi - Mini) / 100 then
                        pointCourant := 0;
                     if axeDateTime then
                        NombreAff := ChaineDuree(pointCourant)
                     else
                        NombreAff := formatAxe(pointCourant / exposant);
                     canvas.TextOut(xs, yAxeX + marge, NombreAff);
                  end;
               end;
               PointCourant := PointCourant + deltaAxe;
            until (PointCourant >= Maxi);
            SetTextAlign(canvas.handle, TA_left + TA_top);
            if axeDateTime then
               NombreAff := nom
            else
               NombreAff := nomAxe(exposant);
            canvas.TextOut(limiteFenetre.left + marge, yAxeX + marge, nombreAff);
         end;
      end;{traceAxeX}

      procedure traceAxeY(m: indiceVoieGTS2);
      var
         xx, ys: integer;
         nombreAff: strUnite;
         limiteTop: integer;
      begin
         with voieOrphy[m] do
         begin
            LimiteTop := CourbeTop + marge;
            if AnalyseurLogique then
               Inc(LimiteTop, hautCarac);
            setTextAlign(canvas.handle, TA_right + TA_top);
            xx := limiteCourbe.left;
            canvas.pen.color := couleur;
            PointCourant := PointDebut;
            ys := XWindow(0);
            penDefault;
            if (ys > CourbeTop) and (ys < CourbeBottom) then
               segment(xx, ys, limiteCourbe.right, ys);
            segment(xx, CourbeBottom, xx, CourbeTop);
            { flche }
            segment(xx - marge, CourbeTop + 3 * marge, xx, CourbeTop);
            segment(xx + marge, CourbeTop + 3 * marge, xx, CourbeTop);
            canvas.font.Color := couleur;
            if Quadrillage then
            begin
               canvas.pen.style := psDot;
               canvas.pen.Width := 1;
            end;
            xx := xx - marge;
            repeat
               ys := Xwindow(PointCourant);
               {coordonne cran de marque}
               if (ys > CourbeTop) and (ys < CourbeBottom) then
               begin
                  if quadrillage then
                     segment(limiteCourbe.left, ys, limiteCourbe.right, ys)
                  else
                     segment(limiteCourbe.left, ys, limiteCourbe.left + marge, ys);
                  if (ys > (LimiteTop)) and (ys < (CourbeBottom - marge)) and
                     (abs(ys - yAxeX) > marge) then
                  begin
                     if abs(pointCourant) < (Maxi - Mini) / 1000 then
                        pointCourant := 0;
                     NombreAff := formatAxe(pointCourant / exposant);
                     Canvas.TextOut(xx, ys - marge, NombreAff);
                  end;
               end;
               pointCourant := pointCourant + deltaAxe;
            until (pointCourant >= Maxi);
            canvas.TextOut(xx, courbeTop - marge, nomAxe(exposant));
         end;
      end;{traceAxeY}

      procedure zeroX;
      var
         x1: integer;
      begin
         with voieTemps do
         begin
            x1 := XWindow(0);
            if (x1 > limiteCourbe.left) and (x1 < limiteCourbe.right) then
            begin
               canvas.pen.style := psDot;
               canvas.pen.Width := 1;
               segment(x1, limiteCourbe.bottom, x1, limiteCourbe.top);
            end;
         end;
      end;{zeroX}

      procedure zeroY(m: indiceVoieGTS2);
      var
         y1: integer;
      begin
         with voieOrphy[m] do
         begin
            y1 := Xwindow(0);
            if (y1 > CourbeTop) and (y1 < CourbeBottom) then
            begin
               penDefault;
               canvas.pen.color := couleur;
               segment(limiteCourbe.left, y1, limiteCourbe.right, y1);
            end;
         end;
      end;{zeroY}

   var
      voie: indiceVoieGTS2;
   begin // DrawAxis
      yAxeX := limiteCourbe.bottom;
      if avecAxeX then
      begin
         initAxeX;
         traceAxeX;
         zeroX;
      end;
      if AnalyseurLogique then
      begin
         for voie := voieGTS2min to voieGTS2Max do
            with voieOrphy[voie] do
               if active then
                  if avecAxeY then
                     traceAxeY(voie)
                  else
                     zeroY(voie);
      end
      else
      begin
         if avecAxeY then
         begin
            for voie := voieGts2Min to voieGTS2Max do
               if voieOrphy[voie].active then begin
                  if avecAxeY then traceAxeY(voie) else zeroY(voie);
                  break;
               end
         end;
      end;
      SetTextAlign(canvas.handle, TA_left + TA_top);
   end; // DrawAxis 

begin
   with limiteFenetre do
      setHauteurFonte((bottom - top) div 30);
   penDefault;
   canvas.pen.style := psDot;
   canvas.pen.Width := 1;
   canvas.pen.color := voieTemps.couleur;
   canvas.brush.style := bsClear;
   setBkMode(canvas.handle, transparent);
   chercheMonde;
   drawAxis;
   TraceCourbes;
   ReticuleTrace := False;
end;

procedure TgrapheGTS.TraceCourbes;

   procedure TraceUneCourbe(voie: integer);

      procedure TracePoints;

      procedure TraceMotif(xi, yi: integer; motif: Tmotif);
      var polyLigne : array[1..5] of Tpoint;
begin
   case motif of
      mCroix: begin
         Segment(xi - dimPoint, yi, xi + dimPoint, yi);
         Segment(xi, yi - dimPoint, xi, yi + dimPoint);
      end;
      mCroixDiag: begin
         segment(xi - dimPoint, yi - dimPoint, xi + dimPoint, yi + dimPoint);
         segment(xi + dimPoint, yi - dimPoint, xi - dimPoint, yi + dimPoint);
      end;
      mCercle: canvas.ellipse(xi - dimPoint, yi - dimPoint, xi +
            dimPoint + 1, yi + dimPoint + 1);
      mCarre: canvas.rectangle(xi - dimPoint, yi - dimPoint, xi +
            dimPoint, yi + dimPoint);
      mLosange: begin
         PolyLigne[1] := Point(xi - dimPoint, yi);
         PolyLigne[2] := Point(xi, yi - dimPoint);
         PolyLigne[3] := Point(xi + dimPoint, yi);
         PolyLigne[4] := Point(xi, yi + dimPoint);
         PolyLigne[5] := Point(xi - dimPoint, yi);
         canvas.polyLine(PolyLigne);
      end;
   end;
end;

      var
         i: integer;
         xi, yi: integer;
      begin
         with voieOrphy[voie] do
         begin
            canvas.brush.color := couleur;
            for i := 0 to pred(NbreAff) do
            begin
               xi := voieTemps.XWindow(voieTemps.vecteurAff[i]);
               yi := XWindow(vecteurAff[i]);
               if (xi <> IntSansSignif) and (yi <> IntSansSignif) then
                  TraceMotif(xi, yi, motif);
            end;{i}
            penDefault;
         end;
      end; // tracePoints

      procedure traceLignes;
      var
         i: integer;
         PointPrecedent: boolean;
         xi, yi: integer;
      begin
         with voieOrphy[voie] do
         begin
            PointPrecedent := False;
            for i := 0 to pred(NbreAff) do
            begin
               xi := voieTemps.XWindow(voieTemps.vecteurAff[i]);
               yi := XWindow(vecteurAff[i]);
               if (xi = IntSansSignif) or (yi = IntSansSignif) then
                  pointPrecedent := False
               else if PointPrecedent then
                  canvas.LineTo(xi, yi)
               else
               begin
                  canvas.MoveTo(xi, yi);
                  pointPrecedent := True;
               end;
            end;{for i}
         end; {i}
      end; // traceLignes

   begin
      with voieOrphy[voie] do
      begin
         canvas.pen.color := couleur;
         if oTraceLigne then
            TraceLignes;
         if oTracePoints then
            TracePoints;
      end;
   end;{ TraceUneCourbe }

var
   voie: indiceVoieGTS2;
begin
   penDefault;
   if NbreAff > 0 then
      for voie := voieGTS2Min to voieGTS2Max do
         if voieOrphy[voie].active then
            traceUneCourbe(voie);
end; // TraceCourbes

destructor TgrapheGTS.Destroy;
var
   i: indiceVoieGTS2;
   fichier: tMemIniFile;
   NomFichier: shortString;
begin
   NomFichier := changeFileExt(ParamStr(0), '.INI');
   fichier := tMemIniFile.Create(nomFichier);
   fichier.UpdateFile;
   fichier.Free;
   for i := voieGTS2Min to VoieGTS2Max do
      voieOrphy[i].Free;
   inherited Destroy;
end;

constructor TgrapheGTS.Create;

   procedure ValeurDefaut;
   begin
      oTraceLigne := True;
      oTracePoints := False;
      quadrillage := False;
      AnalyseurLogique := False;
      Monocoup := True;
      AvecUnAxe := True;
   end;

var
   voie: indiceVoieGTS2;
begin
   inherited Create;
   BaseDeTemps := TbaseDeTemps.Create;
   voieTemps := BaseDeTemps;
   for voie := voieGTS2Min to voieGTS2Max do
      voieOrphy[voie] := Tcourbe.Create(voie - 1);
   for voie := voieGTS2Min to voieGTS2Max do
      with voieOrphy[voie] do
      begin
         if couleur = clWindow then
            couleur := clActiveCaption;
         setMinMax;
      end;
   ValeurDefaut;
end;

constructor Tcourbe.Create(num: byte);
begin
   inherited Create(num);
   active := False;
   A := 1;
   B := 0;
end;

constructor TbaseDeTemps.Create;
begin
   inherited Create(0);
   Mini := 0;
   Maxi := 1;
   numOrphy := -1;
   active := true;
   A := 1;
   B := 0;
   nom := 't';
   NomUnite := 's';
end;

procedure Tcourbe.SetDelta(grandAxe: boolean; min, max: double);
{ Exposant=puissance de 10 multiple de 3 de manire  avoir entre
NbreGraduations (10) et N/2.5 (4) graduations sur l'axes.
ou NbreGradMin (5) et N/2.5 (2) graduations sur l'axes.
Delta est l'intervalle entre les graduations qui commencent  PremiereGraduation }
const
   coeff: array[0..5] of double = (0.1, 0.2, 0.5, 1, 2, 5);
var
   AbsMax, AbsMin, largeur, z: double;
   Exponent, index: integer;
   N: byte;
begin
   if Maxi < Mini then
   begin
      z := Maxi;
      Maxi := Mini;
      Mini := z;
   end;
   VerifMinMaxDouble(min, max);
   if Maxi = Mini then
      Maxi := Mini + 1;
   if Max = Min then
      Max := Min + 1;
   AbsMax := abs(max);
   AbsMin := abs(min);
   if AbsMax < AbsMin then
      AbsMax := AbsMin;
   axeDateTime := (nomUnite = 's') and (AbsMax > QuartdHeure);
   if axeDateTime then
   begin
      NbreChiffre := longNombreAxe;
      DeltaAxe := (Max - Min) / 6;
      if DeltaAxe > UnJour then
         DeltaAxe := ceil(DeltaAxe / UnJour) * UnJour
      else if DeltaAxe > UneHeure then
      begin
         N := round(DeltaAxe / UneHeure);
         case N of
            1, 2, 3, 4: ;
            5: N := 6;
            6: ;
            7: N := 8;
            8: ;
            9, 10, 11: N := 12;
            12: ;
            13, 14, 15, 16, 17: N := 12;
            18, 19, 20, 21, 22, 23, 24: N := 24;
         end;
         DeltaAxe := N * UneHeure;
      end
      else if DeltaAxe > QuartdHeure then
         DeltaAxe := ceil(DeltaAxe / QuartdHeure) * QuartdHeure
      else
         DeltaAxe := ceil(DeltaAxe / UneMinute) * UneMinute;
      PointDebut := 0;
      Exposant := 1;
      exit;
   end;
   Exponent := floor(Log10(Max - Min));
   largeur := (max - min) * dix(-exponent);
   index := 0;
   if grandAxe then
      while ((largeur / coeff[index]) >= NbreGraduations) and (index < 5) do
         Inc(index)
   else
      while ((largeur / coeff[index]) >= NbreGradMin) and (index < 5) do
         Inc(index);
   DeltaAxe := coeff[index] * dix(exponent);
   NbreChiffre := ceil(Log10(AbsMax)) - floor(Log10(DeltaAxe));
   PointDebut := ceil(min / deltaAxe) * deltaAxe;
   Index := Exponent div 3;
   if ((Exponent mod 3) < 0) then
      Dec(index);
   { la division euclidienne des informaticiens <> div des maths ! }
   Exposant := Dix(3 * Index);
   N := ceil(Log10(AbsMax / Exposant));
   if N > NbreChiffre then
      NbreChiffre := N;
   if NbreChiffre < 3 then
      NbreChiffre := 3;
   if (NbreChiffre > longNombreAxe) then
      NbreChiffre := longNombreAxe;
end;

function Tcourbe.FormatAxe(x: double): strUnite;
begin
   FormatAxe := FloatToStrF(x, ffGeneral, NbreChiffre, 0);
end;

procedure TgrapheGTS.CourantVersAff;
var
   voie: indiceVoieGTS2;
   CompteurVoieAna: integer;

   procedure Decale(voie: indiceVoieGTS2);
   var
      j: integer;
      alphaLoc: double;
   begin
      with voieOrphy[voie] do
      begin
         AlphaLoc := CoeffInterp * CompteurVoieAna;
         vecteurAff[0] := valeurReal(vecteurCourant[0]) * (1 + alphaLoc) -
            valeurReal(vecteurCourant[1]) * alphaLoc;
         for j := 1 to pred(nbreCourant) do
            vecteurAff[j] := valeurReal(vecteurCourant[pred(j)]) *
               alphaLoc + valeurReal(vecteurCourant[j]) * (1 - alphaLoc);
         Inc(CompteurVoieAna);
      end;
   end;

begin
   compteurVoieAna := 0;
   for voie := voieGts2Min to VoieGts2Max do
      if voieOrphy[voie].active then
         decale(voie);
   NbreAff := NbreCourant;
end; // CourantVersAff

procedure TbaseDetemps.setMinMax;
var
   maxmax: double;
   exponent: integer;
begin
   if Mini >= Maxi then
      Maxi := Mini + 1;
   MaxMax := abs(maxi);
   if MaxMax < abs(mini) then
      MaxMax := abs(mini);
   Exponent := floor(Log10(MaxMax));
   if (Exponent mod 3) < 0 then
      Exponent := (Exponent div 3) - 1
   else
      Exponent := Exponent div 3;
   // la division euclidienne des informaticiens <> div des maths !
   setExposant(maxmax, Exponent);
end;

procedure TbaseDeTemps.Load(Avoie: strUnite; fichier: tMemIniFile);
begin
   inherited load(Avoie, fichier);
   Maxi := fichier.ReadFloat(Avoie, 'Dure', 1.0);
end;

procedure TbaseDeTemps.Store(Avoie: strUnite; fichier: tMemIniFile);
begin
   inherited store(Avoie, fichier);
   with fichier do
      WriteFloat(Avoie, 'Dure', Maxi);
end;

procedure TgrapheGTS.setNbreInter(Avalue: integer);
var
   voie: indiceVoieGTS2;
   Nmax: integer;
begin
   FnbreInter := Avalue;
   NbreVoie := 0;
   for voie := voieGts2Min to voieGTs2Max do
      if voieOrphy[voie].active then
         Inc(NbreVoie);
   if NbreVoie = 0 then
   begin
      voieOrphy[0].active := True;
      NbreVoie := 1;
   end;
   Nmax := MaxResulGTS div NbreVoie;
   if Nmax > maxVecteur then
      Nmax := MaxVecteur;
   if Avalue > NMax then
      Avalue := NMax;
   if Avalue < MinPointsTemps then
      Avalue := MinPointsTemps;
   FnbreInter := Avalue;
end;

procedure TbaseDeTemps.setDureeBalayage(Avalue: double);
var
   dureeMin, dureeMax: double;
begin
   if GrapheGTS.NbreVoie = 1 then
      dureeMin := 1e-4
   else
      dureeMin := minPointsTemps * GrapheGTS.NbreVoie * deltatOrphy;
   dureeMax := dureeMaxMax;
   if Avalue < dureeMin then
      Avalue := dureeMin;
   if Avalue > dureeMax then
      Avalue := dureeMax;
   FdureeBalayage := avalue;
   mini := 0;
   maxi := FdureeBalayage;
end;


procedure Tcourbe.chercheMinmax(Nbre: integer; var min, max: double);
var
   i: integer;
   marge: double;
begin
   Min := vecteurAff[0];
   Max := Min;
   for i := 1 to pred(Nbre) do
      if vecteurAff[i] < Min then
         Min := vecteurAff[i]
      else if vecteurAff[i] > Max then
         Max := vecteurAff[i];
   marge := (maxi - mini) / 50;
   min := min - marge;
   if min < mini then
      min := mini;
   max := max + marge;
   if max > maxi then
      max := maxi;
end;

procedure TgrapheGTS.PenDefault;
begin
   canvas.pen.mode  := pmCopy;
   canvas.pen.style := psSolid;
   canvas.pen.Width := 1;
end;


end.

