unit GTSDEF;

interface

uses Classes, SysUtils, oomisc, inifiles, Graphics,
   uniteker, maths, gtsutil, adport, constgts, registry,
   Dialogs, Math, gti2_dynamique;

const
   MaxiHardSA     = 255;
   MaxiHardEA     = 65536;
   HardMaxi       = +32767;
   MinPointsTemps = 8;
   MinSeuil       = 8;
   MaxResulGTS    = 60000;

type
   Tmotif        = (mCroix, mCroixDiag, mCercle, mCarre, mLosange);
   TetatAcq      = (eaAttenteSynchro, eaAttenteResul, eaMajResul, eaTerminee);
   TetatAvail    = (eaNone, eaResul);

const
   promptMontant: array[boolean] of string[2] = (' ', ' ');
   DeltatMax   = 3600.0; { s = 1 heure }
   DureeMaxMax = deltatMax * MaxVecteur;
   { 8192 heures = 341 jours = 11 mois }
   { 1024 heures = 43 jours = 1,5 mois }
   DeltatOrphy = 1.0e-5;
   VitesseMax  = 3;
   VitesseInt: array[0..VitesseMax] of longint = (9600, 19200, 38400, 57600);
   PriseVoie: array[0..11] of char = 'AAFFBBCCGHGH';
   PriseVoieGTI2: array[0..11] of char = 'AAAAAAAAGHIJ';

type
   TCapteur = class
      OffsetC:        double;
      UnVoltC:        double;
      nomC:           strUnite;
      nomUniteC:      strUnite;
      signifC, aideC: shortString;
      MiniAff, MaxiAff: double;
      constructor Create;
      procedure Assign(Acapteur: Tcapteur);
   end;

   Tvoie = class(tunite)
   private
      FnumOrphy:    integer;
      Fmini, Fmaxi: double;
      procedure setMaxi(Value: double);
      procedure setMini(Value: double);
      procedure setNumOrphy(Value: integer);
   protected
      function GetNomVoieOrphy: strUnite; virtual;
   public
      Offset, UnVolt: double;
      ValeurCourante, valeurAff: double;
      aide: shortString;
      vecteurAff:  vecteur;
      vecteurCourant: vecteurInt;
      active:      boolean;
      couleur:     Tcolor;
      Motif:       Tmotif;
      AvecCapteur: boolean;
//      valeurParam : double;
      valeurParamTxt : shortString;
      property NumOrphy: integer Read FnumOrphy Write setNumOrphy;
      property Maxi: double Read Fmaxi Write setMaxi;
      property Mini: double Read Fmini Write setMini;
      property NomVoieOrphy: strUnite Read GetNomVoieOrphy;
      constructor Create(num : byte);
      function CodeVoie: strUnite;
      procedure AffecteCapteur(capteurCourant: Tcapteur);
      function AffVoltmetre: strUnite; virtual;
      function AffValeur(i: integer): strUnite; virtual;
      function ValeurReal(V: integer): double; virtual;
      function ValeurVolt(V: integer): double; virtual;
      function VoltToReal(V: double): double;
      function CalibreMaxi: double;
      function CalibreMini: double;
      procedure RazVecteur;
      procedure setMinMax; virtual;
      procedure load(Avoie: strUnite; Fichier: tMemIniFile); virtual;
      procedure store(Avoie: strUnite; Fichier: tMemIniFile); virtual;
      function calibreVoie: strUnite; virtual;
   end;

const
   crLF = #10;
   crCR = #13;

type
   TlisteCapteur = class(TList)
   protected
      function Get(Index: integer): Tcapteur;
      procedure Put(Index: integer; Item: Tcapteur);
   public
      function Add(Item: Tcapteur): integer;
      function IndexOf(Item: Tcapteur): integer;
      function Remove(Item: Tcapteur): integer;
      property Items[Index: integer]: Tcapteur Read Get Write Put;
      procedure Clear; override;
   end;

var
   NomFichierCapteurs: shortString;
   ListeCapteur:  TlisteCapteur;

procedure ChargeListeCapteurs;
procedure SauveListeCapteurs;

function nomEtatAcq(etat: TetatAcq): shortString;

const

   TitreVoieGTI2: array[indiceVoieGTS2] of string[20] = (
      'EA0',
      'EA1',
      'EA2',
      'EA3',
      'EA4',
      'EA5',
      'EA6',
      'EA7',
      'EAD0 diff. prise G',
      'EAD1 diff. prise H',
      'EAD2 diff. prise I',
      'EAD3 diff. prise J');

implementation

const
   CouleurDefaut: array[indiceVoieGTS2] of Tcolor =
      (clBlack, clRed, clBlack, clBlack, clRed, clNavy,
      clGreen, clPurple, clBlue, clFuchsia, clRed, clBlue);

procedure SauveListeCapteurs;
{  sauvegarde de la liste  }
var
   fichier: tMemIniFile;

   procedure Ecrit;
   var
      i: integer;
      ligne: shortString;
   begin
      fichier.writeInteger('Capteurs', 'Nombre', ListeCapteur.Count);
      for i := 0 to pred(ListeCapteur.Count) do
         with ListeCapteur.items[i] do
         begin
            ligne := IntToStr(i + 1);
            fichier.writeFloat(ligne, '0V', Offsetc);
            fichier.writeFloat(ligne, '1V', UnVoltc);
            fichier.writeFloat(ligne, 'Mini', MiniAff);
            fichier.writeFloat(ligne, 'Maxi', MaxiAff);
            fichier.writeString(ligne, 'Nom', nomC);
            fichier.writeString(ligne, 'Unite', nomUniteC);
            fichier.writeString(ligne, 'Signif', signifC);
            fichier.writeString(ligne, 'Aide', aideC);
         end;
   end;

var
   regIni: TregIniFile;
begin
   fichier := tMemIniFile.Create(nomFichierCapteurs);
   try
      Ecrit;
      fichier.UpdateFile;
      fichier.Free;
   except
   end;
   try
      RegIni := TRegIniFile.Create(RegistreGTS);
      RegIni.writeString('Capteurs', 'Fichier', NomFichierCapteurs);
      RegIni.Free;
   except
   end;
end;

procedure ChargeListeCapteurs;

   procedure TrierCapteurs;
   { Tri Shell Meyer-Baudoin p 456 }
   { origine vecteur en 0 }
   var
      cle:  shortString;
      i, j: integer;
      SauveAppareil: Tcapteur;
   begin
      i := 1;  // lment 0 tri
      while (i < ListeCapteur.Count) do
      begin
         sauveAppareil := ListeCapteur[i];
         cle := sauveAppareil.signifC;
         j := pred(i);
         while (j >= 0) and (ListeCapteur.items[j].signifC > cle) do
         begin
            ListeCapteur[succ(j)] := ListeCapteur[j];
            Dec(j);
         end;
         ListeCapteur[succ(j)] := sauveAppareil;
         Inc(i);
      end;
   end;

   var
      i: integer;
      NbreCapteurs: integer;
      CapteurCourant: Tcapteur;
      ligne: shortString;
      fichier: tMemIniFile;
   begin
      fichier := tMemIniFile.Create(nomFichierCapteurs);
      listeCapteur.Clear;
      NbreCapteurs := fichier.readInteger('Capteurs', 'Nombre', 0);
      for i := 1 to NbreCapteurs do
      begin
         CapteurCourant := Tcapteur.Create;
         ListeCapteur.add(CapteurCourant);
         with CapteurCourant do
         begin
            ligne := IntToStr(i);
            Offsetc := fichier.ReadFloat(ligne, '0V', 0);
            UnVoltc := fichier.ReadFloat(ligne, '1V', 1);
            MiniAff := fichier.ReadFloat(ligne, 'Mini', 0);
            MaxiAff := fichier.ReadFloat(ligne, 'Maxi', 5);
            nomC  := fichier.readString(ligne, 'Nom', nomC);
            nomUniteC := fichier.readString(ligne, 'Unite', '');
            signifC := fichier.readString(ligne, 'Signif', '');
            aideC := fichier.readString(ligne, 'Aide', '');
         end;
      end;
      if NbreCapteurs = 0 then
      begin
         CapteurCourant := TCapteur.Create;
         ListeCapteur.add(CapteurCourant);
      end;
      TrierCapteurs;
      fichier.Free;
end; { ChargeListeCapteurs }

procedure Tvoie.setNumOrphy(Value: integer);
begin
   FnumOrphy := Value;
end;

procedure Tvoie.setMaxi(Value: double);
begin
   Fmaxi := Value;
end;

procedure Tvoie.setMini(Value: double);
begin
   Fmini := Value;
end;

procedure Tvoie.Store(Avoie: strUnite; Fichier: tMemIniFile);
begin
   with fichier do
   begin
      WriteString(Avoie, 'Nom', nom);
      WriteString(Avoie, 'Unite', nomUnite);
      WriteString(Avoie, 'Signif', valeurParamTxt);
//      WriteFloat(Avoie, 'valeurParam', valeurParam);
      WriteFloat(Avoie, 'Mini', mini);
      WriteFloat(Avoie, 'Maxi', maxi);
      if not avecCapteur and (numOrphy >= 0) then
      begin
         offset := 0;
         UnVolt := 1;
      end;
      WriteFloat(Avoie, 'Offset', Offset);
      WriteFloat(Avoie, 'UnVolt', UnVolt);
      WriteBool(Avoie, 'Active', active);
      WriteInteger(Avoie, 'Couleur', couleur);
      WriteInteger(Avoie, 'Motif', Ord(motif));
      WriteBool(Avoie, 'Capteur', avecCapteur);
   end;
end; // Tvoie.store

constructor Tvoie.Create(num : byte);
begin
   inherited Create;
   FnumOrphy := num;
   nomUnite := 'V';
   nom := 'V'+intTostr(num);
   valeurParamTxt := '';
   Mini := -5;
   Maxi := 5;
   Offset := 0;
   UnVolt := 1;
   ValeurCourante := 0;
   Motif := mCroix;
   couleur := CouleurDefaut[num];
end;

procedure Tvoie.setMinMax;
var
   maxmax, marge: double;
   exponent: integer;
begin
   if nomUnite = 'hPa' then
      setExposant(100, 0);
   if not AvecCapteur then begin
      if Fmini < -10 then
         Fmini := -10;
      if Fmaxi > +10 then
         Fmaxi := +10;
   end;
   VerifMinMaxDouble(Fmini, Fmaxi);
   Marge := abs(ValeurReal(64) - ValeurReal(0));
   if (Fmaxi - Fmini) < marge then
      Fmaxi := Fmini + Marge;
   MaxMax := abs(Fmaxi);
   if MaxMax < abs(Fmini) then
      MaxMax := abs(Fmini);
   Exponent := floor(Log10(MaxMax));
   if (Exponent mod 3) < 0 then
      Exponent := (Exponent div 3) - 1
   else
      Exponent := Exponent div 3;
   // la division euclidienne des informaticiens <> div des maths ! 
   setExposant(maxmax, Exponent);
end; // setMinMax

function Tvoie.CodeVoie: strUnite;
begin
   codeVoie := IntToStr(FnumOrphy);
end;

function Tvoie.GetNomVoieOrphy: strUnite;
begin
      case FnumOrphy of
         0..7: Result  := 'EA' + IntToStr(FnumOrphy); // module
         8..11: Result := 'EAD-' + char(FnumOrphy + Ord('G') - 8); // DB15
      end
end;

function Tvoie.ValeurReal(V: integer): double;
begin
   Result := valeurVolt(V);
   if avecCapteur then
      Result := Offset + unVolt * Result;
end;

function Tvoie.VoltToReal(V: double): double;
begin
   Result := V;
   if avecCapteur then
      Result := Offset + unVolt * Result;
end;

function Tvoie.CalibreMaxi: double;
begin
   Result := valeurReal(HardMaxi);
end;

function Tvoie.CalibreMini: double;
begin
   Result := valeurReal(-HardMaxi);
end;

function Tvoie.ValeurVolt(V: integer): double;
begin
   Result := 20.0 * V / MaxiHardEA;
end;

procedure Tvoie.RazVecteur;
var
   i: integer;
begin
   for i := 0 to MaxVecteur do
   begin
      vecteurAff[i] := 0;
      vecteurCourant[i] := 0;
   end;
end;

procedure Tvoie.Load(Avoie: strUnite; fichier: tMemIniFile);
begin
   with fichier do
   begin
      AvecCapteur := ReadBool(Avoie, 'Capteur', False);
      Nom  := ReadString(Avoie, 'Nom', Nom);
      NomUnite := ReadString(Avoie, 'Unite', NomUnite);
      valeurParamTxt := ReadString(Avoie, 'Signif', '');
//      valeurParam := ReadFloat(Avoie, 'ValeurParam', 0);
      Mini := ReadFloat(Avoie, 'Mini', -5);
      Maxi := ReadFloat(Avoie, 'Maxi', 5);
      offset := ReadFloat(Avoie, 'Offset', 0);
      unVolt := ReadFloat(Avoie, 'UnVolt', 1);
      active := ReadBool(Avoie, 'Active', False);
      Couleur := readInteger(Avoie, 'Couleur', couleur);
      Motif := Tmotif(readInteger(Avoie, 'Motif', 0));
      if not AvecCapteur and
         ((offset <> 0) or (UnVolt <> 1)) then
         avecCapteur := True; (****)
      setMinMax;
   end;
end;

function Tvoie.CalibreVoie: strUnite;
begin
   if maxi * mini < 0 then
      if (maxi + mini) < (maxi - mini) / 10 then
         Result := '' + formatUnite(maxi)
      else
         Result := formatUniteDouble(mini, maxi)
   else if maxi > 0 then
      Result := formatUnite(maxi)
   else
      Result := formatUnite(mini);
end;

function Tvoie.AffVoltmetre: strUnite;
begin
   Result := FormatUniteExp(valeurAff, True);
end;

function Tvoie.AffValeur(i: integer): strUnite;
begin
   Result := FormatUniteExp(vecteurAff[i], False);
end;

function TlisteCapteur.Add(Item: Tcapteur): integer;
begin
   Result := inherited Add(Item);
end;

function TlisteCapteur.Get(Index: integer): Tcapteur;
begin
   Result := Tcapteur(inherited Get(Index));
end;

function TlisteCapteur.IndexOf(Item: Tcapteur): integer;
begin
   Result := inherited IndexOf(Item);
end;

procedure TlisteCapteur.Put(Index: integer; Item: Tcapteur);
begin
   inherited Put(Index, Item);
end;

function TlisteCapteur.Remove(Item: Tcapteur): integer;
begin
   Result := inherited Remove(Item);
   Item.Free;
end;

procedure TListeCapteur.Clear;
var
   i: integer;
begin
   for i := 0 to pred(Count) do
      Items[i].Free;
   inherited Clear;
end;

constructor Tcapteur.Create;
begin
   Offsetc := 0;
   unVoltc := 10;
   nomC  := 'T';
   NomUniteC := 'C';
   signifC := '';
   aideC := '';
end;

procedure Tcapteur.Assign(Acapteur: Tcapteur);
begin
   OffsetC := Acapteur.offsetc;
   UnVoltC := Acapteur.unVoltc;
   nomC  := Acapteur.nomC;
   NomUniteC := Acapteur.NomUniteC;
   signifC := Acapteur.signifC;
   aideC := Acapteur.aideC;
   MiniAff := Acapteur.miniAff;
   MaxiAff := Acapteur.maxiAff;
end;

function nomEtatAcq(etat: TetatAcq): shortString;
begin
   case etat of
      eaAttenteSynchro: Result := trWaitSynchro;
      eaTerminee: Result := trAcqFinie;
      eaAttenteResul: Result := trAttenteResul;
      eaMajResul: Result := '';
   end;
end;

procedure Tvoie.affecteCapteur(capteurCourant: TCapteur);
begin
   with CapteurCourant do
   begin
      nom  := nomC;
      nomUnite := nomUniteC;
      aide := aideC;
      Mini := MiniAff;
      Maxi := MaxiAff;
      avecCapteur := True;
      Offset := OffsetC;
      UnVolt := UnVoltC;
      setMinMax;
   end;
end;

initialization
   NomFichierCapteurs := changeFileExt(ParamStr(0), '.CPT');
   ListeCapteur := TlisteCapteur.Create;
   ChargeListeCapteurs;

finalization
   ListeCapteur.Clear;
   ListeCapteur.Free;
end.

