unit GBF_osc;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, spin, StdCtrls, Buttons, inifiles, ExtCtrls,
  maths_osc, gti2_osc_util, gti2_osc_graphe, uniteker, gti2Const, parser10, Lcd,
  gti2_osc;

type

  Tsignal = (sCarre,sTriangle,sSinus,sArbitraire,sContinu);
  TGBF = class
     private
     ParserI : Tparser;
     data : vecteurGBF;
     Procedure setExpression(avalue : wideString);
     Function getExpression : wideString;
     public
     Monocoup : boolean;
     Frequence : double;
     Amplitude,Offset,Duty : double;
     signal  : Tsignal;
     EnMarcheContinue : boolean;
     MajAFaire : boolean;
     property expression : wideString read getExpression write setExpression;
     Procedure CalculValeur;
     Procedure Marche;
     Procedure Arret;
     procedure Paint(Acanvas : Tcanvas;largeur,hauteur : integer);
     destructor Destroy;override;
     constructor Create;
  end;

var GBF : Tgbf;

implementation

Procedure TGBF.CalculValeur;

// 3FFF donc 16383 correspond  +10 V
// 2000 donc 8192 correspond a 0 V
// 0 correspond  -10 V

function valeurI(valeurR : double) : integer;
begin
     result := round(-valeurR*MaxiGBF/gammeGBF)+zeroGBF;
     if result<0 then result := 0;
     if result>MaxiGBF then result := maxiGBF;
end;

var valeurR,dt,pentePos,penteNeg : double;
    posDuty,longDuty,i,ref : integer;
begin
     case signal of
        sContinu : begin
           gti2.freqGBF := 1000;
           gti2.NbreGBF := 1024;
        end;
        else begin
           gti2.freqGBF := frequence;
           frequence := gti2.freqGBF; // valeur exacte
        end;
     end;
     dt := 1/gti2.NbreGBF;
     case signal of
        sContinu : for i := 0 to pred(gti2.NbreGBF) do
                       data[i] := valeurI(Offset);
        sArbitraire : for i := 0 to pred(gti2.NbreGBF) do begin
                 ParserI.T := i*dt;
                 valeurR := ParserI.Value;
                 data[i] := valeurI(valeurR);
        end;
        sSinus : for i := 0 to pred(gti2.NbreGBF) do begin
                valeurR := Amplitude*sin(2*pi*i*dt);
                data[i] := valeurI(valeurR+Offset);
        end;
        sTriangle : begin
          PosDuty := trunc(gti2.NbreGBF*Duty/2);
          if posDuty<2 then posDuty := 2;
          if posDuty>(gti2.NbreGBF/2-2) then posDuty := trunc(gti2.NbreGBF/2-2);
          LongDuty := gti2.NbreGBF-2*posDuty;
          pentePos := Amplitude/posDuty;
          penteNeg := -Amplitude*2.0/longDuty;
          for i := 0 to posDuty do
              data[i] := valeurI(i*pentePos+Offset);
          valeurR  := posDuty*pentePos+Offset;
          for i := 1 to longDuty do
              data[i+posDuty] := valeurI(i*penteNeg+valeurR);
          valeurR  := valeurR+longDuty*penteNeg;
          for i := 1 to posDuty do
              data[i+posDuty+longDuty] := valeurI(i*pentePos+valeurR);
        end;
        else begin
          PosDuty := trunc(gti2.NbreGBF*Duty);
          if posDuty<2 then posDuty := 2;
          if posDuty>(gti2.NbreGBF-2) then posDuty := gti2.NbreGBF-2;
          for i := 0 to posDuty do
              data[i] := valeurI(Amplitude+Offset);
          for i := succ(posDuty) to pred(gti2.nbreGBF) do
              data[i] := valeurI(-Amplitude+Offset);
        end;
     end;
     for i := gti2.nbreGBF to NbreMaxGBF do data[i] := zeroGBF;
     if monocoup then begin
        ref := data[pred(gti2.nbreGBF)];
        case signal of
             sCarre : begin
                data[0] := valeurI(Offset);
                ref := valeurI(Offset);
             end;
             sTriangle :;
             sSinus :;
             sArbitraire: ;
             sContinu :;
        end;
        for i := gti2.nbreGBF to gti2.nbreGBF+2 do
              data[i] := ref;
     end;
end;

destructor TGBF.Destroy;
begin
    parserI.free;
    inherited destroy;
end;

constructor TGBF.Create;
begin
     inherited create;
     ParserI := Tparser.create(nil);
     ParserI.expression := 't*5';
     monocoup := false;
end;

procedure TGBF.SetExpression(avalue : wideString);
begin
     if ParserI.expression=avalue then exit;
     try
     ParserI.expression := avalue;
     except
     on E: EMathParserError do begin
        showMessage(E.Message);
        end;
     end;
end;

function TGBF.getExpression : wideString;
begin
     result := ParserI.expression
end;

procedure TGBF.Marche;
begin
     gti2.stopGenerateur; // arrt
     EnMarcheContinue := not monocoup and (signal<>sContinu);
     gti2.boucleGenerateur := EnMarcheContinue;
     gti2.lanceGenerateur(data);
     MajAFaire := false;
end;

procedure TGBF.Arret;
begin
     gti2.stopGenerateur; // arrt
     EnMarcheContinue := false;
     MajAFaire := false;          
end;

procedure TGBF.Paint(Acanvas : Tcanvas;largeur,hauteur : integer);
var i : indiceVecteur;
    xi,yi,debut : integer;
    AX,AY : double;

Procedure GetXY;
begin
     yi := round(AY*data[i]);
     xi := debut+round(AX*i);
     Acanvas.LineTo(xi,yi);
end;

begin with Acanvas do begin
     brush.Style := bsSolid;
     brush.color := clWindow;
     pen.width := 1;
     pen.style := psSolid;
     pen.color := clBlack;
     rectangle(0,0,largeur,hauteur);
     pen.color := clBlue;
     AY := hauteur/maxiGBF; // y vers le bas compens par signe GTI2 ?
     yi := round(AY*data[0]);
     if signal=sContinu
        then begin
           MoveTo(0,yi);
           LineTo(largeur,yi);
         end // continu
         else begin
           AX := largeur/gti2.NbreGBF/1.4; // 20 % avant et aprs
           debut := round(AX*gti2.NbreGBF/5);
           MoveTo(debut,yi);
           for i := 0 to pred(gti2.NbreGBF) do GetXY;
           if monocoup
              then begin  // attente  zro
                 yi := hauteur div 2;
                 MoveTo(0,yi);
                 LineTo(debut,yi);
                 MoveTo(largeur-debut,yi);
                 LineTo(largeur,yi);
              end
              else begin // dlimitation priode
                 debut := round(6*AX*gti2.NbreGBF/5); // dbut prochaine priode
                 for i := 0 to gti2.NbreGBF div 5 do GetXY;
                 debut := -round(4*AX*gti2.NbreGBF/5); // fin priode prcdente
                 yi := round(AY*data[4*gti2.NbreGBF div 5]);
                 MoveTo(0,yi);
                 for i := (4*gti2.NbreGBF div 5) to pred(gti2.NbreGBF) do GetXY;
                 pen.style := psDash;
                 pen.color := clRed;
                 xi := round(AX*gti2.NbreGBF/5);
                 MoveTo(xi,0);
                 LineTo(xi,hauteur);
                 xi := round(6*AX*gti2.NbreGBF/5);
                 MoveTo(xi,0);
                 LineTo(xi,hauteur);
              end;
          end;// not continu
     pen.style := psDot;
     pen.color := clBlack;
     yi := hauteur div 2;
// zro
     if monocoup
        then begin
           MoveTo(debut,yi);
           LineTo(largeur-debut,yi);
         end
         else begin
            MoveTo(0,yi);
            LineTo(largeur,yi);
         end;
end end;

initialization
  gbf := tgbf.create;

finalization
  gbf.Free;

end.


