unit GTI2_osc_util;

interface

uses
  SysUtils, Windows, Messages, Classes, Dialogs,
  comCtrls, StdCtrls,ExtCtrls, Controls, Graphics, Forms, Buttons,
  printers, consts, registry, math, inifiles,
  gti2Const, uniteker, shlObj;

const
        WM_Affiche_Reglage = WM_USER + 4;
        MaxiHardEA = 65536;
        HardMaxi = +32767;
        indexCapteurMarteau = 1;
        indexCapteurMagnum = 15;
        indexCapteurSondor = 11;

        MaxiGBF = 16383; // 14 bits
        ZeroGBF = 8192; // la moiti
        GammeGBF = 20; // +-10 V
        SansSignif : double = 1e-26;
        voieG = 8;
        voieJ = 11;
        MaxPotarAuto = 4;
        MaxCapteurAuto = 22;

        RegistreGTI2 = 'Software\GTI2';

        scaleCurseur = 10;
        LongNombreAxe = 5;
        NbreGraduations = 10;
        NbreGradMin = 5;
        IntSansSignif = -1;

        PrecisionGTI2 = 5; // 14 bits
        PrecisionSynchro = 3; // 12 bits        
        PrecisionTime = 7;
        LongUnite = 32;
        SymbReg : char = '';
        SymbReg2 : char = '&';

        UnJour : double = 24*3600.0;
        UneHeure : double = 3600.0;
        QuartdHeure : double = 15*60.0;
        UneMinute : double = 60.0;

        alpha = #128;
        beta = #129;
        chi = #130;
        delta = #131;
        deltaMaj = #150;
        epsilon = #132;
        phi = #133; phiMaj = #151;
        gamma = #134; gammaMaj = #152;
        eta = #135;
        lambda = #136; lambdaMaj = #153;
        nu = #138;
        piMin = #139;piMaj = #154;
        theta = #140;thetaMaj = #155;
        rho = #141;
        sigma = #142;sigmaMaj = #156;
        tau = #143;
        omega = #144;omegaMaj = #157;
        xi = #147;xiMaj = #158;
        psi = #148;psiMaj = #159;
        zeta = #149;
        mu = #181;
        vide = #248;videMaj = #216;

type
    EGetFloatError = class(Exception);
    indiceCapteurAuto = 1..MaxCapteurAuto;
    TsetCapteurAuto = set of indiceCapteurAuto;
    indexPotar = 0..MaxPotarAuto;
    ChaineAzt = array[0..255] of char;
    TcharSet = set of char;
    strUnite = string[LongUnite];
    CoefUnite = (zero,yocto,zepto,atto,femto,pico,nano,micro,milli,
                 nulle,kilo,mega,giga,tera,peta,exa,zetta,yotta,infini);
    Tcalibre = (c50m,c100m,c200m,c500m,c1,c2,c5,c15);

    TCapteur = class
        OffsetC : double;
        UnVoltC : double;
        nomC : strUnite;
        nomUniteC : strUnite;
        signifC : shortString;
        MiniAff,MaxiAff : double;
        constructor create;
        procedure Assign(Acapteur : Tcapteur);
   end;


    TCapteurAuto = class
        protected
        public
        PotarActif : array[indexPotar] of boolean;
        ValeurPotar : array[indexPotar] of word;
        ValeurAttendue : array[indexPotar] of double;
        AidePotar : array[indexPotar] of shortString;
        OffsetC : double;
        UnVoltC : double;
        CodeAuto : integer;
        nomC : shortString;
        nomUniteC : strUnite;
        signifC : shortString;
        MiniAff,MaxiAff : double;
        EtalonnageFait : boolean;
        constructor create(Acode : integer);
        function ValeurReal(V : integer) : double;
        function AvecEtalonnage : boolean;
        procedure load(fichier : TmemIniFile);
        procedure store(fichier : TmemIniFile);
   end;

  TlisteCapteur = class(TList)
  protected
    function Get(Index : Integer) : Tcapteur;
    procedure Put(Index : Integer;Item : Tcapteur);
  public
    function Add(Item : Tcapteur) : Integer;
    function IndexOf(Item : Tcapteur) : Integer;
    function Remove(Item : Tcapteur) : Integer;
    property Items[Index : Integer] : Tcapteur read Get write Put;
    Procedure Clear;override;
  end;



Const
        nomPotar : array[indexPotar] of string[12] =
             ('offset zoom','gain zoom','offset','gain','seuil');
        crSupprArr = char(vk_back);
        crSuppr = char(vk_clear);
        crHaut = char(vk_up);
        crBas = char(vk_down);
        crGauche = char(vk_left);
        crDroite = char(vk_right);
        crHome = char(vk_home);
        crEnd = char(vk_end);
        crCR = #13;
        crLF = #10;
        crESC = #27;
        crTab = char(vk_tab);
        Separateur : TCharSet = [' ',crTab,','];
        Majuscule   : TcharSet = ['A'..'Z'];
        Minuscule   : TcharSet = ['a'..'z'];
        Grecque  : TcharSet = [alpha,beta,gamma,gammaMaj,delta,deltaMaj,chi,
              epsilon,phi,phiMaj,eta,psi,psiMaj,lambda,lambdaMaj,mu,nu,
              piMin,piMaj,theta,thetaMaj,rho,sigma,sigmaMaj,tau,omega,omegaMaj,
              xi,xiMaj,zeta];
        ToutCarac   : TcharSet = [#032..#151,#224..#250];
        Chiffre     : TcharSet = ['0'..'9'];
        ChiffreEtc : TcharSet = ['0'..'9','+','-','e','E','.'];
        ChiffreWin : TcharSet = ['0'..'9','+','-','e','E','.',','];
        ToucheEdition : TcharSet = [crSupprArr,crGauche,crDroite,crHome,crEnd,crSuppr];

        caracPrefixe : TcharSet =
           ['y','z','a','f','p','n','','m','k','M','G','T','P','E'];
        caracUnite : TcharSet =
           ['A'..'Z','a'..'z',OmegaMaj,'1'..'3','-','.','/',''];

        prefixeUnite : array[coefunite] of char =
           ('0','y','z','a','f','p','n','','m',
            ' ','k','M','G','T','P','E','Z','Y','');
        valUnite : array[coefunite] of double =
          (1E-27,1E-24,1E-21,1E-18,1E-15,1E-12,1E-9,1E-6,1E-3,
                1,1E+3,1E+6,1E+9,1E+12,1E+15,1E+18,1E+21,1E+24,1E+27);
        puissanceUnite : array[coefunite] of shortInt =
          (-27,-24,-21,-18,-15,-12,-9,-6,-3,
                0,+3,+6,+9,+12,+15,+18,+21,+24,+27);
        expUnite : array[coefUnite] of string[4] =
        ('E-27','E-24','E-21','E-18','E-15','E-12','E-9','E-6','E-3',
                '','E+3','E+6','E+9','E+12','E+15','E+18','E+21','E+24','E+27');
        CalibreStr : array[Tcalibre] of string[6] =
             ('50 mV','0,1 V','0,2 V','0,5 V',
              '1 V','2 V','5 V','15 V');
        CalibreGTS2Str : array[Tcalibre] of string[6] =
             ('50 mV','0,1 V','0,2 V','0,5 V',
              '1 V','2 V','5 V','10 V');
        ValeurCalibre : array[Tcalibre] of double =
             (0.05,0.1,0.2,0.5,1,2,5,15);
        ValeurCalibreGTS2 : array[Tcalibre] of double =
             (0.05,0.1,0.2,0.5,1,2,5,10);

Function OKReg(NumOK : shortString) : boolean;
Procedure Info(NumInfo : shortString);
Procedure afficheErreur(Erreur : shortString);
Procedure afficheErreurFormat(Erreur : shortString;args : array of const);
Function ToucheValidation(key : word) : boolean;
Procedure TrimComplet(var S : shortString);
Function Pad(S : shortString;Len : integer) : shortString;
Function ConvertitChaine(nombre : strUnite;var valeur : double) : boolean;
Function FormatDefaut(nombre : double) : strUnite;
Function PuissToStr(puiss : integer) : strUnite;
Function CompareStr(s1,s2 : shortString) : boolean;
Function FloatToStrPoint(valeur : double) : strUnite;
Function StrToFloatWin(nombre : strUnite) : double;
Function EncodeDuree(duree : LongInt) : TdateTime;
Function DureeEcoulee(InstantDebut : TdateTime) : double;
Function CaracToChiffre(carac : char) : integer;
function PosSeparateur(C : char;S : string;N : Integer): Integer;
Function existeOption(opt : char;defaut : boolean) : boolean;
Function FormatGTI2(nombre : double) : strUnite;
Function FormatSynchro(nombre : double) : strUnite;
Function FormatTime(nombre : double) : strUnite;
Function DateTimeToFloat(nombre : strUnite) : double;
function valeur125(valeur : double) : double;
function NumToCodePotar(numOrphy : integer) : char;
function DonneVersion(nomExe : string) : String ;
Function stOuiNon(isOui : boolean) : shortString;
Procedure SauveListeCapteurs;

var
     WidthCursor : integer = 3;
     dimPoint : integer = 3;
     avecCurseur : boolean = true;
     grille : boolean = true;
     FichierConfig : string;
     FichierConfigGeneral : string;
     MesDocsDir,DataDir : string;
     FichierCapteurAuto : boolean;
     ListeCapteursAuto : array[indiceCapteurAuto] of TCapteurAuto;
     ListeCapteur : TlisteCapteur;
     NomFichierCapteurs : string;

implementation

const

NsauveCapAuto = 217;

SauveCapteurAuto : array[0..NsauveCapAuto] of shortString =
(
'[Auto1]',
'PotarActif0=0',
'PotarActif1=0',
'PotarActif2=0',
'PotarActif3=0',
'0V=0',
'1V=1',
'Mini=0',
'Maxi=1',
'Nom=MR',
'Unite=',
'Signif=Marteau rflexe',
'Attente=1',
'ValeurPotar4=0',
'PotarActif4=0',
'AidePotar4=',
'Valeur4=0',
'Synchro=1',
'[Auto2]',
'PotarActif0=0',
'PotarActif1=0',
'ValeurPotar2=0',
'ValeurPotar3=0',
'ValeurPotar4=0',
'0V=0',
'1V=250',
'Mini=0',
'Maxi=2500',
'Nom=P',
'Unite=hPa',
'Signif=pression absolue',
'[Auto3]',
'PotarActif0=0',
'PotarActif1=0',
'PotarActif2=0',
'PotarActif3=0',
'0V=40',
'1V=8',
'Mini=4',
'Maxi=120',
'Nom=T',
'Unite=dB',
'Signif=Decible mtre acoustique',
'Attente=1',
'PotarActif4=0',
'Synchro=0',
'[Auto4]',
'PotarActif0=0',
'PotarActif1=0',
'ValeurPotar2=496',
'PotarActif2=1',
'AidePotar2=Mettre l''entre  la pression ambiante',
'Valeur2=0',
'PotarActif3=0',
'PotarActif4=0',
'0V=0',
'1V=200',
'Mini=-2000',
'Maxi=2000',
'Nom=P',
'Unite=Pa',
'Signif=pression relative',
'[Auto5]',
'PotarActif0=0',
'PotarActif1=0',
'PotarActif2=0',
'PotarActif3=0',
'0V=0',
'1V=10',
'Mini=0',
'Maxi=100',
'Nom=RH',
'Unite=%',
'Signif=Hygromtrie',
'Attente=1',
'PotarActif4=0',
'Synchro=0',
'[Auto6]',
'PotarActif0=0',
'PotarActif1=0',
'ValeurPotar2=108',
'PotarActif2=1',
'AidePotar2=Mettre le teslamtre dans une zone sans champ',
'Valeur2=0',
'PotarActif3=0',
'PotarActif4=0',
'0V=0',
'1V=0.005',
'Mini=-0.05',
'Maxi=0.05',
'Nom=B',
'Unite=T',
'Signif=Champ magntique',
'[Auto7]',
'PotarActif0=0',
'PotarActif1=0',
'PotarActif2=0',
'PotarActif3=0',
'0V=0',
'1V=10',
'Mini=0',
'Maxi=100',
'Nom=T',
'Unite=%',
'Signif=Turbidit',
'Attente=1',
'PotarActif4=0',
'Synchro=0',
'[Auto8]',
'PotarActif0=0',
'Valeur0=20.95',
'ValeurPotar1=1023',
'PotarActif1=0',
'AidePotar1=Mettre le capteur dans l''air libre',
'Valeur1=0',
'PotarActif2=0',
'ValeurPotar3=150',
'PotarActif3=1',
'AidePotar3=Mettre le capteur  l''air libre',
'Valeur3=20.95',
'PotarActif4=0',
'0V=0',
'1V=2.2',
'Mini=0',
'Maxi=22',
'Nom=O2',
'Unite=%',
'Signif=Pourcentage d''oxygne',
'[Auto9]',
'PotarActif0=0',
'PotarActif1=0',
'PotarActif2=0',
'PotarActif3=0',
'0V=0',
'1V=1',
'Mini=0',
'Maxi=5',
'Nom=T',
'Unite=',
'Signif=',
'Attente=1',
'PotarActif4=0',
'Synchro=0',
'[Auto10]',
'PotarActif0=0',
'PotarActif1=0',
'ValeurPotar2=800',
'PotarActif2=1',
'AidePotar2=Plonger dans une solution de pH 7',
'Valeur2=7',
'ValeurPotar3=820',
'PotarActif3=1',
'AidePotar3=Plonger dans une solution de pH 4',
'Valeur3=4',
'PotarActif4=0',
'0V=0',
'1V=1.4',
'Mini=0',
'Maxi=14',
'Nom=pH',
'Unite=',
'Signif=pH',
'[Auto12]',
'PotarActif0=0',
'PotarActif1=0',
'PotarActif2=0',
'PotarActif3=0',
'PotarActif4=0',
'0V=0',
'1V=11',
'Mini=-50',
'Maxi=110',
'Nom=T',
'Unite=C',
'Signif=Temprature',
'[Auto14]',
'PotarActif0=0',
'PotarActif1=0',
'PotarActif2=0',
'PotarActif3=0',
'PotarActif4=0',
'0V=0',
'1V=12000',
'Mini=0',
'Maxi=120000',
'Nom=',
'Unite=lux',
'Signif=luminosit',
'[Auto15]',
'PotarActif0=0',
'PotarActif1=0',
'PotarActif2=0',
'PotarActif3=0',
'0V=0',
'1V=1',
'Mini=0',
'Maxi=5',
'Nom=y',
'Unite=m',
'Signif=Fouche optique',
'Attente=1',
'PotarActif4=0',
'Synchro=0',
'[Auto16]',
'PotarActif0=0',
'PotarActif1=0',
'PotarActif2=0',
'PotarActif3=0',
'0V=0',
'1V=7E-5',
'Mini=0',
'Maxi=7e-4',
'Nom=Vecg',
'Unite=V',
'Signif=Electrocardiogramme',
'Attente=1',
'PotarActif4=0',
'Synchro=0'
);

Procedure UpCaseStr(var s : shortstring);
var i : integer;
begin
     for i := 1 to length(s) do s[i] := UpCase(S[i])
end;

Procedure TrimComplet(var s : shortstring);
var longueur,posCourant : integer;
begin
	longueur := Length(s);
	posCourant := 1;
	while (posCourant<=longueur) do
		if ord(s[posCourant])<=32
			then begin
			   Delete(s,posCourant,1);
			   dec(longueur);
			end
			else inc(posCourant);
end;

Function OKReg(NumOK : shortString) : boolean;
begin
     OKreg := MessageDlg(NumOK,mtConfirmation,[mbYes,mbNo],0)=idYes
end;

Procedure Info(NumInfo : shortString);
begin
     MessageDlg(NumInfo,mtInformation,[mbOK],0)
end;

Procedure ConvertitPrefixe(var nombre : strUnite);
var u : coefUnite;
    c : char;
begin
   c := nombre[length(nombre)];
   if c in caracPrefixe then begin
        u := zero;
        repeat inc(u) until (c=prefixeUnite[u]);
        delete(nombre,length(nombre),1);
        nombre := nombre+expUnite[u];
   end;
end;

Function StrToFloatWin(nombre : strUnite) : double;
var posSeparator : integer;
begin
   if decimalSeparator<>',' then begin
      posSeparator := pos(',',nombre);
      if posSeparator>0 then Nombre[posSeparator] := decimalSeparator
   end
   else if decimalSeparator<>'.' then begin
      posSeparator := pos('.',nombre);
      if posSeparator>0 then Nombre[posSeparator] := decimalSeparator
   end;
   ConvertitPrefixe(nombre);
   try
   StrToFloatWin :=  strToFloat(nombre);
   except
   StrToFloatWin := sansSignif;
   end;
end;

Function ConvertitChaine(nombre : strUnite;var valeur : double) : boolean;
begin
   if (upcase(nombre[1])='E')
	  then nombre := '1'+nombre
	  else if (nombre[1]='.') or (nombre[1]=',') 
		   then nombre := '0'+nombre;
   valeur := StrToFloatWin(nombre);
   convertitChaine := valeur<>sansSignif;
end;

Function FormatDefaut(Nombre : double) : strUnite;
var Nexp : integer;
begin
     if Nombre=sansSignif then begin
        result := ' ';
        exit;
     end;
     if abs(Nombre)<1e-10
         then Nexp := 2
         else if abs(Nombre)<1e+10
              then Nexp := 1
              else Nexp := 2;
      result := FloatToStrF(nombre,ffGeneral,5,Nexp)
end;

procedure afficheErreur(Erreur : shortString);
var SauveCursor : Tcursor;
begin
     SauveCursor := Screen.cursor;
     Screen.cursor := crDefault;
     MessageDlg(Erreur,mtError,[mbOK],0);
     Screen.cursor := SauveCursor;
end;

procedure afficheErreurFormat(Erreur : shortString;args : array of const);
var SauveCursor : Tcursor;
begin
     SauveCursor := Screen.cursor;
     Screen.cursor := crDefault;
     MessageDlg(Format(Erreur,args),mtError,[mbOK],0);
     Screen.cursor := SauveCursor;
end;

Function PuissToStr(puiss : integer) : strUnite;
var dest : strUnite;
begin
      if puiss=0
         then dest := ''
         else dest := IntToStr(puiss);
      PuissToStr := dest
end;

Function CompareStr(s1,s2 : shortstring) : boolean;
begin
    upCaseStr(s1);
    upCaseStr(s2);
    compareStr := s1=s2;
end;

Function ToucheValidation(key : word) : boolean;
begin
    ToucheValidation := key in [vk_tab,vk_left,vk_right,vk_down,vk_up,vk_return]
end;

Procedure ValidateDouble(Sender : Tedit;var z : double);
begin
     z := StrToFloatWin(sender.text);
     if z=sansSignif then sender.text := '';
end;

function Pad(S : shortString;Len : integer) : shortstring;
{-Return a string right-padded to length len with ' '}
var
  o : shortstring;
  Slen : integer;
begin
   Slen := length(S);
   if SLen>Len then S := copy(S,1,Len);
   setLength(o,len);
   Move(S[1], o[1], SLen);
   FillChar(o[Succ(SLen)], Len-SLen,' ');
   Pad := o;
end;

Function EncodeDuree(duree : LongInt) : TdateTime;
var h,m,s,ms : word;
begin
    h := duree div 3600;
    duree := duree-h*3600;
    m := duree div 60;
    duree := duree-m*60;
    s := duree;
    ms := 0;
    EncodeDuree := EncodeTime(h,m,s,ms);
end;

Function DureeEcoulee(InstantDebut : TdateTime) : double;
var h,m,s,ms : word;
begin
      decodeTime(now-instantDebut,h,m,s,ms);
      DureeEcoulee := h*3600.0+m*60.0+s*1.0+ms*0.001;
end;

Function CaracToChiffre(carac : char) : integer;
begin
      if carac in chiffre
         then result := ord(carac)-ord('0')
         else case carac of
              #186 : result := 0;
              #185 : result := 1;
              #178 : result := 2;
              #179 : result := 3;
              #165 : result := 4;
              #169 : result := 5;
              #172 : result := 6;
              #173 : result := 7;
              #174 : result := 8;
              #188 : result := 9;
              else result := 0;
         end;
end;

function PosSeparateur(C : char;S : string;N : Integer): Integer;
var
  j,i : Integer;
begin
  j := 0;
  for i := 1 to length(S) do
      if S[i]=C then begin
         inc(j);
         if j=N then begin
            result := i;
            exit;
         end;
      end;   
  result := 0;
end;

Function existeOption(opt : char;defaut : boolean) : boolean;
var str : shortstring;
    i : integer;
begin
    result := defaut;
    for i := 1 to ParamCount do begin
       str := ParamStr(i);
       if (length(str)>1) and (str[1]='/') and (upcase(str[2])=opt) then begin
          if length(str)=2
             then result := true
             else result := (str[3]='+');
          exit;
       end;
    end;
end;

Function FormatGeneral(Nombre : double;Nchiffres : integer) : strUnite;
var Nexp : integer;
begin
     if Nombre=sansSignif then begin
        FormatGeneral := ' ';
        exit;
     end;
     if abs(Nombre)<1e-10
         then Nexp := 2
         else if abs(Nombre)<1e+10
              then Nexp := 1
              else Nexp := 2;
      FormatGeneral := FloatToStrF(nombre,ffGeneral,Nchiffres,Nexp)
end;


Function FormatGTI2(Nombre : double) : strUnite;
begin
     result := FormatGeneral(Nombre,PrecisionGTI2)
end;

Function FormatSynchro(Nombre : double) : strUnite;
begin
     result := FormatGeneral(Nombre,PrecisionSynchro)
end;

Function FormatTime(Nombre : double) : strUnite;
begin
     FormatTime := FormatGeneral(Nombre,PrecisionTime)
end;

function PosN(const C : shortString;S : shortString;const K : Integer): Integer;
var
  P : Integer;
begin
  Result := 0;
  if K>0 then Delete(S,1,K);
  P := Pos(C, S);
  if P>0 then result := K+P;
end;


Function DateTimeToFloat(nombre : strUnite) : double;
var i,Njour : integer;
begin
     result := 0;
     trimComplet(nombre);
     i:=pos(':',nombre);
     if i>0
        then begin
           i:=pos('j',nombre);
           Njour := 0;
           if i>0
               then begin
                 if i>1 then begin
                   try
                   Njour := StrToInt(copy(nombre,1,pred(i)));
                   except
                   end;
                 end; { i>1 }
                 delete(nombre,1,i);
               end { i>0 }
               else begin
                   i := posN(':',nombre,2);
                   if i=0 then nombre := '0:'+nombre;
               end;
           try
           result := (StrToTime(nombre)+Njour)*60*60*24;
           except
           end;
        end {i>0}
        else begin
        i:=pos('j',nombre);
        if i>0 then begin
           if i>1 then begin
              try
              result := StrToInt(copy(nombre,1,pred(i)))*60*24*60;
              except
              end;
           end;
           delete(nombre,1,i);
        end;
        i:=pos('h',nombre);
        if i>0 then begin
           if i>1 then begin
              try
              result := result+StrToInt(copy(nombre,1,pred(i)))*60*60;
              except
              end;
           end;
           delete(nombre,1,i);
        end;
        i:=pos('min',nombre);
        if i>0 then begin
           if i>1 then begin
              try
              result := result+StrToFloatWin(copy(nombre,1,pred(i)))*60;
              except
              end;
           end;
           delete(nombre,1,i+2);
        end;
        i:=pos('m',nombre);
        if i>0 then begin
           if i>1 then begin
              try
              result := result+StrToFloatWin(copy(nombre,1,pred(i)))*1e-3;
              except
              end;
           end;
           delete(nombre,1,i);
        end;
        i:=pos('',nombre);
        if i>0 then begin
           if i>1 then begin
               try
               result := result+StrToFloatWin(copy(nombre,1,pred(i)))*1e-6;
               except
               end;
           end;
           delete(nombre,1,i);
        end;
        i:=pos('s',nombre);
        if i>0 then delete(nombre,i,1);
        if nombre<>'' then begin
            try
            result := result+strToFloatWin(nombre);
            except
            end;
        end;
        if result = 0 then result := 0.1;
     end;
end;

function valeur125(valeur : double) : double;
var mantisse,exposant,coeff : double;
begin
     exposant := log10(valeur);
     coeff := Int(exposant);
     if (exposant<0) and (coeff<>exposant) then coeff := coeff-1;
     coeff := power(10,coeff);
     mantisse := Int(valeur/coeff);
     if mantisse>2 then if mantisse<6
           then mantisse := 5
           else mantisse := 10;
     result := mantisse*coeff;
end;

Function FloatToStrPoint(valeur : double) : strUnite;
var nombre : strUnite;
    posSeparator : byte;
begin
   nombre := FloatToStrF(valeur,ffGeneral,7,2);
   if decimalSeparator<>'.' then begin
      posSeparator := pos(decimalSeparator,nombre);
      if posSeparator>0 then Nombre[posSeparator] := '.';
   end;
   result := nombre;
end;

Procedure ChargeListeCapteurs;
var fichier : tMemIniFile;

Procedure TrierCapteurs;
{ Tri Shell Meyer-Baudoin p 456 }
{ origine vecteur en 0 }
var cle : shortString;
    i,j : integer;
    SauveAppareil : Tcapteur;
Begin
    i := 1;  // lment 0 tri
    while (i<ListeCapteur.count) do begin
        sauveAppareil := ListeCapteur[i];
        cle := sauveAppareil.signifC;
        j := pred(i);
        while (j>=0) and (ListeCapteur.items[j].signifC>cle) do begin
             ListeCapteur[succ(j)] := ListeCapteur[j];
             dec(j);
        end;
        ListeCapteur[succ(j)] := sauveAppareil;
        inc(i);
    end;
end;

Procedure Capteurs;
{  chargement de la liste si vide cration  l'aide de defaut }
var i : integer;
    NbreCapteurs : integer;
    CapteurCourant : Tcapteur;
    ligne : shortString;
begin
    ListeCapteur := TlisteCapteur.create;
    NbreCapteurs := fichier.readInteger('Capteurs','Nombre',0);
    for i := 1 to NbreCapteurs do begin
      CapteurCourant := Tcapteur.create;
      ListeCapteur.add(CapteurCourant);
      with CapteurCourant do begin
        ligne := IntToStr(i);
        Offsetc := fichier.ReadFloat(ligne,'0V',0);
        UnVoltc := fichier.ReadFloat(ligne,'1V',1);
        MiniAff := fichier.ReadFloat(ligne,'Mini',0);
        MaxiAff := fichier.ReadFloat(ligne,'Maxi',5);
        nomC := fichier.readString(ligne,'Nom',nomC);
        nomUniteC := fichier.readString(ligne,'Unite','');
        signifC := fichier.readString(ligne,'Signif','');
    end end;
    if NbreCapteurs=0 then begin
       CapteurCourant := TCapteur.create;
       ListeCapteur.add(CapteurCourant);
    end;
    TrierCapteurs;
end; { Capteurs }

Procedure CapteursAuto;
var i : integer;
begin
    for i := 1 to MaxCapteurAuto do begin
        ListeCapteursAuto[i].load(fichier);
        FichierCapteurAuto := FichierCapteurAuto or (ListeCapteursAuto[i].signifC[0]<>'');
    end;
end; { CapteursAuto }

Procedure CapteursAutoInterne;
var i : integer;
    liste : TstringList;
begin
    liste := TstringList.create;
    for i := 0 to NsauveCapAuto do
        liste.add(sauveCapteurAuto[i]);
    fichier.setStrings(liste);
    CapteursAuto;
    liste.Free;
end; // CapteursAutoInterne

var i : integer;
begin
    NomFichierCapteurs := changeFileExt(application.exeName,'.CPT');
    fichier := tMemIniFile.create(nomFichierCapteurs);
    Capteurs;
    FichierCapteurAuto := false;
    for i := 1 to MaxCapteurAuto do
        ListeCapteursAuto[i] := TcapteurAuto.create(i);
    CapteursAuto;
    if not FichierCapteurAuto then CapteursAutoInterne;
    fichier.free;
end; // ChargeListeCapteurs 

Procedure SauveListeCapteurs; // sauvegarde de la liste
var fichier : tMemIniFile;

Procedure Ecrit;
Var i : integer;
    ligne : shortString;
begin
    fichier.writeInteger('Capteurs','Nombre',ListeCapteur.count);
    for i := 0 to pred(ListeCapteur.count) do
       with ListeCapteur.items[i] do begin
        ligne := IntToStr(i+1);
        fichier.writeFloat(ligne,'0V',Offsetc);
        fichier.writeFloat(ligne,'1V',UnVoltc);
        fichier.writeFloat(ligne,'Mini',MiniAff);
        fichier.writeFloat(ligne,'Maxi',MaxiAff);
        fichier.writeString(ligne,'Nom',nomC);
        fichier.writeString(ligne,'Unite',nomUniteC);
        fichier.writeString(ligne,'Signif',signifC);
    end;
end;

var regIni : TregIniFile;
begin
    fichier := tMemIniFile.create(nomFichierCapteurs);
    try
    Ecrit;
    fichier.UpdateFile;
    fichier.free;
    except
    end;
    try
    RegIni := TRegIniFile.create(RegistreGTI2);
    RegIni.writeString('Capteurs','Fichier',NomFichierCapteurs);
    RegIni.free;
    except
    end;
end; // sauveListeCapteurs

constructor TcapteurAuto.create(acode : integer);
var i : integer;
begin
        CodeAuto := Acode;
        Offsetc := 0;
        unVoltc := 10;
        nomC := 'T';
        NomUniteC := 'C';
        signifC := '';
        for i := 0 to MaxPotarAuto do begin
            valeurPotar[i] := 0;
            PotarActif[i] := false;
            valeurAttendue[i] := 0;
        end;
        MiniAff := -50;
        MaxiAff := 110;
        EtalonnageFait := false;
end;

function TcapteurAuto.avecEtalonnage;
var i : integer;
begin
     result := false;
     for i := 0 to maxPotarAuto do
         result := result or potarActif[i]
end;

Function TcapteurAuto.ValeurReal(V : integer) : double;
begin
     result := 20.0*V/MaxiHardEA;
     result := OffsetC+unVoltC*result;
end;

Procedure TcapteurAuto.Store(fichier : TmemIniFile);
var ligne : shortString;
    j : integer;
begin
            ligne := 'Auto'+IntToStr(codeAuto);
            for j := 0 to MaxPotarAuto do begin
                fichier.writeInteger(ligne,'ValeurPotar'+intToStr(j),valeurPotar[j]);
                fichier.writeBool(ligne,'PotarActif'+intToStr(j),PotarActif[j]);
                fichier.writeString(ligne,'AidePotar'+intToStr(j),aidePotar[j]);
                fichier.writeFloat(ligne,'Valeur'+intToStr(j),valeurAttendue[j]);
            end;
            fichier.writeFloat(ligne,'0V',Offsetc);
            fichier.writeFloat(ligne,'1V',UnVoltc);
            fichier.writeFloat(ligne,'Mini',MiniAff);
            fichier.writeFloat(ligne,'Maxi',MaxiAff);
            fichier.writeString(ligne,'Nom',nomC);
            fichier.writeString(ligne,'Unite',nomUniteC);
            fichier.writeString(ligne,'Signif',signifC);
end;

Procedure TcapteurAuto.Load(fichier : TmemIniFile);
var ligne : shortString;
    j : integer;
begin
            ligne := 'Auto'+IntToStr(codeAuto);
            EtalonnageFait := true;
            for j := 0 to MaxPotarAuto do begin
                valeurPotar[j] := fichier.readInteger(ligne,'ValeurPotar'+intToStr(j),valeurPotar[j]);
                PotarActif[j] := fichier.readBool(ligne,'PotarActif'+intToStr(j),PotarActif[j]);
                if PotarActif[j] then EtalonnageFait := false;                
                aidePotar[j] := fichier.readString(ligne,'AidePotar'+intToStr(j),aidePotar[j]);
                valeurAttendue[j] := fichier.readFloat(ligne,'Valeur'+intToStr(j),valeurAttendue[j]);
            end;
            Offsetc := fichier.readFloat(ligne,'0V',Offsetc);
            UnVoltc := fichier.readFloat(ligne,'1V',UnVoltc);
            MiniAff := fichier.readFloat(ligne,'Mini',MiniAff);
            MaxiAff := fichier.readFloat(ligne,'Maxi',MaxiAff);
            nomC := fichier.readString(ligne,'Nom',nomC);
            nomUniteC := fichier.readString(ligne,'Unite',nomUniteC);
            signifC := fichier.readString(ligne,'Signif',signifC);
end;

function NumToCodePotar(numOrphy : integer) : char; // ??
begin
     result := char(ord('0')+numOrphy-voieG)
end;

function DonneVersion(nomExe : string) : String ;
var
  VerInfoSize, VerValueSize,
  Dummy      : DWORD;
  VerValue   : PVSFixedFileInfo;
  VerInfo    : Pointer;
begin
  Result := '0.0.0';
  VerInfoSize := GetFileVersionInfoSize(PChar(NomExe),Dummy);
  If VerInfoSize = 0 Then Exit;
  GetMem(VerInfo, VerInfoSize);
  try
    if VerInfo <> nil then begin
      GetFileVersionInfo(PChar(NomExe), 0, VerInfoSize,VerInfo);
      VerQueryValue(VerInfo, '\', Pointer(VerValue), VerValueSize);
      with VerValue^ do begin
        Result := Format('%d.%d.%d', [dwFileVersionMS shr 16,
        dwFileVersionMS and $FFFF,dwFileVersionLS and $FFFF ]);
      end; { with }
     end; { if }
  finally
    FreeMem(VerInfo, VerInfoSize);
  end;
end;

{
    VerQueryValue(PBlock, PChar('\\StringFileInfo\\040C04E4\\FileDescription'),PS, Size);
    VerQueryValue(PBlock, PChar('\\StringFileInfo\\040C04E4\\FileVersion'),PS, Size);
    VerQueryValue(PBlock, PChar('\\StringFileInfo\\040C04E4\\Comments'),PS, Size);
    VerQueryValue(PBlock, PChar('\\StringFileInfo\\040C04E4\\LegalCopyright'),PS, Size);
    VerQueryValue(PBlock, PChar('\\StringFileInfo\\040C04E4\\CompanyName'),PS, Size);
// 040C04E4 = Langage / Id locale $040C / Franais - Standard
}


Function AddBackSlash(aPath : String):String;
Begin
  Result := EmptyStr;
  If (aPath <> '') and (aPath[Length(aPath)] <> '\') Then
    Result := aPath + '\';
End;

// permet de retrouver un repertoire d'aprs sa valeur CSIDL
Function GetFolder(CSIDLValue: integer): String;
Var
  IdList            : PITEMIDLIST;
  Folder            : Array[0..MAX_PATH] Of Char;
Begin
  if Not Failed(SHGetSpecialFolderLocation(0, CSIDLValue, IdList)) then begin
      SHGetPathFromIDList(IdList, Folder);
      Result := AddBackSlash(Folder);
    End;
End;

function TlisteCapteur.Add(Item : Tcapteur) : Integer;
begin
     result := inherited Add(Item)
end;

function TlisteCapteur.Get(Index : Integer) : Tcapteur;
begin
  Result := Tcapteur(inherited Get(Index))
end;

function TlisteCapteur.IndexOf(Item : Tcapteur) : Integer;
begin
  Result := inherited IndexOf(Item)
end;

procedure TlisteCapteur.Put(Index : Integer;Item : Tcapteur);
begin
  inherited Put(Index,Item)
end;

Function stOuiNon(isOui : boolean) : shortString;
begin
    if isOui then result := 'Oui' else result := 'Non'
end;

procedure Tcapteur.assign(Acapteur : Tcapteur);
begin
      OffsetC := Acapteur.offsetc;
      UnVoltC := Acapteur.unVoltc;
      nomC := Acapteur.nomC;
      NomUniteC := Acapteur.NomUniteC;
      signifC := Acapteur.signifC;
      MiniAff := Acapteur.miniAff;
      MaxiAff := Acapteur.maxiAff;
end;

function TlisteCapteur.Remove(Item : Tcapteur) : Integer;
begin
  Result := inherited Remove(Item);
  Item.free;
end;

Procedure TListeCapteur.clear;
var i : Integer;
begin
  for i := 0 to pred(Count) do Items[i].Free;
  inherited Clear;
end;

constructor Tcapteur.create;
begin
        Offsetc := 0;
        unVoltc := 10;
        nomC := 'T';
        NomUniteC := 'C';
        signifC := '';
end;

Initialization
//     winDir := GetFolder(CSIDL_Windows);
     mesDocsDir := GetFolder(CSIDL_Personal);
     if not DirectoryExists(MesDocsDir+'Regressi') then
       createDir(MesDocsDir+'Regressi');
     MesDocsDir := mesDocsDir+'Regressi\';
     DataDir := GetFolder(CSIDL_COMMON_APPDATA);
     if not DirectoryExists(DataDir+'Regressi') then
       createDir(DataDir+'Regressi');
     DataDir := DataDir+'Regressi\';
     FichierConfig := MesDocsDir+'GTI2.GTI2';
     FichierConfigGeneral := changeFileExt(Application.exeName,'.GTI2');     
     ChargeListeCapteurs;

{
COMMON : commun  tous les utilisateurs.

CSIDL_APPDATA : Rpertoire contenant les donnes des applications.
CSIDL_COMMON_APPDATA : Rpertoire contenant les donnes des applications.

CSIDL_FAVORITES : Rpertoire Favoris.
CSIDL_COMMON_FAVORITES : Rpertoire contenant les Favoris

CSIDL_STARTMENU: Rpertoire Menu Dmarrer.
CSIDL_COMMON_STARTMENU : Rpertoire du menu dmarrer

CSIDL_PROGRAMS: Rpertoire Programmes du Menu Dmarrer.
CSIDL_COMMON_PROGRAMS : Rpertoire Programmes du menu dmarrer

CSIDL_STARTUP: Rpertoire du groupe Dmarrage du Menu Dmarrer.
CSIDL_COMMON_STARTUP : Rpertoire du groupe Dmarrage du menu dmarrer

CSIDL_DESKTOP : Rpertoire correspondant  votre Bureau.
CSIDL_DESKTOPDIRECTORY : Rpertoire correspondant  votre Bureau.
CSIDL_COMMON_DESKTOPDIRECTORY : Rpertoire correspondant au bureau

CSIDL_COOKIES : Rpertoire ou sont stocks les cookies d'Internet Explorer.
CSIDL_FONTS: Rpertoire dans lequel sont stockes toutes les polices de caractres.
CSIDL_HISTORY: Rpertoire contenant les historiques d'Internet Explorer.
CSIDL_INTERNET_CACHE: Rpertoire ou sont stocks les fichiers temporaires d'Internet Explorer.
CSIDL_NETHOOD: Rpertoire Voisinage Rseau.
CSIDL_NETWORK: Network Neighborhood virtual folder representing the top level of the network hierarchy.
CSIDL_PERSONAL: Rpertoire Mes Documents.
CSIDL_PRINTHOOD: Rpertoire de voisinage d'impression.
CSIDL_PRINTERS: Printers folder virtual folder containing installed printers.
CSIDL_RECENT: Rpertoire dans lequel se trouvent les raccourcis vers les
    Fichiers rcemment ouverts.
CSIDL_SENDTO: Rpertoire dans lequel se trouvent les raccourcis Envoyer vers
CSIDL_TEMPLATES: Rpertoire contenant les modles de documents de Windows.
CSIDL_BITBUCKET: Recycle bin file system directory containing file objects
   in the user's recycle bin. The location of this directory is not in the
   registry; it is marked with the hidden and system attributes to prevent the
   user from moving or deleting it.
CSIDL_CONTROLS: Control Panel virtual folder containing icons for the control panel applications.
CSIDL_DRIVES: My Computer virtual folder containing everything on the local
   computer: storage devices, printers, and Control Panel. The folder may also
   contain mapped network drives.
}



end.

