// gauss : include de gti2_osc_graphe

const
   degre = 1;
   ordre = degre+1;

type
   matrice = array[1..ordre,1..ordre+1] of double;
   matriceLigne = array[1..ordre] of double;
   EsystemeLineaireError = class(Exception);
   vectorPuiss = array[0..2*degre] of double;

Procedure Resolution(A : matrice;var Result : matriceLigne);
 { Cette procdure rsoud un systme AX = B }
 { crit sous la forme AX = A(n+1) }
 { par la mthode d'limination de Gauss (pivot partiel) }

  Procedure Elimination;

    FUNCTION Pivot(i :integer):integer;
    Var n,temp : integer;
    BEGIN
      temp := i;
      for n := i+1 to ordre do
          if abs(A[n,i]) > abs(A[i,i]) then temp:= n;
      pivot:=temp;
      if abs(A[temp,i]) <= 1e-100 then
         raise EsystemeLineaireError.Create('Erreur rsolution systme linaire')
    END;

    PROCEDURE Permute(i,j:integer);
    Var temp:double;
        k:integer;
    BEGIN
       if i<>j then For k:=i to ordre+1 do BEGIN
          temp:=A[i,k];
          A[i,k]:=A[j,k];
          A[j,k]:=temp;
       END;
    END;

  var I,J,K : integer;
      M : double;
  begin
    for k := 1 to ordre do begin
          Permute(k,pivot(k));
          for i:=k+1 to ordre do begin
              M := A[i,k]/A[k,k];
              for j:=k+1 to ordre+1 do A[i,j]:=A[i,j]-M*A[k,j];
              A[i,k]:=0.0;
          end;
       end;
    end; {Elimination}

    Procedure Substitution;
    var I,J : integer;
        S : double;
    begin
         try
         for I:=ordre downto 1 do begin
             S:=A[I,ordre+1];
             for J:=I+1 to ordre do S:=S-A[I,J]*Result[J];
             Result[I]:=S/A[I,I];
         end;
         except
            on EzeroDivide do raise EsystemeLineaireError.Create('Erreur rsolution systme linaire')
         end;
    end;

begin {Resolution}
  Elimination;
  Substitution;
end; {rsolution}

{
procedure DeriveeGauss(const Yr : vecteur;N : integer;var Yder : vecteur);
var puissX : vectorPuiss;
    A : matrice;
    Coeff : matriceLigne;
    ii,puissMax : byte;
    NbrePointDer : integer;

procedure initXY;
var j : integer;
    loc : double;
    i : byte;
begin
    for i := 1 to puissMax do puissX[i] := 0;
    puissX[0] := NbrePointDer;
    for i := 1 to ordre do A[i,ordre+1] := 0;
    for j := 0 to pred(NbrePointDer) do begin
        loc := j;
        for i := 1 to puissMax do begin
            puissX[i] := puissX[i]+loc;
            loc := loc*j;
        end;
        loc := yr[j];
        for i := 1 to ordre do begin
            A[i,ordre+1] := A[i,ordre+1]+loc;
            loc := loc*j;
        end;
    end;
end;

Procedure CalculXY(i : integer);
var ajoute,retire : double;
    k : byte;
begin
        ajoute := i+ii;
        retire := i-ii-1;
        for k := 1 to puissMax do begin
            puissX[k] := puissX[k]+ajoute-retire;
            retire := retire*(i-ii-1);
            ajoute := ajoute*(i+ii);
        end;
        ajoute := Yr[i+ii];
        retire := Yr[i-ii-1];
        for k := 1 to ordre do begin
            A[k,ordre+1] := A[k,ordre+1]+ajoute-retire;
            retire := retire*(i-ii-1);
            ajoute := ajoute*(i+ii);
        end;
end;

procedure AffecteCoeff;
var i,j : byte;
begin
     for i := 1 to ordre do
         for j := 1 to ordre do
             A[i,j] := PuissX[i+j-2];
     try
     Resolution(A,Coeff);
     except
         coeff[2] := sansSignif;
//         coeff[3] := 0;
     end;
end;

Procedure AffecteDerivee(i : integer);
begin
   case degre of
        1 : yder[i] := coeff[2];
//        2 : yder[i] := coeff[2]+2*coeff[3]*x[i];
   end;
end;

var i : integer;
begin // DeriveeGauss 
    NbrePointDer := 2*(N div 64)+1;
    if NbrePointDer<5 then NbrePointDer := 5;
    if NbrePointDer>15 then NbrePointDer := 15;    
    for i := 0 to pred(N) do yder[i] := sansSignif;
    if N<NbrePointDer then exit;
    puissMax := 2*degre;
    ii := NbrePointDer div 2;
    initXY;
    affecteCoeff;
    for i := 0 to ii do affecteDerivee(i);
    for i := ii+1 to N-ii-1 do begin
        calculXY(i);
        affecteCoeff;
        affecteDerivee(i);
    end;
    for i := N-ii to pred(N) do affecteDerivee(i);
end; // DeriveeGauss
}

function ChercheDroite(const Yr : vecteur;debut,fin : integer;var origine,pente : double) : boolean;
var puissX : vectorPuiss;
    A : matrice;
    Coeff : matriceLigne;
    puissMax : byte;

procedure initXY;
var j : integer;
    loc : double;
    i : byte;
begin
    for i := 1 to puissMax do puissX[i] := 0;
    puissX[0] := (fin-debut+1);
    for i := 1 to ordre do A[i,ordre+1] := 0;
    for j := debut to fin do begin
        loc := j;
        for i := 1 to puissMax do begin
            puissX[i] := puissX[i]+loc;
            loc := loc*j;
        end;
        loc := yr[j];
        for i := 1 to ordre do begin
            A[i,ordre+1] := A[i,ordre+1]+loc;
            loc := loc*j;
        end;
    end;
end;

procedure AffecteCoeff;
var i,j : byte;
begin
     for i := 1 to ordre do
         for j := 1 to ordre do
             A[i,j] := PuissX[i+j-2];
     try
     Resolution(A,Coeff);
     except
         coeff[2] := 0;
         coeff[1] := 0;
     end;
end;

Function CalcCorrelation : boolean;
var i,nbre : integer;
    xsigma,ysigma,xmoy,ymoy : double;
    Sx,Sy,Sx2,Sy2,Sxx,Syy : double;
    cor : double;
begin
    result := false;
    try
    Sx := 0;Sy := 0;
    Sx2 := 0;Sy2 := 0;
    Nbre := succ(Fin-Debut);
    for  i := Debut to Fin do begin
       Sx := Sx+i;Sy := Sy+Yr[i];
       Sx2 := Sx2+i*i;Sy2 := Sy2+sqr(Yr[i]);
    end;
    Sxx := sx2-sqr(sx)/Nbre;
    Syy := sy2-sqr(sy)/Nbre;
    Xsigma := sqrt(Sxx/pred(Nbre));
    Ysigma := sqrt(Syy/pred(Nbre));
    Xmoy := Sx/Nbre;Ymoy := Sy/Nbre;
    Cor := 0;
    for i := Debut to Fin do
      Cor := Cor+(i-Xmoy)*(Yr[i]-Ymoy);
    Cor := Cor/pred(Nbre);
    Cor := Cor/Xsigma/Ysigma;
    result := abs(Cor)>0.9;
    except
    end;
end; { Calcul stat deux variables }

begin // ChercheDroite
    if fin<debut+5 then fin := debut+5;
    if fin>debut+15 then fin := debut+15;
    puissMax := 2*degre;
    initXY;
    affecteCoeff;
    pente := coeff[2];
    origine := coeff[1];
    result := calcCorrelation;
end; // ChercheDroite


