unit gti2_osc;

interface
uses Windows, Messages, Forms, sysUtils, registry,
     gti2_dll, gti2_osc_util, math, maths_osc, oomisc, inifiles;

const
    deltaVgti2 = 30/16384;  // +-15 V sur 14 bits
    deltaVsynchro = 30/4096;  // +-15 V sur 14 bits
    NbrePreAcqGTI2 = 550; // 500 plus marge d'erreur de 10%
    FreqMaxGBF = 990000;
    FreqMinGBF = 0.02;
    DeltatOrphy = 10e-6; // 10 s
    RetardGTS2 = 6e-6; // retard entre synchro GTI2 et GTS2
    DeltaVoltGTS2 = 10/32768;
    longueurReco = 12;

type
    TConnectique = (coNone,coDIN,coExtension);
    pseudoWord = record case byte of
             0 : (octets : array[0..longueurReco] of char);
             1 : (mot : word);
             2 : (entier : smallInt);
             3 : (longMot : longWord);
             4 : (octet : byte);
             5 : (index : array[indiceVoieYD] of byte;
                  valeur : array[indiceVoieYD] of smallInt);
    end;

    entierGTS2 = record case byte of
             0 : (carac : array[0..1] of char);
             1 : (entier : smallInt);
    end;

    PointYD = array[indiceVoieYD] of double;
    PointHardYD = array[indiceVoieYD] of integer;

    indiceVoieGTS2 = 0..11;
    TbaseDeTemps = (b10mu,b20mu,b50mu,b100mu,b200mu,
                    b500mu,b1m,b2m,b5m,b10m,
                    b20m,b50m,b100m,b200m,b500m,
                    b1,b2,b5,b10,b20,b50,b100,
                    b200,b10min,b20min,b1h);

    TGTI2 = class(Tobject)
          private
              DataGet : Tdata_RS232;
              FhandleGTI2 : Thandle;
              FThreadAcqLente : Thandle;
              FnbreCourant : integer;
              FnbreVoies : integer;
              FdataStructure : Data_Acquisition;

              FdataGenerateur : Data_Generateur;
              FloopGBF : boolean;
              Fstop : boolean;              
              FreloadGBF : integer;
              FfreqGBF : double;
              FfreqEchGBF : double;
              FhorlogeGBF : indiceHorloge;
              FNbreGBF : integer;
              FversionDLL: shortString;
              
              FvaleurVoie : array[indiceVoieYD] of vecteurWord;
              FoffsetVoie : array[indiceVoieYD,boolean,boolean] of integer; // diffrentiel puis 15V
              FdeltaVoie : array[indiceVoieYD] of double;
              Factive : array[indiceVoieYD] of boolean;
              Fdifferentiel : array[indiceVoieYD] of boolean;
              FbaseDeTemps : TbaseDeTemps;
              FnbrePoints : integer;
              FOffsetSeuil : array[boolean] of double;
   // valeur pour passer de seuil demand  seuil de la SA
   // valeurSA=seuil+offsetSeuil[montant]
              AcquisitionEnCours : boolean;
  // pour GTS2
              FnbreGTS2 : integer;
              FnbreVoiesGTS2 : integer;
              FnbreOctetsGTS2 : integer;
              FvaleurGTS2 : vecteurGTS2;
              FactiveGTS2 : array[indiceVoieGTS2] of boolean;
              FordreGTS2 : shortString;
              FdeltatGTS2 : double;
              FcoeffInterp : array[indiceVoieGTS2] of double;
              FindexGTS2 : integer;
              GTS2_a_initialiser : boolean;
              valeurPointHard : pointHardYD;
              procedure setSynchro(asynchro : integer);
              function getSynchro : integer;
              function getNbreTotal : integer;
              Procedure setSeuilVolt(y : double);
              procedure setFreqGBF(afreq : double);
              procedure setFreqEchGBF(afreq : double);
              function getSeuilVolt : double;
              procedure setCalibre15V(v : indiceVoieYD;is15V : boolean);
              function getCalibre15V(v : indiceVoieYD) : boolean;
              procedure setActive(avoie : indiceVoieYD;active : boolean);
              function getActive(avoie : indiceVoieYD) : boolean;
              procedure setActiveGTS2(avoie : indiceVoieGTS2;active : boolean);
              function getActiveGTS2(avoie : indiceVoieGTS2) : boolean;
              function getOffsetVoie(avoie : indiceVoieYD) : integer;
              function getOffsetSeuil(montant : boolean) : double;
              procedure setEtalonnage(active : boolean);
              function getEtalonnage : boolean;
              procedure setNbrePoints(n : integer);
              procedure setDifferentiel(avoie : indiceVoieYD;diff : boolean);
              function getDifferentiel(avoie : indiceVoieYD) : boolean;
              function getAcqTerminee : boolean;
              function getAcqGTS2Terminee : boolean;
              procedure setMontant(isMontant : boolean);
              procedure setOffsetSeuil(montant : boolean;valeur : double);
              function getMontant : boolean;
              function getFreqEch : integer;
              function getVersion : string;
              procedure setBaseDeTemps(abase : TbaseDeTemps);
              procedure calculBaseDeTemps;
              procedure regleTimerGTS2;
              function getNbreCourantGTS2 : integer;
              procedure send(commande : shortString);
              Procedure flushInBuffer;
              function AcquisitionLente : integer;
          protected
              CodeFreq : byte;
              CodeReload : integer;
              CodeUnSurN : integer;
          public
              valeurPoint : pointYD;
              connectique : Tconnectique;
              valeurReco : pseudoWord;
              OffsetSeuilM,OffsetSeuilD : double;
              gts2permis : boolean;
   // valeur pour passer de seuil demand  seuil de la SA
   // valeurSA=seuil+offsetSeuil[montant]
              property output : shortString write send;
              property acqTerminee : boolean read getAcqTerminee;
              property acqGTS2Terminee : boolean read getAcqGTS2Terminee;              
              property SeuilVolt : double read getSeuilVolt write setSeuilVolt;
              property VoieSynchro : integer read getSynchro write setSynchro;
              property NbreGBF : integer read FNbreGBF write FNbreGBF;
              property NbreCourant : integer read FnbreCourant;
              property NbrePoints : integer read FNbrePoints write setNbrePoints;
              property NbreCourantGTS2 : integer read getNbreCourantGTS2;
              property NbreTotal : integer read getNbreTotal;
              property NbreGTS2 : integer read FnbreGTS2;
              property NbreVoiesGTS2 : integer read FnbreVoiesGTS2;
              property FreqEch : integer read getFreqEch;
              property montant : boolean read getMontant write setMontant;
              property active[voie : indiceVoieYD] : boolean read getActive write setActive;
              property activeGTS2[voie : indiceVoieGTS2] : boolean read getActiveGTS2 write setActiveGTS2;
              property offsetVoie[voie : indiceVoieYD] : integer read getOffsetVoie;
              property calibre15V[voie : indiceVoieYD] : boolean read getCalibre15V write setCalibre15V;
              property offsetSeuil[montant : boolean] : double read getOffsetSeuil write setOffsetSeuil;
              property differentiel[voie : indiceVoieYD] : boolean read getDifferentiel write setDifferentiel;
              property baseDeTemps : TbaseDeTemps read FbaseDeTemps write setBaseDetemps;
              property BoucleGenerateur : boolean read FloopGBF write FloopGBF;
              property FreqGBF : double read FFreqGBF write setFreqGBF;
              property FreqEchGBF : double read FFreqEchGBF write setFreqEchGBF;
              property version : string read getVersion;
              property versionDLL : shortString read FVersionDLL;
              property etalonnage : boolean read getEtalonnage write setEtalonnage;
              function isOpen : boolean;
              function Open : boolean;
              function OpenGTS2 : boolean;
              procedure getReco;
              procedure Close;
              procedure CloseGTS2;
              procedure store;
              procedure load;
              function LanceAcquisition : boolean;
              procedure stopAcquisition;
              procedure EtalonneZero(voie : indiceVoieYD;diff : boolean;var signal : boolean);
              procedure resetOffset;
              procedure LanceGenerateur(const data : vecteurGBF);
              procedure StopGenerateur;
              procedure GetValeurTemps(var valeurT : vecteur);
              procedure GetValeurVoie(v : indiceVoieYD;var valeur : vecteur);
              procedure GetValeurTempsGTS2(var valeurT : vecteur);
              procedure GetValeurVoieGTS2(v : indiceVoieGTS2;var valeur : vecteur);
              function GetValeurInit(v : indiceVoieYD) : double;
              function getResul : boolean;
              function getResulGTS2 : boolean;
              function getInt(commande : shortString) : integer;
              procedure LanceAcqLente;
              procedure StopAcqLente;
              
              constructor Create;
              destructor Destroy;override;
          end;

var GTI2 : TGTI2;

const
  FreqEchOsc : array[0..2] of extended = (6e6,2e6,10e6);
  BaseDeTempsStr : array[TbaseDeTemps] of string[6] =
            ('10 s','20 s','50 s','100 s','200 s',
             '500 s','1 ms','2 ms','5 ms','10 ms',
             '20 ms','50 ms','100 ms','200 ms','500 ms',
             '1 s','2 s',' 5 s','10 s','20 s','50 s','100 s',
             '200 s','10 min','20 min','1 h');
  DureeBalayage : array[TbaseDeTemps] of double =
            (1e-5,2e-5,5e-5,1e-4,2e-4,
             5e-4,1e-3,2e-3,5e-3,1e-2,
             2e-2,5e-2,1e-1,2e-1,5e-1,
             1,2,5,10,20,50,100,
             200,600,1200,3600);
  NombreMaxi : array[TbaseDeTemps] of integer =
            (150,300,900,1500,3000,
             5000,5000,10000,10000,10000,
             15000,15000,15000,15000,15000,             
             15000,15000,15000,15000,15000,15000,15000,
             15000,15000,15000,15000);

TitreVoieY : array[boolean,indiceVoieYD] of string[3] =
(('Y0','Y1','Y2','Y3'),('YD0','YD1','YD2','YD3')); // boolean = diffrentiel

TitreVoieGTI2 : array[Tconnectique,0..11] of string[16] = (
(
'EA0',
'EA1',
'EA2',
'EA3',
'EA4, G',
'EA5, H',
'EA6, I',
'EA7, J',
'EAD0, G',
'EAD1, H',
'EAD2, I',
'EAD3, J'),
(
'EA0, A',
'EA1, A',
'EA2, F',
'EA3, F',
'EA4, B ou G',
'EA5, B ou H',
'EA6, C ou I',
'EA7, C ou J',
'EAD0, G',
'EAD1, H',
'EAD2, I',
'EAD3, J'),
(
'EA0',
'EA1',
'EA2',
'EA3',
'EA4 BNC ou G',
'EA5 BNC ou H',
'EA6 BNC ou I',
'EA7 BNC ou J',
'EAD0, G',
'EAD1, H',
'EAD2, I',
'EAD3, J')
);

AideVoieGTI2 : array[Tconnectique,0..11] of string[32] = (
(
'EA0',
'EA1',
'EA2',
'EA3',
'EA4 sur prise G',
'EA5 sur prise H',
'EA6 sur prise I',
'EA7 sur prise J',
'EAD0 diffrentielle sur prise G',
'EAD1 diffrentielle sur prise H',
'EAD2 diffrentielle sur prise I',
'EAD3 diffrentielle sur prise J'),
(
'EA0 sur prise A',
'EA1 sur prise A',
'EA2 sur prise F',
'EA3 sur prise F',
'EA4 sur prise B ou G',
'EA5 sur prise B ou H',
'EA6 sur prise C ou I',
'EA7 sur prise C ou J',
'EAD0 diffrentielle sur prise G',
'EAD1 diffrentielle sur prise H',
'EAD2 diffrentielle sur prise I',
'EAD3 diffrentielle sur prise J'),
(
'EA0',
'EA1',
'EA2',
'EA3',
'EA4 BNC ou prise G',
'EA5 BNC ou prise H',
'EA6 BNC ou prise I',
'EA7 BNC ou prise J',
'EAD0 diffrentielle sur prise G',
'EAD1 diffrentielle sur prise H',
'EAD2 diffrentielle sur prise I',
'EAD3 diffrentielle sur prise J')
);

implementation

function ThreadAcqLente : integer;stdcall;
begin
    result := gti2.AcquisitionLente
end;

function TGTI2.isOpen : boolean;
begin
    result := FhandleGTI2>0 
end;

function TGTI2.Open : boolean;

procedure setGts2Permis;
var version : array[0..255] of char;
    versionStr : shortString;
    debut,fin,posV : integer;
    numero : double;
begin
          gts2permis := false;
          _GetVersion(FhandleGTI2,version);
          versionStr := string(version)+' ';
          posV := pos('V',versionStr);
          debut := posV;
          repeat inc(debut)
          until (versionStr[debut] in ['0'..'9','.']);
          fin := debut;
          repeat inc(fin)
          until not (versionStr[fin] in ['0'..'9','.']);
          numero := strToFloatWin(copy(versionStr,debut,fin-debut));
          gts2permis := numero>1.055;
          FversionDLL := donneVersion('GTICOMMX.DLL');
          gts2permis := gts2permis and (FversionDLL[1]>='2');
end;

begin
     if FhandleGTI2<=0 then begin
        FHandleGTI2 := _SetPortOpen('OrphyGTI2-0');
        sleep(100);
     end;
     if FhandleGTI2<=0 then begin
        FHandleGTI2 := _SetPortOpen('OrphyGTI2-1');
        sleep(100);
     end;
     result := FhandleGTI2>0;
     if result then setGTS2permis else gts2permis := false;
end;

function TGTI2.OpenGTS2 : boolean;
begin
     result := (FhandleGTI2>0) and (_isRunThread=1);
     if result then exit;
     if FhandleGTI2>0 then begin
          _LanceThread(FHandleGTI2);
          sleep(50);
          result := _isRunThread=1;
     end;
     if result and gts2_a_initialiser then begin
        send('WRESET');
        sleep(1000);
        send('WBIN'); // car reset passe en ASCII
        sleep(10);
        gts2_a_initialiser := false;
     end;
end;

procedure TGTI2.Close;
begin
     stopAcquisition;
     if FhandleGTI2>0 then begin
        _SetPortClose(FHandleGTI2);
        sleep(100);
     end;
     FhandleGTI2 := 0;
end;

procedure TGTI2.CloseGTS2;
begin
     if (FhandleGTI2>0) and (_isRunThread=1) then begin
          _StopThread;
          sleep(50);
     end;
end;

function TGTI2.GetVersion : string;
var version : array[0..255] of char;
begin
   if isOpen
      then begin
          _GetVersion(FhandleGTI2,version);
          result := string(version);
      end
      else result := 'GTI2 non branche ?'
end;

constructor TGTI2.Create;
var voie : indiceVoieYD;
    k1,k2 : boolean;
begin
  inherited create;
  gts2_a_initialiser := true;
  FhandleGTI2 := 0;
  FThreadAcqLente := 0;
  FDataStructure.Trig.Valeur := 2048;
  FdataStructure.calibrage := 1;
  FnbreVoies := 1;
  FnbreGBF := 1024;
  with FdataGenerateur do begin
       NbreData := 480;
       BaseDeTemps := 0; // 100 kHz
       Reload := 0;
  end;
  for voie := low(indiceVoieYD) to high(indiceVoieYD) do begin
      FdeltaVoie[voie] := deltaVgti2;
      for k1 := false to true do
          for k2 := false to true do
              FoffsetVoie[voie,k1,k2] := 0;
  end;
  FnbrePoints := 1000;
end;

destructor TGTI2.Destroy;
begin
    close;
    inherited destroy;
end;

function TGTI2.GetResul : boolean;
var IncNbre,i : LongInt;
//    imax : LongInt;
    DataChannel : data_gti;
    v : indiceVoieYD;
begin
        result := false;
        try
        IncNbre := _GetUSBdata(FhandleGTI2,DataChannel,FDataStructure);
        IncNbre := IncNbre div FNbreVoies;
        if IncNbre>0 then begin
           if (FnbreCourant+incNbre)>maxVecteur then IncNbre := maxVecteur-FnbreCourant;
           for v := low(indiceVoieYD) to high(indiceVoieYD) do if Factive[v] then begin
//               imax := DataChannel.DataLen[v];
               for i := 0 to pred(incNbre) do
                   FvaleurVoie[v,FNbreCourant+i] := DataChannel.BufferIn[v,i];
           end;
           FNbreCourant := FNbreCourant + IncNbre;
           result := true;
        end;
        except
        end;
end;

function TGTI2.GetResulGTS2 : boolean;
var i : LongInt;
    longueur : integer;
begin
        result := FindexGTS2<FnbreOctetsGTS2;
        if not result then exit;
        result := (FhandleGTI2>0) and (_isRunThread=1);
        try
        if not result then exit;
        longueur := _GetLenH8;
        result := longueur>0;
        if result then begin
           if longueur>MaxDataGet then longueur := MaxDataGet;
           longueur := 2*(longueur div 2);
           longueur := _GetH8(dataGet,longueur,0);
           for i := 0 to pred(longueur) do begin
               FvaleurGTS2[FindexGTS2] := dataGet.bufferC[i];
               inc(FindexGTS2);
           end;
        end;
        if (FindexGTS2>=FnbreOctetsGTS2) then closeGTS2;
        except
           closeGTS2;
           result := false;
        end;
end;

procedure TGTI2.StopAcquisition;
begin
     if not isOpen then exit;
     if (_isRunThread=1) then begin                
         send('WPE');
         flushInBuffer;
         closeGTS2;
     end;
     if AcquisitionEnCours then begin
     _StopAcqui(FhandleGTI2);
     sleep(100);
     FnbreCourant := 0;
     AcquisitionEnCours := false;
     end;
end;

Function TGTI2.LanceAcquisition : boolean;
var v : indiceVoieYD;
    v2 : indiceVoieGTS2;
begin
        result := FhandleGTI2>0;
        if not result then exit;
        stopAcquisition;
        if gts2permis then GetReco;
        FnbreVoies := 0;
        FNbreCourant := 0;
        for v := low(indiceVoieYD) to high(indiceVoieYD) do
            if Factive[v] then begin
               inc(FnbreVoies);
               FDataStructure.Voie[v].Differentiel := ord(Fdifferentiel[v]);
               FDataStructure.Voie[v].On := 1;
            end
            else FDataStructure.Voie[v].On := 0;
        FnbreVoiesGTS2 := 0;
        if gts2permis then
            for v2 := low(indiceVoieGTS2) to high(indiceVoieGTS2) do
                if FactiveGTS2[v2] then inc(FnbreVoiesGTS2);
        calculBaseDeTemps;
        if FnbreVoiesGTS2>0 then RegleTimerGTS2;
        With FDataStructure.config do begin
             BaseDeTemps := CodeFreq;
             Reload := CodeReload;
             UnSurN := CodeUnSurN;
        end;
        if gts2permis and
           (FnbreVoiesGTS2>0) and
           openGTS2 then begin // d'abord GTS2
           send(FordreGTS2);
           if FDataStructure.Trig.TrigOn=0
              then send('WGOR') // SR quand mme
              else send('WGOSR');
           FindexGTS2 := 0;
        end;
        AcquisitionEnCours := _LanceAcqui(FhandleGTI2,FDataStructure)=1;
        FnbreCourant := 0;
        result := AcquisitionEnCours;
end;

Procedure TGTI2.setSynchro(asynchro : integer);
begin
     FDataStructure.Trig.TrigOn := aSynchro;
// presynchro systmatique si synchro seuil
     if asynchro>0
{
// pas de presynchro si GTS2  ?
      then if GTS2permis and (FnbreVoiesGTS2>0)
             then FDataStructure.Trig.preTrigOn := 0
}
             then FDataStructure.Trig.preTrigOn := 1
        else FDataStructure.Trig.preTrigOn := 0
end;

Function TGTI2.getSynchro : integer;
begin
     result := FDataStructure.Trig.TrigOn
end;

Procedure TGTI2.setSeuilVolt(y : double);
var seuilInt : integer;
begin // 2048 correspond  0V, 0 correspond  -15V et 4095 correspond  +15V
      seuilInt := round(4096*(y+FoffsetSeuil[montant]+15)/30);
      if (seuilInt>96) and (seuilInt<4000) then
         FDataStructure.Trig.Valeur := seuilInt
end;

Function TGTI2.getSeuilVolt : double;
var seuilInt : word;
begin
     seuilInt := FDataStructure.Trig.Valeur;
     result := seuilInt*deltaVsynchro-15-FoffsetSeuil[montant]
end;

procedure TGTI2.setBaseDeTemps(abase : TbaseDeTemps);
begin
    FbaseDeTemps := abase;
    calculBaseDeTemps;
    With FDataStructure.config do begin
         BaseDeTemps := CodeFreq;
         Reload := CodeReload;
         UnSurN := CodeUnSurN;
    end;
end;

Procedure TGTI2.setMontant(isMontant : boolean);
begin
     FDataStructure.Trig.FrontMontant := ord(isMontant)
end;

Function TGTI2.getMontant : boolean;
begin
     result := FDataStructure.Trig.FrontMontant<>0
end;

procedure TGTI2.setActive(avoie : indiceVoieYD;active : boolean);
var voie : indiceVoieYD;
begin
     if active and not(Factive[aVoie]) then FdataStructure.calibrage := 1;     
     Factive[aVoie] := active;
     FDataStructure.Voie[aVoie].On := ord(active);
     FnbreVoies := 0;
     for voie := low(indiceVoieYD) to high(indiceVoieYD) do
         if Factive[voie] then inc(FnbreVoies);
end;

function TGTI2.getActive(avoie : indiceVoieYD) : boolean;
begin
     result := Factive[aVoie]
end;

procedure TGTI2.setActiveGTS2(avoie : indiceVoieGTS2;active : boolean);
var voie : indiceVoieGTS2;
begin
     FactiveGTS2[aVoie] := active;
     FnbreVoiesGTS2 := 0;
     for voie := low(indiceVoieGTS2) to high(indiceVoieGTS2) do
         if FactiveGTS2[voie] then inc(FnbreVoiesGTS2);
end;

function TGTI2.getActiveGTS2(avoie : indiceVoieGTS2) : boolean;
begin
     result := FactiveGTS2[aVoie]
end;

function TGTI2.getOffsetVoie(avoie : indiceVoieYD) : integer;
begin
     result := FoffsetVoie[avoie,Fdifferentiel[avoie],calibre15V[avoie]]
end;

procedure TGTI2.setDifferentiel(avoie : indiceVoieYD;diff : boolean);
begin
     Fdifferentiel[avoie] := diff;
     FDataStructure.Voie[avoie].Differentiel := ord(diff);
     if Factive[aVoie] then FdataStructure.calibrage := 1;
end;

function TGTI2.getDifferentiel(avoie : indiceVoieYD) : boolean;
begin
     result := Fdifferentiel[avoie]
end;

procedure TGTI2.LanceGenerateur(const data : vecteurGBF);
var i : integer;
begin
    if not isOpen then exit;
    FdataGenerateur.NbreData := FnbreGBF;
    FdataGenerateur.BaseDeTemps := FhorlogeGBF;
    FdataGenerateur.Reload := FreloadGBF;
    for i := 0 to pred(NbreMaxGBF) do
        FdataGenerateur.bufferOut[i] := data[i];
    _LanceGenerateur(FhandleGTI2,ord(FloopGBF),FdataGenerateur);
end;

procedure TGTI2.StopGenerateur;
begin
    if FhandleGTI2>0 then _StopGenerateur(FhandleGTI2)
end;

procedure TGTI2.setFreqGBF(afreq : double);
var Nbre : longInt;
begin
    if afreq<FreqMinGBF then afreq := FreqMinGBF;
    if afreq>FreqMaxGBF then afreq := FreqMaxGBF;
    if afreq>500 then begin // on peut essayer sans dbordement d'entier
       FReloadGBF := 0;
       FhorlogeGBF := 0;         // 48 MHz
       Nbre := round(freqHorloge[0]/afreq);
       if Nbre<NbreMaxGBF then begin
         FNbreGBF := Nbre;
         FfreqGBF := freqHorloge[0] div FNbreGBF;
         FfreqEchGBF := freqHorloge[0];
         exit;    // 3 kHz  48 MHz
       end;
       FreloadGBF := 65536-1;
       FhorlogeGBF := 1; // 12 MHz
       Nbre := round(freqHorloge[1]/afreq);
       if Nbre<NbreMaxGBF then begin
          FNbreGBF := Nbre;
          FfreqGBF := freqHorloge[1] div FNbreGBF;
          FfreqEchGBF := freqHorloge[1];
          exit; // 750 Hz  3 kHz
       end;
   end;
   FhorlogeGBF := 2;
   Nbre := round(freqHorloge[2]/afreq);// 4 MHz
   if Nbre<NbreMaxGBF then begin
      FNbreGBF := Nbre;
      FfreqGBF := freqHorloge[2] div FNbreGBF;
      FfreqEchGBF := freqHorloge[2];
      exit; // 250 Hz  750 Hz
   end;
   FReloadGBF := (Nbre div NbreMaxGBF) + 1;
   FNbreGBF := Nbre div FreloadGBF;
   FfreqGBF := freqHorloge[FhorlogeGBF]/FNbreGBF/FReloadGBF;
   FfreqEchGBF := FfreqGBF*FNbreGBF;
   FReloadGBF := 65536-FreloadGBF;
end;

procedure TGTI2.setFreqEchGBF(afreq : double);
const
   freqMax=12e6; // 12 MHz
   freqMin=100; // 4e6/65536=61 Hz
begin
   if afreq<freqMin then afreq := freqMin;
   if afreq>freqMax then afreq := freqMax;
   FhorlogeGBF := 1; // 12 MHz
   FReloadGBF := round(freqHorloge[FhorlogeGBF]/afreq);
   if FReloadGBF>32 then begin
      FhorlogeGBF := 2; // 4 MHz
      FReloadGBF := round(freqHorloge[FhorlogeGBF]/afreq);
   end;
   FfreqEchGBF := freqHorloge[FhorlogeGBF]/FReloadGBF;
   FfreqGBF := FfreqEchGBF/FNbreGBF;
   FReloadGBF := 65536-FreloadGBF;
end;

procedure TGTI2.GetValeurTemps(var valeurT : vecteur);
var dt : double;
    i : integer;
begin
    dt := 1/freqEch;
    for i := 0 to MaxVecteur do
        valeurT[i] := i*dt;
end;

procedure TGTI2.GetValeurTempsGTS2(var valeurT : vecteur);
var i : integer;
begin
    for i := 0 to MaxVecteur do
        valeurT[i] := i*FdeltatGTS2;
end;

procedure TGTI2.GetValeurVoie(v : indiceVoieYD;var valeur : vecteur);
var i,imax,k : integer;
    offset : double;
begin
    imax := nbreTotal;
    offset := offsetVoie[v];
    for i := 0 to imax do begin
        k := FvaleurVoie[v,i];
        if k>8192 then k := k-16384;
        valeur[i] := (k-offset)*FdeltaVoie[v];
    end;
    for i := succ(imax) to MaxVecteur do
        valeur[i] := 0;
end;

procedure TGTI2.GetValeurVoieGTS2(v : indiceVoieGTS2;var valeur : vecteur);

   procedure Decale;
   var
      j: integer;
      alphaLoc,sauveJ,oldValeur: double;
   begin
         AlphaLoc := FCoeffInterp[v];
         oldValeur := valeur[0];
         valeur[0] := oldValeur*(1 + alphaLoc) -
                      valeur[1]*alphaLoc; // extrapolation
         for j := 1 to pred(nbreGTS2) do begin // interpolation
            sauveJ := valeur[j];
            valeur[j] := oldValeur*alphaLoc +
                         valeur[j]*(1 - alphaLoc);
            oldValeur := sauveJ;
         end;
   end;

var i,j : integer;
    pas : integer;
    v2 : indiceVoieGTS2;
    valeurMot : entierGTS2;
begin
    j := 0;
    if v>low(indiceVoieGTS2) then // sinon le compilateur rale 
       for v2 := low(indiceVoieGTS2) to pred(v) do
           if FactiveGTS2[v2] then inc(j,2);
    pas := 2*FnbreVoiesGTS2;
    for i := 0 to pred(FnbreGTS2) do begin
        valeurMot.carac[0] := FvaleurGTS2[j];
        valeurMot.carac[1] := FvaleurGTS2[j+1];
        valeur[i] := valeurMot.entier*deltaVoltGTS2;
        inc(j,pas);
    end;
    decale;
    for i := FnbreGTS2 to MaxVecteur do
        valeur[i] := 0;
end;

function TGTI2.GetValeurInit(v : indiceVoieYD) : double;
var k : integer;
begin
    k := FvaleurVoie[v,0];
    if k>8192 then k := k-16384;
    result := (k-offsetVoie[v])*FdeltaVoie[v];
end;

function TGTI2.getNbreTotal : integer;
begin
   if FDataStructure.Trig.preTrigOn=1
      then result := FNbrePoints+NbrePreAcqGTI2
      else result := FNbrePoints
end;

function TGTI2.getAcqTerminee : boolean;
begin
    result := FnbreCourant>=NbreTotal
end;

function TGTI2.getAcqGTS2Terminee : boolean;
begin
    result := FindexGTS2>=FnbreOctetsGTS2
end;

procedure TGTI2.setCalibre15V(v : indiceVoieYD;is15V : boolean);
begin
    if is15V
       then begin
          if FdataStructure.voie[v].calibre=1 then begin
             FdataStructure.voie[v].calibre := 0;
             FdataStructure.calibrage := 1;        
             FdeltaVoie[v] := deltaVgti2;
          end;
       end
       else begin
          if FdataStructure.voie[v].calibre=0 then begin
              FdataStructure.voie[v].calibre := 1;
              FdataStructure.calibrage := 1;
              FdeltaVoie[v] := deltaVgti2/2;
          end;
       end;
end;

function TGTI2.getCalibre15V(v : indiceVoieYD) : boolean;
begin
   result := FdataStructure.voie[v].calibre=0
end;

function TGTI2.getFreqEch : integer;
begin
   calculBaseDeTemps;
   result := round(freqEchOsc[codeFreq]/
            ((256-CodeReload)*CodeUnSurN))
end;

procedure TGTI2.setEtalonnage(active : boolean);
begin
    FdataStructure.calibrage := ord(active)
end;

function TGTI2.getEtalonnage : boolean;
begin
    result := FdataStructure.calibrage=1
end;

function TGTI2.getOffsetSeuil(montant : boolean) : double;
begin
     result := FOffsetSeuil[montant]
end;

procedure TGTI2.setOffsetSeuil(montant : boolean;valeur : double);
begin
     if abs(valeur)<0.5 then FOffsetSeuil[montant] := valeur
end;

procedure TGTI2.EtalonneZero(voie : indiceVoieYD;diff : boolean;var signal : boolean);

Procedure Test(is15V : boolean);
const N = 1000;
      Nmax = 2000;
var i,k : integer;
    valeur : longInt;
begin
     FoffsetVoie[voie,diff,is15V] := 0;
     calibre15V[voie] := is15V;
     LanceAcquisition;
     sleep(100);
     repeat sleep(10)
     until not getResul or (FNbreCourant>Nmax);
     StopAcquisition;
     valeur := 0;
     for i := NbrePreAcqGTI2+1 to NbrePreAcqGTI2+N do begin
        k := FvaleurVoie[voie,i];
        if k>8192 then k := k-16384;
        valeur := valeur+k;
     end;
     valeur := valeur div N;
     signal := abs(valeur)>64;
     if not signal then FOffsetVoie[voie,diff,is15V] := valeur;
end;

var i : indiceVoieYD;
    sauveBaseDeTemps : TbaseDeTemps;
    sauve15V : boolean;
begin
     StopAcquisition;
     voieSynchro := 0; // pas de synchro
     Fdifferentiel[voie] := diff;
     for i := 0 to 3 do Factive[i] := false;
     Factive[voie] := true;
     sauveBaseDeTemps := FbaseDeTemps;
     sauve15V := Calibre15V[voie];
     FbaseDeTemps := b50m; //  50 ms
     test(false);
     test(true);
     FBaseDeTemps := sauveBaseDeTemps;
     Calibre15V[voie] := sauve15V;
end;

procedure TGTI2.store;
var Fichier : TRegistryIniFile;
    voie : indiceVoieYD;
    FichierIni : TMemIniFile;
begin
     try
     Fichier := TRegistryIniFile.create(RegistreGTI2);
     for voie := 0 to 3 do begin
         Fichier.WriteInteger('Voie'+intToStr(voie),
                              'offsetYD',FoffsetVoie[voie,true,false]);
         Fichier.WriteInteger('Voie'+intToStr(voie),
                              'offsetY',FoffsetVoie[voie,false,false]);
         Fichier.WriteInteger('Voie'+intToStr(voie),
                              'offsetYD15',FoffsetVoie[voie,true,true]);
         Fichier.WriteInteger('Voie'+intToStr(voie),
                              'offsetY15',FoffsetVoie[voie,false,true]);
     end;
     Fichier.WriteFloat('Synchro','offsetM',offsetSeuil[true]);
     Fichier.WriteFloat('Synchro','offsetD',offsetSeuil[false]);
     Fichier.WriteInteger('GTI2','Nbre',FnbrePoints);
     Fichier.free;
     except
     end;
     try
     FichierIni := TMemIniFile.create(FichierConfig);
     for voie := 0 to 3 do begin
         FichierIni.WriteInteger('Voie'+intToStr(voie),
                              'offsetYD',FoffsetVoie[voie,true,false]);
         FichierIni.WriteInteger('Voie'+intToStr(voie),
                              'offsetY',FoffsetVoie[voie,false,false]);
         FichierIni.WriteInteger('Voie'+intToStr(voie),
                              'offsetYD15',FoffsetVoie[voie,true,true]);
         FichierIni.WriteInteger('Voie'+intToStr(voie),
                              'offsetY15',FoffsetVoie[voie,false,true]);
     end;
     FichierIni.WriteFloat('Synchro','offsetM',offsetSeuil[true]);
     FichierIni.WriteFloat('Synchro','offsetD',offsetSeuil[false]);
     FichierIni.WriteInteger('GTI2','Nbre',FnbrePoints);
     FichierIni.updateFile;
     FichierIni.free;
     except
     end;
end;

procedure TGTI2.load;

procedure LoadIni;
var FichierIni : TMemIniFile;
    voie : indiceVoieYD;
begin
     try
     FichierIni := TMemIniFile.create(fichierConfig);
     for voie := 0 to 3 do begin
         FoffsetVoie[voie,true,false] := FichierIni.ReadInteger('Voie'+intToStr(voie),
                               'offsetYD',0);
         FoffsetVoie[voie,false,false] := FichierIni.ReadInteger('Voie'+intToStr(voie),
                               'offsetY',0);
         FoffsetVoie[voie,true,true] := FichierIni.ReadInteger('Voie'+intToStr(voie),
                               'offsetYD15',0);
         FoffsetVoie[voie,false,true] := FichierIni.ReadInteger('Voie'+intToStr(voie),
                               'offsetY15',0);
     end;
     offsetSeuil[false] := FichierIni.ReadFloat('Synchro','offsetM',0);
     offsetSeuil[true] := FichierIni.ReadFloat('Synchro','offsetD',0);
     FnbrePoints := FichierIni.ReadInteger('GTI2','Nbre',1000);
     fichierIni.free;
     except
     end;
end;

Procedure LoadReg;
var Fichier : TRegistryIniFile;
    voie : indiceVoieYD;
begin
     try
     Fichier := TRegistryIniFile.create(RegistreGTI2);
     for voie := 0 to 3 do begin
         FoffsetVoie[voie,true,false] := Fichier.ReadInteger('Voie'+intToStr(voie),
                               'offsetYD',0);
         FoffsetVoie[voie,false,false] := Fichier.ReadInteger('Voie'+intToStr(voie),
                               'offsetY',0);
         FoffsetVoie[voie,true,true] := Fichier.ReadInteger('Voie'+intToStr(voie),
                               'offsetYD15',0);
         FoffsetVoie[voie,false,true] := Fichier.ReadInteger('Voie'+intToStr(voie),
                               'offsetY15',0);
     end;
     offsetSeuil[false] := Fichier.ReadFloat('Synchro','offsetM',0);
     offsetSeuil[true] := Fichier.ReadFloat('Synchro','offsetD',0);
     FnbrePoints := Fichier.ReadInteger('GTI2','Nbre',1000);
     fichier.free;
     except
     end;
end;

begin
     if fileExists(fichierConfig)
        then LoadIni
        else LoadReg;
end;

procedure TGTI2.resetOffset;
var voie : indiceVoieYD;
    k1,k2 : boolean;
begin
    for voie := 0 to 3 do
        for k1 := false to true do
            for k2 := false to true do
                FoffsetVoie[voie,k1,k2] := 0;
end;

procedure TGTI2.setNbrePoints(N : integer);
begin
  if N>NombreMaxi[FbaseDeTemps] then N := NombreMaxi[FbaseDeTemps];
  if N<100 then N := 100; // soit 10 s  10 MHz 
  FnbrePoints := N;
  calculBaseDeTemps;
end;

procedure TGTI2.calculBaseDeTemps;
var freq : double;
begin
     if FnbrePoints>NombreMaxi[FbaseDeTemps] then FnbrePoints := NombreMaxi[FbaseDeTemps];
     Freq := FnbrePoints/dureeBalayage[FbaseDeTemps];
     CodeUnSurN := 1;
     CodeReload := 255;
     if freq>9.95e6 then begin // 10 MHz
        CodeFreq := 2;
        CodeUnSurN := 1;
        CodeReload := 255;
        FnbrePoints := trunc(FreqEchOsc[CodeFreq]*dureeBalayage[FbaseDeTemps]);
        if FnbrePoints>10000 then FnbrePoints := 10000;
        exit
     end;
     if freq>5.95e6 then begin // 6 MHz
        CodeFreq := 0;
        CodeUnSurN := 1;
        CodeReload := 255;
        FnbrePoints := trunc(FreqEchOsc[CodeFreq]*dureeBalayage[FbaseDeTemps]);
        if FnbrePoints>10000 then FnbrePoints := 10000;
        exit
     end;
     if freq>4.95e6 then begin // 5 MHz
        CodeFreq := 2;
        CodeReload := 255;
        CodeUnSurN := 2;
        FnbrePoints := trunc(FreqEchOsc[CodeFreq]*dureeBalayage[FbaseDeTemps]/2);
        if FnbrePoints>10000 then FnbrePoints := 10000;
        exit
     end;
     if freq>2.95e6 then begin // 3 MHz
        CodeFreq := 0;
        CodeReload := 255;
        CodeUnSurN := 2;
        FnbrePoints := trunc(FreqEchOsc[CodeFreq]*dureeBalayage[FbaseDeTemps]/2);
        if FnbrePoints>10000 then FnbrePoints := 10000;
        exit
     end;
     if freq>1.95e6 then begin // 2 MHz
        CodeFreq := 1;
        CodeUnSurN := 1;
        CodeReload := 255;
        FnbrePoints := trunc(FreqEchOsc[CodeFreq]*dureeBalayage[FbaseDeTemps]);
        if FnbrePoints>10000 then FnbrePoints := 10000;
        exit
     end;
     CodeFreq := 1; // 2 MHz
     CodeReload := 256-ceil(FreqEchOsc[CodeFreq]/freq);
     if codeReload>255 then codeReload := 255;
     if codeReload<56 then begin
        codeReload := 56;
        CodeUnSurN := ceil(FreqEchOsc[CodeFreq]/freq/(256-codeReload));
     end;
     FnbrePoints := trunc(FreqEchOsc[CodeFreq]*dureeBalayage[FbaseDeTemps]/(256-CodeReload)/CodeUnSurN);
end;

Procedure TGTI2.send(commande : shortString);
var i,longueur : integer;
    ET : eventTimer;
begin
    longueur := length(commande);
    for i := 0 to pred(longueur) do
        dataGet.bufferC[i] := commande[i+1];
    dataGet.buffer[longueur] := 13; // fin de chaine
    inc(longueur); 
    _SendH8(dataGet,longueur);
    sleep(longueur);
    newTimer(ET,2);
    repeat
        sleep(1)
    until (_emptyBuffer=0) or timerExpired(ET);
end;

Procedure TGTI2.getReco;
var ET : eventTimer;
    indexOctet,i,longueur : integer;
begin
    if not openGTS2 then exit;
    flushInBuffer;
    send('WRECO');
    newTimer(ET,9);
    for i := 0 to pred(longueurReco) do
        valeurReco.octets[i] := #0;
    indexOctet := 0;
    repeat
      longueur := _GetLenH8;
      if longueur>0 then begin
         longueur := _GetH8(dataGet,longueur,0);
         for i := 0 to pred(longueur) do begin
             valeurReco.octets[indexOctet] := dataGet.bufferC[i];
             inc(indexOctet);
         end;
      end;
   until (indexOctet=longueurReco) or timerExpired(ET);
   closeGTS2;
end;

function TGTI2.getInt(commande : shortString) : integer;
const
   attente = 5;
   count = 2;
var ET : eventTimer;
    indexOctet,i,longueur : integer;
    FvaleurMot : pseudoWord;
begin
    indexOctet := 0;
    send(commande);
    newTimer(ET,attente);
    FvaleurMot.entier := 0;
    repeat
      longueur := _GetLenH8;
      if longueur>0 then begin
         longueur := _GetH8(dataGet,longueur,0);
         for i := 0 to pred(longueur) do begin
             FvaleurMot.octets[indexOctet] := dataGet.bufferC[i];
             inc(indexOctet);
         end;
      end;
   until (indexOctet=count) or timerExpired(ET);
   result := FvaleurMot.entier;
end;

procedure Tgti2.RegleTimerGTS2;

function codeVoie(voie : indiceVoieGTS2) : shortString;
begin
    if FactiveGTS2[voie]
       then result := ' '+intToStr(voie)
       else result := ''
end;

const bmax = 65535;
      tmax = 1310;
var bGTS,tGTS : integer;
    deltatMin : double;
    coeffInterp,decalageVoie : double;
    voie : indiceVoieGTS2;
    nbreMax : integer;
begin // deltatGTS2 = t*b en s
     FNbreVoiesGTS2 := 0;
     for voie := low(indiceVoieGTS2) to high(indiceVoieGTS2) do
         if FactiveGTS2[voie] then inc(FNbreVoiesGTS2);
     if FnbreVoiesGTS2=0 then exit;
     FdeltatGTS2 := 1/freqEch;
     deltatMin := deltatOrphy*FNbreVoiesGTS2;
     if FdeltatGTS2<deltatMin then FdeltatGTS2 := deltatMin;
     FdeltatGTS2 := round(FdeltatGTS2/deltatOrphy); // en unit deltatOrphy=10 s
     bGTS := 1;
     tGTS := trunc(FdeltatGTS2);
     if tGTS<1 then tGTS := 1;
     if tGTS>tmax then begin
         tGTS := 100; // 1 ms
         bGTS := round(FdeltatGTS2/tGTS);
         if bGTS<=0 then bGTS := 1;
         if bGTS>bmax then bGTS := bmax;
     end;
     FdeltatGTS2 := tGTS*bGTS; // en unit deltatOrphy=10 s
     decalageVoie := 1/FdeltatGTS2; // entre voies GTS2
     FdeltatGTS2 := FdeltatGTS2*deltatOrphy; // en seconde
     coeffInterp := retardGTS2/FdeltatGTS2; // entre GTI2 et GTS2
     FnbreGTS2 := ceil(DureeBalayage[baseDeTemps]*1.05/FdeltatGTS2);
     nbreMax := maxVecteurGTS2 div FNbreVoiesGTS2;
     if FnbreGTS2>nbreMax then FnbreGTS2 := nbreMax;
     if FnbreGTS2>maxVecteur then FnbreGTS2 := maxVecteur;
     FordreGTS2 := 'WPE '+intToStr(FNbreVoiesGTS2)+' '+
                         intToStr(FnbreGTS2)+' '+
                         intToStr(tGTS*10)+' '+
                         intToStr(bGTS);
     for voie := low(indiceVoieGTS2) to high(indiceVoieGTS2) do
         if FactiveGTS2[voie] then begin
            FordreGTS2 := FordreGTS2+codeVoie(voie);
            FcoeffInterp[voie] := coeffInterp;
           coeffInterp := coeffInterp+decalageVoie;
         end;
     FnbreOctetsGTS2 := FNbreVoiesGTS2*FnbreGTS2*2;
end; // regleTimerGTS2

function Tgti2.getNbreCourantGTS2 : integer;
begin
     result := (FindexGTS2 div 2) div FNbreVoiesGTS2;
end;

Procedure TGTI2.flushInBuffer;
var ET : eventTimer;
    longueur : integer;
begin
    newTimer(ET,18);
    repeat
        sleep(20);
        longueur := _GetLenH8;
        if longueur>0 then begin
           if longueur>MaxDataGet then longueur := MaxDataGet;
           _GetH8(dataGet,longueur,0);
        end;
    until (longueur=0) or timerExpired(ET);
end;

function TGTI2.AcquisitionLente : integer;
var IncNbre : LongInt;
    voie : indiceVoieYD;
    DataChannel : data_gti;
begin
   result := 0;
   With FdataStructure.config do begin
       BaseDeTemps := 1; // 2 MHz
       Reload := 56; // divis par (256-56)=200 soit 10 kHz
       UnSurN := 1; // soit 10 kHz  soit dure totale 1,8 s 
   end;
   With FdataStructure.trig do begin
      TrigOn := 0; // immdiat
      PreTrigOn := 0;
      FrontMontant := 0;
      Valeur := 2048;
   end;
   Fstop := false;
   try
   repeat
      AcquisitionEnCours := _LanceAcqui(FhandleGTI2,FDataStructure)=1;
      FNbreCourant := 0;
      repeat
         IncNbre := _GetUSBdata(FhandleGTI2,DataChannel,FDataStructure);
         IncNbre := IncNbre div FnbreVoies;
         if IncNbre>0 then begin
            for voie := low(indiceVoieYD) to high(indiceVoieYD) do
                if Factive[voie] then
                   valeurPointHard[voie] := DataChannel.BufferIn[voie,incNbre-1];
            FNbreCourant := FNbreCourant + IncNbre;
         end; // incNbre>0
       until Fstop or (FnbreCourant>=MaxVecteur);
    until Fstop;
    for voie := low(indiceVoieYD) to high(indiceVoieYD) do
        if Factive[voie] then begin
           if valeurPointHard[voie]>8192 then valeurPointHard[voie] := valeurPointHard[voie]-16384;
           valeurPoint[voie] := (valeurPointHard[voie]-offsetVoie[voie])*FdeltaVoie[voie];
        end;
    except
    for voie := low(indiceVoieYD) to high(indiceVoieYD) do
        valeurPoint[voie] := 0;
    end;
    StopAcquisition;
end;

procedure TGTI2.LanceAcqLente;
var code : cardinal;
begin
   if FhandleGTI2>0 then begin
      try
      StopAcqLente;
      FThreadAcqLente := CreateThread(nil,0,@ThreadAcqLente,nil,0,code);
      except
      end;
   end;
end;

Procedure TGTI2.StopAcqlente;
var code : cardinal;
    acqEnCours : boolean;
    atimer : eventTimer;
begin
      if (FhandleGTI2>0) and (FThreadAcqLente<>0) then begin
         Fstop := true;
         newTimer(Atimer,18);
         repeat
              AcqEnCours := getExitCodeThread(FThreadAcqLente,code) and
                           (code=Still_active);
         until not AcqEnCours or timerExpired(Atimer);
//         if AcquisitionEnCours then stopAcquisition; // on force
         closeHandle(FThreadAcqLente);
         FThreadAcqLente := 0;
     end;
end;

end.

{
Code Olivier Mecacel

    Call LanceTHREAD(HandleFile)
    EnvoiCommande ('WBIN')
    EnvoiCommande ('WPE')
    EnvoiCommande ('WGOSR')
    Call GTI2.LanceAcquisitionGTI2 'lancement synchronis
    Maxi = (LONG_GTI2LENT + 1) * 2 * nbVoie2
    Debut = GetTickCount()
    strTAB = ""
    Do
      DoEvents
      strTAB = strTAB & RecoiDonne2(500)
    While (Len(strTAB) < Maxi) And ((GetTickCount - Debut) < 1500)
    
    If GetTickCount - Tempo2 < 1500 Then
      affectePoints
    endif

Je ralenti volontairement le systeme pour eviter tout chevauchement de donnes
et surtout car je voulais faire un code rapide pour tester. Ce code est
plus que probablement perfectible, surtout concernant le cadencement du
rapatriement.
 
Pour faire simple : 
 
Mode non synchronis
configure la voie rapide 
lance acquisition rapide
configure la voie lente
lance acquisition de la voie lente
recupere toutes les donnes de la voie lente
recupere au compte goutte les donnes de la voie rapide en affichant au fur
et  mesure

Mode Synchro +
configure la voie lente
lance acquisition de la voie lente
configure la voie rapide
lance acquisition rapide
recupere toutes les donnes de la voie lente
recupere au compte goutte les donnes de la voie rapide en affichant au fur
et  mesure

En fait si vous avez des donnes dans l'une ou l'autre des modules, c'est que
le trig est arriv. Le principe est que l'on met la partie lente, en mode
"stand by" et lorsque la partie rapide detecte une synchro, il donne l'ordre
 la partie lente de commencer l'acquisition.
Il y donc un petit dcalage temporel, que vous pourrez probablement corriger
graphiquement. A moins que cela ne se voit meme pas (pour des frequence
raisonnables)

}

