unit gti2_point_Graphe;

interface

uses Windows, SysUtils, Graphics, Classes,
   Forms, Controls, Buttons, StdCtrls, ExtCtrls, Dialogs,
   Math, maths_osc, UniteKer, ComCtrls,
   GTI2Util_osc, gti2_dll, gti2_osc_Const, gti2_osc;

type
   Tmotif = (mCroix, mCroixDiag, mCercle, mCarre, mLosange);

   TCourbe = class(Tunite)
   private
      numero:       integer;
      a, b:         double; { entier = rel*a+b }
      deltaAxe, exposant, pointDebut: double;
      { deltaAxe = cart entre graduations sur les axes }
      NbreChiffre:  integer;
      courbeBottom, courbeTop: integer;
      FMini, FMaxi: double;
   public
      active:      boolean;
      signif:      shortString;
      couleur:     Tcolor;
      Motif:       Tmotif;
      exposantAff: double;
      nbreAvantVirgule: integer;
      vecteurAff:  vecteur;
      valeurAff:   double;
      property Mini: double Read Fmini Write FMini;
      property Maxi: double Read Fmaxi Write FMaxi;
      constructor Create(Anom: shortString; Aunite: strUnite; Anumero: integer);
      procedure setDelta(grandAxe: boolean);
      procedure setMinMax;
      function FormatAxe(x: double): strUnite;
      function XWindow(r: double): integer;
      function XMonde(xi: integer): double;
   end;

   TCourbePoint = class(Tcourbe)
   private
   protected
   public
      diffRG:      TradioGroup;
      voieGTI2:    indiceVoieOsc;
      shape:       Tshape;
      labelValeur,labelCalibre: Tlabel;
      zeroCB:      TcheckBox;
      calibreUD:   TUpDown;
      GBvoie:      TgroupBox;
      function calibreVoie: strUnite;
      function AffVoltmetre: strUnite;
   end;

   TCourbeGTS2 = class(TcourbePoint)
   private
   protected
   public
      Offset, UnVolt:   double;
      indexCapteurAuto: integer;
      priseInitialisee: boolean;
      speedVoie:        TSpeedButton;
      EtalonnageFait:   boolean;
      avecCapteur : boolean;
      function AffVoltmetre: strUnite;
      function ValeurReal(V: integer): double;
      function ValeurVolt(V: integer): double;
      function VoltToReal(V: double): double;
      procedure setMinMax;
      constructor Create(Anom: shortString; Aunite: strUnite; Anumero: integer);
   end;


   TgraphePoint = class
      Canvas:       Tcanvas;
      penWidth:     integer;
      PaintBox:     TpaintBox;
      avecAxeX, avecAxeY: boolean;
      grandAxeX, grandAxeY: boolean;
      NbreCourbes:  integer;
      voieOrphy:    array[indiceVoieYD] of TcourbePoint;
      voieGTS2:     array[indiceVoieYD] of TcourbeGTS2;
      voieTemps:    Tcourbe;
      voies:        array[indiceVoieOsc] of Tcourbe;
      voieAbscisse: indiceVoieOsc;
      LimiteCourbe, LimiteFenetre: Trect;
      LargCarac, HautCarac, Marge: integer;
      Nbre:         indiceVecteur;
      xAxeY:        integer;
      avecGTS2:     boolean;
      constructor Create;
      procedure Draw;
      procedure TraceCourbes;
      procedure MyLine(x1, y1, x2, y2: integer);
      procedure TraceMotif(xi, yi: integer; motif: Tmotif);
      procedure TraceReticule(voie: indiceVoieYD);
      procedure TraceUnPoint(voie: indiceVoieYD);
      procedure SetHauteurFonte(var H: integer);
      destructor Destroy; override;
   end;

var
   graphePoint: TgraphePoint;

const
   CouleurDefaut: array[indiceVoieYD] of Tcolor =
      (clBlue, clRed, clFuchsia, clLime);

implementation

function Tcourbe.XMonde(xi: integer): double;
begin
   Result := (xi - B) / A;
end;

function Tcourbe.XWindow(r: double): integer;
begin
   if r = sansSignif then
      Result := IntSansSignif
   else
      try
         Result := round(A * r + B)
      except
         Result := IntSansSignif
      end;
end;

procedure TgraphePoint.SetHauteurFonte(var H: integer);
begin
   if H <= 13 then
      H := 13 { MS sans serif 8 points = 13 pixels }
   else if H <= 18 then
      H := 16 { MS sans serif 10 points = 16 pixels }
   else
      H := 20 { MS sans serif 12 points = 20 pixels };
   canvas.font.Name := 'Regressi';
   canvas.font.Height := H;
end;

procedure TgraphePoint.MyLine(x1, y1, x2, y2: integer);
begin
   canvas.moveTo(x1, y1);
   canvas.lineTo(x2, y2);
end;

procedure TgraphePoint.Draw;

   procedure ChercheMonde;
   var
      voie: indiceVoieYD;
   begin
      with limiteFenetre do
      begin
         avecAxeY  := ((bottom - top) div 5 > NbreGradMin * hautCarac div 4);
         avecAxeX  := (right - left) > NbreGradMin * 4 * largCarac;
         grandAxeY := (bottom - top) div 5 > NbreGraduations * HautCarac div 4;
         grandAxeX := (right - left) > NbreGraduations * 4 * largCarac;
      end;
      limiteCourbe := limiteFenetre;
      Inc(limiteCourbe.left, marge);
      if avecAxeY then
         Inc(limiteCourbe.left, LongNombreAxe * largCarac);
      if avecAxeX then
         Dec(limiteCourbe.bottom, hautCarac + marge);
      for voie := low(indiceVoieYD) to high(indiceVoieYD) do
         with voieOrphy[voie] do
         begin
            CourbeTop := LimiteCourbe.top;
            CourbeBottom := LimiteCourbe.bottom;
         end;
      if avecGTS2 then
         for voie := low(indiceVoieYD) to high(indiceVoieYD) do
            with voieGTS2[voie] do
            begin
               CourbeTop := LimiteCourbe.top;
               CourbeBottom := LimiteCourbe.bottom;
            end;
      with voies[voieAbscisse] do
      begin
         if voieAbscisse = voieT then
         begin
            mini := 0;
            if (nbre > 0) and (vecteurAff[pred(nbre)] > maxi) then
               maxi := 1.5 * vecteurAff[pred(nbre)];
            if maxi < 16 then
               maxi := 16;
         end;
         setMinMax;
         A := (limiteCourbe.right - limiteCourbe.left) / (Fmaxi - Fmini);
         B := limiteCourbe.left - mini * A;
         setDelta(grandAxeX);
      end;
      for voie := low(indiceVoieYD) to high(indiceVoieYD) do
         with voieOrphy[voie] do
            if active and (voie <> voieAbscisse) then
            begin
               setMinMax;
               A := (CourbeTop - CourbeBottom) / (FMaxi - FMini);
               B := CourbeBottom - Mini * A;
               setDelta(grandAxeY);
            end;
      if avecGTS2 then
         for voie := low(indiceVoieYD) to high(indiceVoieYD) do
            with voieGTS2[voie] do
               if active then
               begin
                  setMinMax;
                  A := (CourbeTop - CourbeBottom) / (FMaxi - FMini);
                  B := CourbeBottom - Mini * A;
                  setDelta(grandAxeY);
               end;
   end; { ChercheMonde }

   procedure DrawAxis;{ Trac des axes }
   var
      yAxeX: integer;
      xNombreMax: integer;
      PointCourant: double;

      procedure graduationX(x: integer);
      begin
         if grille then
            MyLine(x, limiteCourbe.top, x, limiteCourbe.bottom)
         else
            MyLine(x, yAxeX - marge, x, yAxeX + marge);
      end;

      procedure graduationY(y: integer);
      begin
         if grille then
            MyLine(limiteCourbe.left, y, limiteCourbe.right, y)
         else
            MyLine(xAxeY - marge, y, xAxeY + marge, y);
      end;


      function YcorrectNombreGTS2(y: integer; voie: indiceVoieYD): boolean;
      begin
         with voieGTS2[voie] do
         begin
            Result := (y > (CourbeTop + 2 * HautCarac)) and (y <
               (CourbeBottom - marge)) and (abs(y - yAxeX) > marge);
         end;
      end;

      function YcorrectGradGTS2(y: integer; voie: indiceVoieYD): boolean;
      begin
         with voieGTS2[voie] do
            Result := (y > (courbeTop + HautCarac)) and (y <
               (CourbeBottom - HautCarac));
      end;

      procedure initAxeGTS2(voie: indiceVoieYD);
      var
         y1: integer;
      begin
         with voieGTS2[voie] do
         begin
            PointCourant := PointDebut;
            y1 := XWindow(0);
            canvas.Pen.color := couleur;
            if (y1 > CourbeTop) and (y1 < CourbeBottom) then
               Myline(limiteCourbe.left, y1, limiteCourbe.right, y1);
            MyLine(xAxeY, yAxeX, xAxeY, CourbeTop);
            { flches }
            MyLine(xAxeY - marge, courbeTop + 3 * marge, xAxeY, courbeTop);
            MyLine(xAxeY + marge, courbeTop + 3 * marge, xAxeY, courbeTop);
         end;
      end; // InitAxeGTS2

      procedure traceAxeGTS2(voie: indiceVoieYD);
      var
         ys, margeY: integer;
         nombreAff:  strUnite;
      begin
         with voieGTS2[voie] do
         begin
            canvas.font.Color := Couleur;
            if grille then
            begin
               canvas.pen.style := psDot;
               canvas.pen.color := clGray;
            end
            else
               canvas.pen.style := psSolid;
            SetTextAlign(canvas.handle, TA_right + TA_bottom);
            margeY := -2 * marge;
            repeat
               ys := Xwindow(PointCourant); // coordonne cran de marque
               if YcorrectGradGTS2(ys, voie) then
               begin
                  graduationY(ys);
                  if YcorrectNombreGTS2(ys, voie) then
                  begin
                     if abs(pointCourant) < (FMaxi - FMini) / 1000 then
                        pointCourant := 0;
                     NombreAff := formatAxe(pointCourant / exposant);
                     Canvas.TextOut(xAxeY + margeY, ys + marge, NombreAff);
                  end;
               end;
               pointCourant := pointCourant + deltaAxe;
            until pointCourant >= Maxi;
            canvas.pen.style := psSolid;
            SetTextAlign(canvas.handle, TA_right + TA_top);
            canvas.TextOut(xAxeY + margeY, CourbeTop, nomAxe(exposant));
         end;
      end; // traceAxeYGTS2

      procedure zeroGTS2(voie: indiceVoieYD);
      var
         y1: integer;
      begin
         with voieGTS2[voie] do
         begin
            y1 := Xwindow(0);
            if (y1 > CourbeTop) and (y1 < CourbeBottom) then
            begin
               canvas.pen.style := psDot;
               canvas.pen.color := voieOrphy[voie].couleur;
               Myline(limiteCourbe.left, y1, limiteCourbe.right, y1);
            end;
            canvas.pen.style := psSolid;
         end;
      end;//zeroGTS2

      function YcorrectNombre(y: integer; voie: indiceVoieYD): boolean;
      begin
         with voieOrphy[voie] do
         begin
            YcorrectNombre := (y > (CourbeTop + 2 * HautCarac)) and
               (y < (CourbeBottom - marge)) and (abs(y - yAxeX) > marge);
         end;
      end;

      function YcorrectGrad(y: integer; voie: indiceVoieYD): boolean;
      begin
         with voieOrphy[voie] do
            YcorrectGrad := (y > (courbeTop + HautCarac)) and
               (y < (CourbeBottom - HautCarac));
      end;

      function XcorrectNombre(x: integer): boolean;
      begin
         XcorrectNombre := (x > (limiteCourbe.left + marge)) and
            (x < xNombreMax) and (abs(x - xAxeY) > marge);
      end;

      function XcorrectGrad(x: integer): boolean;
      begin
         XcorrectGrad := (x > limiteCourbe.left) and (x < limiteCourbe.right);
      end;

      procedure initAxeX(voie: Tcourbe);
      var
         x1: integer;
      begin
         with voie do
         begin
            PointCourant := PointDebut;
            x1 := XWindow(0);
            canvas.font.Color := clBlack;
            canvas.pen.color := clBlack;
            if (x1 > limiteCourbe.left) and (x1 < limiteCourbe.right) then
            begin
               canvas.pen.style := psDash;
               Myline(x1, limiteCourbe.bottom, x1, limiteCourbe.top);
            end;
            canvas.pen.style := psSolid;
            xNombreMax := limiteFenetre.right - largCarac * longNombreAxe div 2;
            with limiteFenetre do
            begin
               MyLine(left, yAxeX, right, yAxeX);
               { flches }
               MyLine(right - marge * 3, yAxeX + marge, right, yAxeX);
               MyLine(right - marge * 3, yAxeX - marge, right, yAxeX);
            end;
         end;
      end;// initAxeX

      procedure traceAxeX(voie: Tcourbe);
      var
         xs: integer;
         NombreAff: strUnite;
      begin
         with voie do
         begin
            SetTextAlign(canvas.handle, TA_center + TA_top);
            if grille then
            begin
               canvas.pen.style := psDot;
               canvas.pen.color := clGray;
            end
            else
               canvas.pen.style := psSolid;
            repeat
               xs := Xwindow(PointCourant);
               if XcorrectGrad(xs) then
               begin
                  graduationX(xs);
                  if XcorrectNombre(xs) then
                  begin
                     NombreAff := formatAxe(pointCourant / exposant);
                     canvas.TextOut(xs, yAxeX + marge, NombreAff);
                  end;
               end;
               PointCourant := PointCourant + deltaAxe;
            until (PointCourant >= maxi);
            canvas.pen.style := psSolid;
            canvas.pen.color := clBlack;
            setTextAlign(canvas.handle, TA_right + TA_top);
            canvas.TextOut(limiteFenetre.right - marge, yAxeX + marge, nomAxe(exposant));
            SetTextAlign(canvas.handle, TA_left + TA_top);
         end;
      end; // traceAxeX

      procedure initAxeY(voie: indiceVoieYD);
      var
         y1: integer;
      begin
         with voieOrphy[voie] do
         begin
            PointCourant := PointDebut;
            y1 := XWindow(0);
            canvas.Pen.color := couleur;
            if (y1 > CourbeTop) and (y1 < CourbeBottom) then
               Myline(limiteCourbe.left, y1, limiteCourbe.right, y1);
            MyLine(xAxeY, yAxeX, xAxeY, CourbeTop);
            { flches }
            MyLine(xAxeY - marge, courbeTop + 3 * marge, xAxeY, courbeTop);
            MyLine(xAxeY + marge, courbeTop + 3 * marge, xAxeY, courbeTop);
         end;
      end; // initAxeY

      procedure traceAxeY(voie: indiceVoieYD);
      var
         ys, margeY: integer;
         nombreAff:  strUnite;
      begin
         with voieOrphy[voie] do
         begin
            canvas.font.Color := Couleur;
            if grille then
            begin
               canvas.pen.style := psDot;
               canvas.pen.color := clGray;
            end
            else
               canvas.pen.style := psSolid;
            SetTextAlign(canvas.handle, TA_right + TA_bottom);
            margeY := -2 * marge;
            repeat
               ys := Xwindow(PointCourant); // coordonne cran de marque
               if YcorrectGrad(ys, voie) then
               begin
                  graduationY(ys);
                  if YcorrectNombre(ys, voie) then
                  begin
                     if abs(pointCourant) < (FMaxi - FMini) / 1000 then
                        pointCourant := 0;
                     NombreAff := formatAxe(pointCourant / exposant);
                     Canvas.TextOut(xAxeY + margeY, ys + marge, NombreAff);
                  end;
               end;
               pointCourant := pointCourant + deltaAxe;
            until pointCourant >= Maxi;
            canvas.pen.style := psSolid;
            SetTextAlign(canvas.handle, TA_right + TA_top);
            canvas.TextOut(xAxeY + margeY, CourbeTop, nomAxe(exposant));
         end;
      end;// traceAxeY

      procedure zeroX(voie: Tcourbe);
      var
         x1: integer;
      begin
         with voie do
            x1 := XWindow(0);
         if (x1 > limiteCourbe.left) and (x1 < limiteCourbe.right) then
         begin
            canvas.pen.style := psDot;
            canvas.pen.color := clBlack;
            Myline(x1, limiteCourbe.bottom, x1, limiteCourbe.top);
         end;
         canvas.pen.style := psSolid;
      end; // zeroX

      procedure zeroY(voie: indiceVoieYD);
      var
         y1: integer;
      begin
         with voieOrphy[voie] do
         begin
            y1 := Xwindow(0);
            if (y1 > CourbeTop) and (y1 < CourbeBottom) then
            begin
               canvas.pen.style := psDot;
               canvas.pen.color := voieOrphy[voie].couleur;
               Myline(limiteCourbe.left, y1, limiteCourbe.right, y1);
            end;
            canvas.pen.style := psSolid;
         end;
      end; // zeroY

   var
      voie: indiceVoieYD;
   begin // DrawAxis 
      yAxeX := limiteCourbe.bottom;
      xAxeY := limiteCourbe.left;
      if avecAxeX then
      begin
         initAxeX(voies[voieAbscisse]);
         traceAxeX(voies[voieAbscisse]);
         zeroX(voies[voieAbscisse]);
      end;
      if avecAxeY then
      begin
         for voie := low(indiceVoieYD) to high(indiceVoieYD) do
            if voieOrphy[voie].active and (voie <> voieAbscisse) then
            begin
               initAxeY(voie);
               traceAxeY(voie);
               zeroY(voie);
               break;
            end;
         if avecGTS2 then
            for voie := low(indiceVoieYD) to high(indiceVoieYD) do
               if voieGTS2[voie].active then
               begin
                  initAxeGTS2(voie);
                  traceAxeGTS2(voie);
               end;
      end;
      for voie := low(indiceVoieYD) to high(indiceVoieYD) do
         if voieOrphy[voie].active and (voie <> voieAbscisse) then
            zeroY(voie);
      if avecGTS2 then
         for voie := low(indiceVoieYD) to high(indiceVoieYD) do
         if voieGTS2[voie].active then
            zeroGTS2(voie);

      SetTextAlign(canvas.handle, TA_left + TA_top);
   end; // DrawAxis

begin // draw
   limiteFenetre := rect(0, 0, PaintBox.Width, PaintBox.Height);
   with limiteFenetre do
      HautCarac := (bottom - top) div 50;
   setHauteurFonte(HautCarac);
   LargCarac := 2 * HautCarac div 3;
   Marge := HautCarac div 3;
   canvas.Brush.color := clWindow;
   canvas.FillRect(limiteFenetre);
   canvas.pen.mode  := pmCopy;
   canvas.pen.color := clBlack;
   canvas.pen.Width := 1;
   setBkMode(canvas.handle, transparent);
   chercheMonde;
   drawAxis;
   if Nbre > 0 then
      TraceCourbes;
   canvas.pen.style := psSolid;
end; // draw

procedure TgraphePoint.TraceCourbes;

   procedure traceLignesXY(voie: indiceVoieYD);
   var
      i: integer;
      xi, yi: integer;
   begin
      canvas.brush.color := voieOrphy[voie].couleur;
      for i := 0 to pred(nbre) do
      begin
         with voies[voieAbscisse] do
            xi := XWindow(vecteurAff[i]);
         with voies[voie] do
            yi := XWindow(vecteurAff[i]);
         traceMotif(xi, yi, voieOrphy[voie].motif);
      end;
   end; // traceLignesXY

   procedure traceLignesGTS2(voie: indiceVoieYD);
   var
      i: integer;
      xi, yi: integer;
   begin
      canvas.brush.color := voieGTS2[voie].couleur;
      for i := 0 to pred(nbre) do
      begin
         with voies[voieAbscisse] do
            xi := XWindow(vecteurAff[i]);
         with voieGTS2[voie] do
            yi := XWindow(vecteurAff[i]);
         traceMotif(xi, yi, voieGTS2[voie].motif);
      end;
   end; // traceLignesGTS2

var
   voie: indiceVoieYD;
begin
   canvas.pen.Width := penWidth;
   for voie := low(indiceVoieYD) to high(indiceVoieYD) do
      if voieOrphy[voie].active and (voie <> voieAbscisse) then
      begin
         canvas.pen.color := voieOrphy[voie].couleur;
         TraceLignesXY(voie);
      end;
   if avecGTS2 then
      for voie := voieG to voieJ do
         if voieGTS2[voie].active then
         begin
            canvas.pen.color := voieGTS2[voie].couleur;
            TraceLignesGTS2(voie);
         end;
   canvas.pen.Width := 1;
end;

destructor TgraphePoint.Destroy;
var
   i: indiceVoieYD;
begin
   for i := low(indiceVoieYD) to high(indiceVoieYD) do
   begin
      voieOrphy[i].Free;
      voieGTS2[i].Free;
   end;
   voieTemps.Free;
   inherited Destroy;
end;

constructor TgraphePoint.Create;
var
   voie: indiceVoieYD;
begin
   inherited Create;
   penWidth  := 1;
   voieTemps := Tcourbe.Create('t', 's', 0);
   voieTemps.signif := 'date d''acquisition';
   voieTemps.active := True;
   voies[voieT] := voieTemps;
   for voie := low(indiceVoieYD) to high(indiceVoieYD) do
   begin
      voieOrphy[voie] := TcourbePoint.Create('V' + IntToStr(voie), 'V', voie);
      voieOrphy[voie].voieGTI2 := voie;
      voieGTS2[voie] := TcourbeGTS2.Create('V' + IntToStr(voie), 'V', voie);
      voies[voie] := voieOrphy[voie];
   end;
end;

constructor Tcourbe.Create(Anom: shortString; Aunite: strUnite; Anumero: integer);
begin
   inherited Create;
   A := 1;
   B := 0;
   nom := Anom;
   NomUnite := Aunite;
   FMaxi := 15;
   FMini := -15;
   exposantAff := 1;
   NbreAvantVirgule := 0;
   signif := '';
   Motif := mCroix;
   couleur := CouleurDefaut[Anumero];
   numero := Anumero;
   if couleur = clWindow then
      couleur := clActiveCaption;
end;

procedure Tcourbe.SetDelta(grandAxe: boolean);
 // Exposant=puissance de 10 multiple de 3 de manire  avoir entre
 // NbreGraduations (10) et N/2.5 (4) graduations sur l'axes.
 // ou NbreGradMin (5) et N/2.5 (2) graduations sur l'axes.
 // Delta est l'intervalle entre les graduations qui commencent  PremiereGraduation
const
   coeff: array[0..5] of double = (0.1, 0.2, 0.5, 1, 2, 5);
var
   AbsMax, AbsMin, largeur: double;
   Exponent, index: integer;
   N: byte;
begin
   AbsMax := abs(Fmaxi);
   AbsMin := abs(Fmini);
   if AbsMax < AbsMin then
      AbsMax := AbsMin;
   Exponent := floor(Log10(AbsMax));
   largeur := (Fmaxi - Fmini) * dix(-exponent);
   index := 0;
   if grandAxe then
      while ((largeur / coeff[index]) >= NbreGraduations) and (index < 5) do
         Inc(index)
   else
      while ((largeur / coeff[index]) >= NbreGradMin) and (index < 5) do
         Inc(index);
   DeltaAxe := coeff[index] * dix(exponent);
   NbreChiffre := ceil(Log10(AbsMax)) - floor(Log10(DeltaAxe));
   PointDebut := ceil(Fmini / deltaAxe) * deltaAxe;
   Index := Exponent div 3;
   if ((Exponent mod 3) < 0) then
      Dec(index);
   { la division euclidienne des informaticiens <> div des maths ! }
   Exposant := Dix(3 * Index);
   N := ceil(Log10(AbsMax / Exposant));
   if N > NbreChiffre then
      NbreChiffre := N;
   if NbreChiffre < 3 then
      NbreChiffre := 3;
   if (NbreChiffre > longNombreAxe) then
      NbreChiffre := longNombreAxe;
end;

function Tcourbe.FormatAxe(x: double): strUnite;
begin
   if abs(x) < ((Fmaxi - Fmini) / 1000) then
      FormatAxe := '0'
   else
      FormatAxe := FloatToStrF(x, ffGeneral, NbreChiffre, 0);
end;

function TcourbePoint.CalibreVoie: strUnite;
begin
   if Fmaxi * Fmini < 0 then
      if (Fmaxi + Fmini) < (Fmaxi - Fmini) / 10 then
         Result := '' + formatUnite(Fmaxi)
      else
         Result := formatUniteDouble(Fmini, Fmaxi)
   else if maxi > 0 then
      Result := formatUnite(Fmaxi)
   else
      Result := formatUnite(Fmini);
end;

procedure TgraphePoint.TraceReticule(voie: indiceVoieYD);
var
   xi, yi: integer;
begin
   if voie = voieAbscisse then
      exit;
   with voies[voieAbscisse] do
      xi := XWindow(valeurAff);
   with voies[voie] do
      yi := XWindow(valeurAff);
   canvas.pen.color := voieOrphy[voie].couleur;
   canvas.brush.color := canvas.pen.color;
   canvas.pen.Width := WidthCursor;
   canvas.pen.mode  := pmNotXor;
   canvas.pen.Width := 3;
   MyLine(xi - scaleCurseur, yi, xi + scaleCurseur, yi);
   MyLine(xi, yi - scaleCurseur, xi, yi + scaleCurseur);
   canvas.pen.Width := 1;
end;// TraceReticule 

procedure TgraphePoint.TraceUnPoint(voie: indiceVoieYD);
var
   xi, yi: integer;
begin
   if voie = voieAbscisse then
      exit;
   canvas.pen.color := voieOrphy[voie].couleur;
   canvas.brush.color := canvas.pen.color;
   canvas.pen.Width := penWidth;
   with voies[voieAbscisse] do
      xi := XWindow(valeurAff);
   with voies[voie] do
      yi := XWindow(valeurAff);
   traceMotif(xi, yi, voieOrphy[voie].motif);
   canvas.pen.Width := 1;
end; // TracePoint 

procedure TgraphePoint.TraceMotif(xi, yi: integer; motif: Tmotif);
var
   PolyLigne: array[1..5] of Tpoint;
begin
   case motif of
      mCroix:
      begin
         MyLine(xi - dimPoint, yi, xi + dimPoint, yi);
         MyLine(xi, yi - dimPoint, xi, yi + dimPoint);
      end;
      mCroixDiag:
      begin
         MyLine(xi - dimPoint, yi - dimPoint, xi + dimPoint, yi + dimPoint);
         MyLine(xi + dimPoint, yi - dimPoint, xi - dimPoint, yi + dimPoint);
      end;
      mCercle: canvas.ellipse(xi - dimPoint, yi - dimPoint, xi +
            dimPoint + 1, yi + dimPoint + 1);
      mCarre: canvas.rectangle(xi - dimPoint, yi - dimPoint, xi +
            dimPoint, yi + dimPoint);
      mLosange:
      begin
         PolyLigne[1] := Point(xi - dimPoint, yi);
         PolyLigne[2] := Point(xi, yi - dimPoint);
         PolyLigne[3] := Point(xi + dimPoint, yi);
         PolyLigne[4] := Point(xi, yi + dimPoint);
         PolyLigne[5] := Point(xi - dimPoint, yi);
         canvas.polyLine(PolyLigne);
      end;
   end;
end;

function TcourbePoint.AffVoltmetre: strUnite;
var
   posVirgule, i: integer;
begin
   if valeurAff = sansSignif then
      Result := '  Erreur'
   else
   if (valeurAff < Fmini) or (valeurAff > Fmaxi) then
      Result := '  ??'
   else
   begin
      Result := FormatUniteExp(valeurAff, ExposantAff, NbreAvantVirgule);
      posVirgule := pos('.', Result);
      if posVirgule = 0 then
         posVirgule := pos(',', Result);
      for i := 0 to (NbreAvantVirgule - posVirgule) do
         Result := ' ' + Result;
   end;
end;

procedure Tcourbe.setMinMax;
var
   maxmax: double;
   exponent: integer;
begin
   VerifMinMaxDouble(Fmini, Fmaxi);
   MaxMax := abs(Fmaxi);
   if MaxMax < abs(Fmini) then
      MaxMax := abs(Fmini);
   Exponent := floor(Log10(MaxMax));
   if (Exponent mod 3) < 0 then
      Exponent := (Exponent div 3) - 1
   else
      Exponent := Exponent div 3;
   // la division euclidienne des informaticiens <> div des maths !
   ExposantAff := Dix(3 * Exponent);
   NbreAvantVirgule := floor(Log10(MaxMax / ExposantAff)) + 2;
end; // setMinMax

function TcourbeGTS2.ValeurReal(V: integer): double;
begin
   Result := valeurVolt(V);
   if indexCapteurAuto > 0 then
   begin
      Result := Offset + unVolt * Result;
      if Result < mini then
         Result := mini;
      if Result > maxi then
         Result := maxi;
   end;
end;

function TcourbeGTS2.VoltToReal(V: double): double;
begin
   Result := V;
   if indexCapteurAuto > 0 then
      Result := Offset + unVolt * Result;
end;

function TcourbeGTS2.ValeurVolt(V: integer): double;
begin
   Result := 20.0 * V / MaxiHardEA;
end;

procedure TcourbeGTS2.setMinMax;
var
   maxmax: double;
   exponent: integer;
begin
   if indexCapteurAuto = 0 then
   begin
      inherited setMinMax;
      exit;
   end;
   if nomUnite = 'hPa' then
      setExposant(100, 0);
   MaxMax := abs(Fmaxi);
   if MaxMax < abs(Fmini) then
      MaxMax := abs(Fmini);
   Exponent := floor(Log10(MaxMax));
   if (Exponent mod 3) < 0 then
      Exponent := (Exponent div 3) - 1
   else
      Exponent := Exponent div 3;
   // la division euclidienne des informaticiens <> div des maths ! 
   setExposant(maxmax, Exponent);
end; // setMinMax

function TcourbeGTS2.AffVoltmetre: strUnite;
begin
   if indexCapteurAuto > 0 then
      Result := inherited AffVoltmetre
   else
      Result := FormatUniteDB15(valeurAff, True);
end;

constructor TcourbeGTS2.Create(Anom: shortString; Aunite: strUnite; Anumero: integer);
begin
   inherited Create(Anom, Aunite, Anumero);
   nom  := 'V';
   nomUnite := 'V';
   signif := '';
   Mini := -5;
   Maxi := 5;
   Offset := 0;
   UnVolt := 1;
   ValeurAff := 0;
   indexCapteurAuto := 0;
end;

end.

