type
    IndiceParam = (Amplitude,Pulsation);
    TableauParam = array[indiceParam] of double;
    MatriceParam = array[indiceParam,indiceParam] of double;

Procedure EffectueModele;

Function sinc(x : reel) : reel;
begin
   try
      sinc := sin(x)/x
   except
        sinc := 1
   end;
end;

Function derSinc(x : reel) : reel;
begin
    try
      derSinc := (X*cos(X)-sin(X))/sqr(X)
    except
          derSinc := 0
    end;
end;

{------- boucle de recherche par mthode gradient-Newton -------}
var
    J,Jprecedent : double;
{ (FonctionTheorique(x,param)-FonctionExperimentale(x))^2 }
    derJ  : TableauParam;
{ derj = drive de J = (fonctionTheorique-Fexp)*dfonctionTheorique/dParam }
    der2J : MatriceParam;
{ der2j = drive seconde de J (approximative) =
  DfonctionTheorique/dparam*DfonctionTheorique/dparam
  on nglige (fonctionTheorique-Fexp)d2Fth/dparam*dparam }
    IncParam : TableauParam;
    SommeCarreY : double;

procedure calculPrecision;
var U,InvDer2j : MatriceParam;

procedure construireU;
var
   v  : extended;
   i,k,l : indiceParam;
begin
 for i := low(IndiceParam) to high(IndiceParam) do begin
     v := der2J[i,i];
     try
     k := low(indiceParam);
     while k>i do begin
           v := v-sqr(U[k,i]);
           dec(k);
     end;
     U[i,i] := sqrt(v);
     if i<high(indiceParam) then
         for l := succ(i) to high(indiceParam) do begin
            v := der2J[l,i];
            k := low(indiceParam);
            while k<i do begin
                v := v-U[k,i]*U[k,l];
                inc(k);
            end;
            U[i,l] := v/U[i,i];
     end;{l}
     except
        for l := low(indiceParam) to high(indiceParam) do begin
              U[i,l] := 0;
              U[l,i] := 0;
              der2J[l,i] := 0;
              der2J[i,l] := 0;
{ erreur sur param[i] mais on veut trouver les autres }
end {l}
      end
  end;{i}
end;{construireU}

procedure resoudreU(l : indiceParam);
var i,k : indiceParam;
    Y   : TableauParam;
    v   : extended;
begin
    for i := low(indiceParam) to high(indiceParam) do if U[i,i]= 0
        then Y[i] := 0
        else begin
           try
           if i=l then v := 1 else v := 0;
           k := low(indiceParam);
           while k<i do begin
                 v := v-U[k,i]*Y[k];
                 inc(k);
           end;
           Y[i] := v/U[i,i];
           except
              Y[i] := 0
           end;
        end;{else}
{ passage  invder2J }
     for i := high(indiceParam) downto low(indiceParam) do begin
         v := Y[i];
         if i<high(indiceParam) then
             for k := succ(i) to high(indiceParam) do
                 v := v-U[i,k]*invDer2J[l,k];
         try
         invDer2J[l,i] := v/U[i,i];
         except
            invDer2J[l,i] := 0
         end;
    end;{i}
end;{resoudreU}

{ Trigeassou p 267 }
var i,l : indiceParam;
begin { calculPrecision }
  construireU; { rsolution successive avec B=(1..0) jusqu' (0..1) }
  for i := low(indiceParam) to high(indiceParam) do resoudreU(i);
{ Inversion de der2J = symtrique dfinie positive par  Mthode de Choleski
  Trigeassou p 105 }
  for i := low(indiceParam) to high(indiceParam) do begin
     try
     incParam[i] := 0;
     for l := low(indiceParam) to high(indiceParam) do
         incParam[i] := incParam[i] - invDer2j[l,i]*derj[l];
     except
         incParam[i] := 0;
     end;
  end;{i}
end; { calculprecision }

procedure InitCalculJ;
{On calcule une valeur relative  l'tendue des mesures}
var k : indiceParam;
    i : integer;
begin
   SommeCarreY := 0;
   for i := 0 to pred(Nbuffer) do
       SommeCarreY := SommeCarreY + sqr(buffer[i]);
   Jprecedent := 2*sommeCarreY;
   J := Jprecedent;
end;{initCalculJ}

procedure CalculJ(var diverge,stable : boolean);

procedure calculLoc;
var k1,k2 : indiceParam;
    i : integer;
    sinusC,derSinusC : double;
    phaseLoc : double;
    ValDerf : TableauParam;{ valeur de la drive premire }
    ecart : extended; {valeur de l'cart entre la fonction thorique et exprimentale }
begin
    for i := j1 to j2 do begin
            PhaseLoc := (i-valeurParam[pulsation])*pi;
            SinusC := sinc(phaseLoc);
            DerSinusC := dersinc(phaseLoc);
            Ecart := valeurParam[amplitude]*SinusC-buffer[i];
            valDerf[amplitude] := SinusC;
            valDerf[pulsation] := -pi*valeurParam[amplitude]*derSinusC;
            J := J + sqr(ecart);
            for k1 := low(indiceParam) to high(indiceParam) do begin
                derj[k1] := derj[k1]+Ecart*valDerf[k1];
                for k2 := low(indiceParam) to k1 do
                    der2j[k1,k2] := der2j[k1,k2]+valDerf[k2]*valDerf[k1];
            end;{k1}
        end;{i}
end;{calculLoc}

Const
      PrecisionAbsolueJ = 1e-3;
      PrecisionRelative = 0.01;
var k1,k2 : indiceParam;
    Delta : extended;
    PrecisionModele : double;
begin
  {Initialisation}
   J := 0;
   for k1 := low(indiceParam) to high(indiceParam) do begin
       derJ[k1] := 0;
       for k2 := low(indiceParam) to high(IndiceParam) do
          der2j[k1,k2] := 0;
   end;{k1}
   CalculLoc;
   PrecisionModele := sqrt(J/SommeCarreY);
   Delta := J-Jprecedent;
   Stable := (abs(Delta)<(Jprecedent*PrecisionRelative)) or
             (PrecisionModele<PrecisionAbsolueJ);
   Diverge := (Delta>0) and not Stable;
end; { calculJ }

const iMax = 16; { time out }
var k : IndiceParam;
    sauveValeurParam : tableauParam;
    i : byte;
    diverge,stable : boolean;
{ boucle de recherche par mthode gradient-Newton }
begin
    InitCalculJ;
    sauveValeurParam := ValeurParam;
    i := 0;
    CalculJ(diverge,stable);
    repeat
        inc(i);
        Jprecedent := J;
        calculPrecision;
        for k := low(indiceParam) to high(indiceParam) do
            valeurParam[k] := valeurParam[k]+incParam[k];
        CalculJ(diverge,stable);
        if diverge
           then begin
              ValeurParam := sauveValeurParam;
              stable := true;
           end
           else sauveValeurParam := ValeurParam;
   until (i>iMax) or stable;
end; { EffectueModele }

Procedure CalculFrequence;
var valeurMax : double;
    i,j1,j2 : integer;
    zz : double;
begin
    valeurMax := 0;
    for i := 0 to pred(NbreF) do begin
        zz := spectre[i];
        if zz>valeurMax then begin
           valeurMax := zz;
           j1 := i;
        end;
    end;
    if spectre[j1-1]>spectre[j1+1] then j1 := j1-1;
    j2 := j1+1;
    ValeurParam[amplitude] := valeurMax;
    valeurParam[pulsation] := j1;
    EffectueModele;
end;


