unit gti2_osc;

interface
uses Windows, Messages, Forms, sysUtils, registry,
     gti2_dll, gti2Util_osc, math, maths_osc, oomisc, inifiles;

const
    deltaVgti2 = 30/16384;  // +-15 V sur 14 bits
    deltaVsynchro = 30/4096;  // +-15 V sur 14 bits
    NbrePreAcqGTI2 = 550; // 500 plus marge d'erreur de 10%
    FreqMaxGBF = 990000;
    FreqMinGBF = 0.02;

type
    pseudoWord = record case byte of
             0 : (octets : array[0..11] of char);
             1 : (mot : word);
             2 : (entier : smallInt);
             3 : (longMot : longWord);
             4 : (octet : byte);
             5 : (index : array[indiceVoieYD] of byte;valeur : array[indiceVoieYD] of smallInt);
    end;

    indiceVoie = 0..7;
    TbaseDeTemps = (b10mu,b20mu,b50mu,b100mu,b200mu,
                    b500mu,b1m,b2m,b5m,b10m,
                    b20m,b50m,b100m,b200m,b500m,
                    b1,b2,b5,b10,b20,b50,b100,
                    b200,b10min,b20min,b1h);

    PointYD = array[indiceVoieYD] of double;

    TGTI2 = class(Tobject)
          private
              DataGet : Tdata_RS232;
              FhandleGTI2 : Thandle;
              FThreadAcqLente : Thandle;
              Fstop : boolean;
              Fnbre : integer;
              FnbreVoies : integer;
              FdataStructure : Data_Acquisition;

              FdataGenerateur : Data_Generateur;
              FloopGBF : boolean;
              FreloadGBF : integer;
              FfreqGBF : double;
              FhorlogeGBF : indiceHorloge;
              FNbreGBF : integer;
              
              FvaleurVoie : array[indiceVoieYD] of vecteurWord;
              FoffsetVoie : array[indiceVoieYD,boolean,boolean] of integer; // diffrentiel puis 15V
              FdeltaVoie : array[indiceVoieYD] of double;
              Factive : array[indiceVoieYD] of boolean;
              Fdifferentiel : array[indiceVoieYD] of boolean;
              FbaseDeTemps : TbaseDeTemps;
              FnbrePoints : integer;
              FOffsetSeuil : array[boolean] of double;
   // valeur pour passer de seuil demand  seuil de la SA
   // valeurSA=seuil+offsetSeuil[montant]
              AcquisitionEnCours : boolean;
              FversionDLL : shortString;
              procedure setSynchro(asynchro : integer);
              function getSynchro : integer;
              function getNbrePointsTotal : integer;
              Procedure setSeuilVolt(y : double);
              procedure setFreqGBF(afreq : double);
              function getSeuilVolt : double;
              procedure setCalibre15V(v : indiceVoieYD;is15V : boolean);
              function getCalibre15V(v : indiceVoieYD) : boolean;
              procedure setActive(avoie : indiceVoieYD;active : boolean);
              function getActive(avoie : indiceVoieYD) : boolean;
              function getOffsetVoie(avoie : indiceVoieYD) : integer;
              function getOffsetSeuil(montant : boolean) : double;
              procedure setEtalonnage(active : boolean);
              function getEtalonnage : boolean;
              procedure setNbrePoints(n : integer);
              procedure setDifferentiel(avoie : indiceVoieYD;diff : boolean);
              function getDifferentiel(avoie : indiceVoieYD) : boolean;
              function getAcqTerminee : boolean;
              procedure setMontant(isMontant : boolean);
              procedure setOffsetSeuil(montant : boolean;valeur : double);
              function getMontant : boolean;
              function getFreqEch : integer;
              function getVersion : string;
              function AcquisitionLente : integer;              
              procedure setBaseDeTemps(abase : TbaseDeTemps);
              procedure calculBaseDeTemps;
              procedure send(commande : shortString);
          protected
              CodeFreq : byte;
              CodeReload : integer;
              CodeUnSurN : integer;
          public
              valeurPoint : pointYD;
              valeurReco : pseudoWord;
              OffsetSeuilM,OffsetSeuilD : double;
   // valeur pour passer de seuil demand  seuil de la SA
   // valeurSA=seuil+offsetSeuil[montant]
              property output : shortString write send;
              property acqTerminee : boolean read getAcqTerminee;
              property SeuilVolt : double read getSeuilVolt write setSeuilVolt;
              property VoieSynchro : integer read getSynchro write setSynchro;
              property NbreGBF : integer read FNbreGBF;
              property Nbre : integer read Fnbre;
              property NbrePoints : integer read FNbrePoints write setNbrePoints;
              property NbrePointsTotal : integer read getNbrePointsTotal;
              property FreqEch : integer read getFreqEch;
              property montant : boolean read getMontant write setMontant;
              property active[voie : indiceVoieYD] : boolean read getActive write setActive;
              property offsetVoie[voie : indiceVoieYD] : integer read getOffsetVoie;
              property calibre15V[voie : indiceVoieYD] : boolean read getCalibre15V write setCalibre15V;
              property offsetSeuil[montant : boolean] : double read getOffsetSeuil write setOffsetSeuil;
              property differentiel[voie : indiceVoieYD] : boolean read getDifferentiel write setDifferentiel;
              property baseDeTemps : TbaseDeTemps read FbaseDeTemps write setBaseDetemps;
              property BoucleGenerateur : boolean read FloopGBF write FloopGBF;
              property FreqGBF : double read FFreqGBF write setFreqGBF;
              property version : string read getVersion;
              property etalonnage : boolean read getEtalonnage write setEtalonnage;
              function isOpen : boolean;
              function Open : boolean;
              procedure Close;
              procedure store;
              procedure load;
              function LanceAcquisition : boolean;
              function stopAcquisition : boolean;
              procedure EtalonneZero(voie : indiceVoieYD;diff : boolean;var signal : boolean);
              procedure resetOffset;
              procedure LanceGenerateur(const data : vecteurGBF);
              procedure StopGenerateur;
              procedure GetValeurTemps(var valeurT : vecteur);
              procedure GetValeurVoie(v : indiceVoieYD;var valeur : vecteur);
              function GetValeurInit(v : indiceVoieYD) : double;
              function getResul : boolean;
              procedure LanceAcqLente;
              procedure StopAcqLente;

              function getReco : pseudoWord;
              function getInt(commande : shortString) : integer;

              constructor Create;
              destructor Destroy;override;
          end;

var GTI2 : TGTI2;

const
  FreqEchOsc : array[0..2] of extended = (6e6,2e6,10e6);
  BaseDeTempsStr : array[TbaseDeTemps] of string[6] =
            ('10 s','20 s','50 s','100 s','200 s',
             '500 s','1 ms','2 ms','5 ms','10 ms',
             '20 ms','50 ms','100 ms','200 ms','500 ms',
             '1 s','2 s',' 5 s','10 s','20 s','50 s','100 s',
             '200 s','10 min','20 min','1 h');
  DureeBalayage : array[TbaseDeTemps] of double =
            (1e-5,2e-5,5e-5,1e-4,2e-4,
             5e-4,1e-3,2e-3,5e-3,1e-2,
             2e-2,5e-2,1e-1,2e-1,5e-1,
             1,2,5,10,20,50,100,
             200,600,1200,3600);
  NombreMaxi : array[TbaseDeTemps] of integer =
            (150,300,900,1500,3000,
             5000,5000,10000,10000,10000,
             15000,15000,15000,15000,15000,
             15000,15000,15000,15000,15000,15000,15000,
             15000,15000,15000,15000);

implementation

function ThreadAcqLente : integer;stdcall;
begin
    result := gti2.AcquisitionLente
end;

function TGTI2.isOpen : boolean;
begin
    result := (FhandleGTI2>0) 
end;

function TGTI2.Open : boolean;
begin
     if FhandleGTI2<=0 then begin
        FHandleGTI2 := _SetPortOpen('OrphyGTI2-0');
        sleep(100);
     end;
     if FhandleGTI2<=0 then begin
        FHandleGTI2 := _SetPortOpen('OrphyGTI2-1');
        sleep(100);
     end;
     result := (FhandleGTI2>0);
end;

procedure TGTI2.Close;
begin
     stopAcquisition;
     if FhandleGTI2>0 then begin
        _SetPortClose(FHandleGTI2);
        sleep(100);
     end;
     FhandleGTI2 := 0;
end;

function TGTI2.GetVersion : string;
var version : array[0..255] of char;
begin
   if FhandleGTI2<=0
      then result := 'GTI2 non branche ?'
      else begin
          _GetVersion(FhandleGTI2,version);
          result := string(version);
      end;
end;

constructor TGTI2.Create;
var voie : indiceVoieYD;
    k1,k2 : boolean;
begin
  inherited create;
  FhandleGTI2 := 0;
  FThreadAcqLente := 0;
  FDataStructure.Trig.Valeur := 2048;
  FdataStructure.calibrage := 1;
  FnbreVoies := 1;
  with FdataGenerateur do begin
       NbreData := 480;
       BaseDeTemps := 0; // 100 kHz
       Reload := 0;
  end;
  for voie := low(indiceVoieYD) to high(indiceVoieYD) do begin
      FdeltaVoie[voie] := deltaVgti2;
      valeurPoint[voie] := 0;
      for k1 := false to true do
          for k2 := false to true do
              FoffsetVoie[voie,k1,k2] := 0;
  end;
  FnbrePoints := 1000;
end;

destructor TGTI2.Destroy;
begin
    close;
    inherited destroy;
end;

function TGTI2.GetResul : boolean;
var IncNbre,i : LongInt;
    DataChannel : data_gti;
    v : indiceVoieYD;
begin
        result := false;
        try
        IncNbre := _GetUSBdata(FhandleGTI2,DataChannel,FDataStructure) div FNbreVoies;
        if IncNbre>0 then begin
           if (Fnbre+incNbre)>maxVecteur then IncNbre := maxVecteur-Fnbre;
           for v := low(indiceVoieYD) to high(indiceVoieYD) do if Factive[v] then begin
//               incNbre := DataChannel.DataLen[v];
               for i := 0 to pred(incNbre) do
                   FvaleurVoie[v,FNbre+i] := DataChannel.BufferIn[v,i];
           end;
           FNbre := FNbre + IncNbre;
           result := true;
        end;
        except
        end;
end;


function TGTI2.StopAcquisition : boolean;

function AcqEnCours : boolean;
var DataChannel : data_gti;
begin
        result := false;
        try
        result := _GetUSBdata(FhandleGTI2,DataChannel,FDataStructure) > 0;
        except
        end;
end;

var atimer : eventTimer;
begin
     result := FhandleGTI2>0;
     if not result then exit;
     _StopAcqui(FhandleGTI2);
     newTimer(Atimer,18);
     repeat
         result := not AcqEnCours; // on vide le tampon
     until result or timerExpired(Atimer);
     Fnbre := 0;
     AcquisitionEnCours := false;
end;

Function TGTI2.LanceAcquisition : boolean;
var v : indiceVoieYD;
begin
        result := FhandleGTI2>0;
        if not result then exit;
        if AcquisitionEnCours then StopAcquisition;
        FnbreVoies := 0;
        FNbre := 0;
        for v := low(indiceVoieYD) to high(indiceVoieYD) do
            if Factive[v] then begin
               inc(FnbreVoies);
               FDataStructure.Voie[v].Differentiel := ord(Fdifferentiel[v]);
               FDataStructure.Voie[v].On := 1;
            end
            else FDataStructure.Voie[v].On := 0;
        if  FDataStructure.Trig.TrigOn>0 then
        calculBaseDeTemps;
        With FDataStructure.config do begin
             BaseDeTemps := CodeFreq;
             Reload := CodeReload;
             UnSurN := CodeUnSurN;
        end;
        AcquisitionEnCours := _LanceAcqui(FhandleGTI2,FDataStructure)=0;
        result := AcquisitionEnCours;
end;

Procedure TGTI2.setSynchro(asynchro : integer);
begin
     FDataStructure.Trig.TrigOn := aSynchro;
// presynchro systmatique si synchro seuil
     if asynchro>0
        then FDataStructure.Trig.preTrigOn := 1
        else FDataStructure.Trig.preTrigOn := 0
end;

Function TGTI2.getSynchro : integer;
begin
     result := FDataStructure.Trig.TrigOn
end;

Procedure TGTI2.setSeuilVolt(y : double);
var seuilInt : integer;
begin // 2048 correspond  0V, 0 correspond  -15V et 4095 correspond  +15V
      seuilInt := round(4096*(y+FoffsetSeuil[montant]+15)/30);
      if (seuilInt>96) and (seuilInt<4000) then
         FDataStructure.Trig.Valeur := seuilInt
end;

Function TGTI2.getSeuilVolt : double;
var seuilInt : word;
begin
     seuilInt := FDataStructure.Trig.Valeur;
     result := seuilInt*deltaVsynchro-15-FoffsetSeuil[montant]
end;

procedure TGTI2.setBaseDeTemps(abase : TbaseDeTemps);
begin
    FbaseDeTemps := abase;
    calculBaseDeTemps;
    With FDataStructure.config do begin
         BaseDeTemps := CodeFreq;
         Reload := CodeReload;
         UnSurN := CodeUnSurN;
    end;
end;

Procedure TGTI2.setMontant(isMontant : boolean);
begin
     FDataStructure.Trig.FrontMontant := ord(isMontant)
end;

Function TGTI2.getMontant : boolean;
begin
     result := FDataStructure.Trig.FrontMontant<>0
end;

procedure TGTI2.setActive(avoie : indiceVoieYD;active : boolean);
var voie : indiceVoieYD;
begin
     if active and not(Factive[aVoie]) then FdataStructure.calibrage := 1;     
     Factive[aVoie] := active;
     FDataStructure.Voie[aVoie].On := ord(active);
     FnbreVoies := 0;
     for voie := low(indiceVoieYD) to high(indiceVoieYD) do
         if Factive[voie] then inc(FnbreVoies);
end;

function TGTI2.getActive(avoie : indiceVoieYD) : boolean;
begin
     result := Factive[aVoie]
end;

function TGTI2.getOffsetVoie(avoie : indiceVoieYD) : integer;
begin
     result := FoffsetVoie[avoie,Fdifferentiel[avoie],calibre15V[avoie]]
end;

procedure TGTI2.setDifferentiel(avoie : indiceVoieYD;diff : boolean);
begin
     Fdifferentiel[avoie] := diff;
     FDataStructure.Voie[avoie].Differentiel := ord(diff);
     if Factive[aVoie] then FdataStructure.calibrage := 1;
end;

function TGTI2.getDifferentiel(avoie : indiceVoieYD) : boolean;
begin
     result := Fdifferentiel[avoie]
end;

procedure TGTI2.LanceGenerateur(const data : vecteurGBF);
var i,fin : integer;
begin
    if FhandleGTI2<=0 then exit;
    FdataGenerateur.NbreData := FnbreGBF;
    FdataGenerateur.BaseDeTemps := FhorlogeGBF;
    FdataGenerateur.Reload := FreloadGBF;
    fin := pred(NbreMaxGBF);
    for i := 0 to fin do
        FdataGenerateur.bufferOut[i] := data[i];
    _LanceGenerateur(FhandleGTI2,ord(FloopGBF),FdataGenerateur);
end;

procedure TGTI2.StopGenerateur;
begin
    if FhandleGTI2>0 then _StopGenerateur(FhandleGTI2)
end;

procedure TGTI2.setFreqGBF(afreq : double);
var Nbre : longInt;
begin
    if afreq<FreqMinGBF then afreq := FreqMinGBF;
    if afreq>FreqMaxGBF then afreq := FreqMaxGBF;
    if afreq>500 then begin // on peut essayer sans dbordement d'entier
       FReloadGBF := 0;
       FhorlogeGBF := 0;         // 48 MHz
       Nbre := round(freqHorloge[0]/afreq);
       if Nbre<NbreMaxGBF then begin
         FNbreGBF := Nbre;
         FfreqGBF := freqHorloge[0] div FNbreGBF;
         exit;    // 3 kHz  48 MHz
       end;
       FreloadGBF := 65536-1;
       FhorlogeGBF := 1; // 12 MHz
       Nbre := round(freqHorloge[1]/afreq);
       if Nbre<NbreMaxGBF then begin
          FNbreGBF := Nbre;
          FfreqGBF := freqHorloge[1] div FNbreGBF;
          exit; // 750 Hz  3 kHz
       end;
   end;
   FhorlogeGBF := 2;
   Nbre := round(freqHorloge[2]/afreq);// 4 MHz
   if Nbre<NbreMaxGBF then begin
      FNbreGBF := Nbre;
      FfreqGBF := freqHorloge[2] div FNbreGBF;
      exit; // 250 Hz  750 Hz
   end;
   FReloadGBF := (Nbre div NbreMaxGBF) + 1;
   FNbreGBF := Nbre div FreloadGBF;
   FfreqGBF := freqHorloge[FhorlogeGBF]/FNbreGBF/FReloadGBF;
   FReloadGBF := 65536-FreloadGBF;
end;

procedure TGTI2.GetValeurTemps(var valeurT : vecteur);
var dt : double;
    i : integer;
begin
    dt := 1/freqEch;
    for i := 0 to MaxVecteur do
        valeurT[i] := i*dt;
end;

procedure TGTI2.GetValeurVoie(v : indiceVoieYD;var valeur : vecteur);
var i,imax,k : integer;
    offset : double;
begin
    imax := nbrePointsTotal;
    offset := offsetVoie[v];
    for i := 0 to imax do begin
        k := FvaleurVoie[v,i];
        if k>8192 then k := k-16384;
        valeur[i] := (k-offset)*FdeltaVoie[v];
    end;
    for i := succ(imax) to MaxVecteur do
        valeur[i] := 0;
end;

function TGTI2.GetValeurInit(v : indiceVoieYD) : double;
var k : integer;
begin
    k := FvaleurVoie[v,0];
    if k>8192 then k := k-16384;
    result := (k-offsetVoie[v])*FdeltaVoie[v];
end;

function TGTI2.getNbrePointsTotal : integer;
begin
   if FDataStructure.Trig.preTrigOn=1
      then result := FNbrePoints+NbrePreAcqGTI2
      else result := FNbrePoints
end;

function TGTI2.getAcqTerminee : boolean;
begin
    result := Fnbre>=NbrePointsTotal
end;

procedure TGTI2.setCalibre15V(v : indiceVoieYD;is15V : boolean);
begin
    if is15V
       then begin
          if FdataStructure.voie[v].calibre=1 then begin
             FdataStructure.voie[v].calibre := 0;
             FdataStructure.calibrage := 1;
             FdeltaVoie[v] := deltaVgti2;
          end;
       end
       else begin
          if FdataStructure.voie[v].calibre=0 then begin
              FdataStructure.voie[v].calibre := 1;
              FdataStructure.calibrage := 1;
              FdeltaVoie[v] := deltaVgti2/2;
          end;
       end;
end;

function TGTI2.getCalibre15V(v : indiceVoieYD) : boolean;
begin
   result := FdataStructure.voie[v].calibre=0
end;

function TGTI2.getFreqEch : integer;
begin
   calculBaseDeTemps;
   result := round(freqEchOsc[codeFreq]/
            ((256-CodeReload)*CodeUnSurN))
end;

function TGTI2.AcquisitionLente : integer;
var IncNbre,i : LongInt;
    voie : indiceVoieYD;
    k : integer;
    DataChannel : data_gti;
begin
   result := 0;
   With FdataStructure.config do begin
       BaseDeTemps := 1; // 2 MHz
       Reload := 56; // divis par (256-56)=200 soit 10 kHz
       UnSurN := 10; // soit 1 kHz  soit dure totale 18 s 
   end;
   With FdataStructure.trig do begin
      TrigOn := 0; // immdiat
      PreTrigOn := 0;
      FrontMontant := 0;
      Valeur := 2048;
   end;
   Fstop := false;
   try
   repeat
      AcquisitionEnCours := _LanceAcqui(FhandleGTI2,FDataStructure)=0;
      FNbre := 0;
      repeat
         IncNbre := _GetUSBdata(FhandleGTI2,DataChannel,FDataStructure);
         IncNbre := IncNbre div FnbreVoies;
         if IncNbre>0 then begin
            for voie := low(indiceVoieYD) to high(indiceVoieYD) do if Factive[voie] then begin
                i := DataChannel.DataLen[voie];
                if i>0 then begin // le dernier point
                   dec(i);
                   k := DataChannel.BufferIn[voie,i];
                   if k>8192 then k := k-16384;
                   valeurPoint[voie] := (k-offsetVoie[voie])*FdeltaVoie[voie];
                end;   
             end;
             FNbre := FNbre + IncNbre;
         end;
       until Fstop or (Fnbre>=MaxVecteur);
    until Fstop;
    except
    end;
    StopAcquisition;
end;

procedure TGTI2.LanceAcqLente;
var code : cardinal;
begin
   if FhandleGTI2>0 then begin
      try
      StopAcqLente;
      FThreadAcqLente := CreateThread(nil,0,@ThreadAcqLente,nil,0,code);
      except
      end;
   end;
end;

Procedure TGTI2.StopAcqlente;
var code : cardinal;
    acqEnCours : boolean;
    atimer : eventTimer;
begin
      if (FhandleGTI2>0) and (FThreadAcqLente<>0) then begin
         Fstop := true;
         newTimer(Atimer,18);
         repeat
              AcqEnCours := getExitCodeThread(FThreadAcqLente,code) and
                           (code=Still_active);
         until not AcqEnCours or timerExpired(Atimer);
//         if AcquisitionEnCours then stopAcquisition; // on force
         closeHandle(FThreadAcqLente);
         FThreadAcqLente := 0;
     end;
end;

procedure TGTI2.setEtalonnage(active : boolean);
begin
    FdataStructure.calibrage := ord(active)
end;

function TGTI2.getEtalonnage : boolean;
begin
    result := FdataStructure.calibrage=1
end;

function TGTI2.getOffsetSeuil(montant : boolean) : double;
begin
     result := FOffsetSeuil[montant]
end;

procedure TGTI2.setOffsetSeuil(montant : boolean;valeur : double);
begin
     if abs(valeur)<0.5 then FOffsetSeuil[montant] := valeur
end;

procedure Tgti2.EtalonneZero(voie : indiceVoieYD;diff : boolean;var signal : boolean);

Procedure Test(is15V : boolean);
const N = 1000;
      Nmax = 2000;
var i,k : integer;
    valeur : longInt;
begin
     FoffsetVoie[voie,diff,is15V] := 0;
     calibre15V[voie] := is15V;
     LanceAcquisition;
     sleep(100);
     repeat sleep(10)
     until not getResul or (FNbre>Nmax);
     StopAcquisition;
     valeur := 0;
     for i := NbrePreAcqGTI2+1 to NbrePreAcqGTI2+N do begin
        k := FvaleurVoie[voie,i];
        if k>8192 then k := k-16384;
        valeur := valeur+k;
     end;
     valeur := valeur div N;
     signal := abs(valeur)>64;
     if not signal then FOffsetVoie[voie,diff,is15V] := valeur;
end;

var i : indiceVoieYD;
    sauveBaseDeTemps : TbaseDeTemps;
    sauve15V : boolean;
begin
     StopAcquisition;
     voieSynchro := 0; // pas de synchro
     Fdifferentiel[voie] := diff;
     for i := 0 to 3 do Factive[i] := false;
     Factive[voie] := true;
     sauveBaseDeTemps := FbaseDeTemps;
     sauve15V := Calibre15V[voie];
     FbaseDeTemps := b50m; //  50 ms
     test(false);
     test(true);
     FBaseDeTemps := sauveBaseDeTemps;
     Calibre15V[voie] := sauve15V;
end;

procedure TGTI2.store;
var Fichier : TRegistryIniFile;
    voie : indiceVoieYD;
    FichierIni : TMemIniFile;
begin
     try
     Fichier := TRegistryIniFile.create(RegistreGTI2);
     for voie := 0 to 3 do begin
         Fichier.WriteInteger('Voie'+intToStr(voie),
                              'offsetYD',FoffsetVoie[voie,true,false]);
         Fichier.WriteInteger('Voie'+intToStr(voie),
                              'offsetY',FoffsetVoie[voie,false,false]);
         Fichier.WriteInteger('Voie'+intToStr(voie),
                              'offsetYD15',FoffsetVoie[voie,true,true]);
         Fichier.WriteInteger('Voie'+intToStr(voie),
                              'offsetY15',FoffsetVoie[voie,false,true]);
     end;
     Fichier.WriteFloat('Synchro','offsetM',offsetSeuil[true]);
     Fichier.WriteFloat('Synchro','offsetD',offsetSeuil[false]);
     Fichier.WriteInteger('GTI2','Nbre',FnbrePoints);
     Fichier.free;
     except
     end;
     try
     FichierIni := TMemIniFile.create(FichierConfig);
     for voie := 0 to 3 do begin
         FichierIni.WriteInteger('Voie'+intToStr(voie),
                              'offsetYD',FoffsetVoie[voie,true,false]);
         FichierIni.WriteInteger('Voie'+intToStr(voie),
                              'offsetY',FoffsetVoie[voie,false,false]);
         FichierIni.WriteInteger('Voie'+intToStr(voie),
                              'offsetYD15',FoffsetVoie[voie,true,true]);
         FichierIni.WriteInteger('Voie'+intToStr(voie),
                              'offsetY15',FoffsetVoie[voie,false,true]);
     end;
     FichierIni.WriteFloat('Synchro','offsetM',offsetSeuil[true]);
     FichierIni.WriteFloat('Synchro','offsetD',offsetSeuil[false]);
     FichierIni.WriteInteger('GTI2','Nbre',FnbrePoints);
     FichierIni.updateFile;
     FichierIni.free;
     except
     end;
end;

procedure TGTI2.load;

procedure LoadIni;
var FichierIni : TMemIniFile;
    voie : indiceVoieYD;
begin
     try
     FichierIni := TMemIniFile.create(fichierConfig);
     for voie := 0 to 3 do begin
         FoffsetVoie[voie,true,false] := FichierIni.ReadInteger('Voie'+intToStr(voie),
                               'offsetYD',0);
         FoffsetVoie[voie,false,false] := FichierIni.ReadInteger('Voie'+intToStr(voie),
                               'offsetY',0);
         FoffsetVoie[voie,true,true] := FichierIni.ReadInteger('Voie'+intToStr(voie),
                               'offsetYD15',0);
         FoffsetVoie[voie,false,true] := FichierIni.ReadInteger('Voie'+intToStr(voie),
                               'offsetY15',0);
     end;
     offsetSeuil[false] := FichierIni.ReadFloat('Synchro','offsetM',0);
     offsetSeuil[true] := FichierIni.ReadFloat('Synchro','offsetD',0);
     FnbrePoints := FichierIni.ReadInteger('GTI2','Nbre',1000);
     fichierIni.free;
     except
     end;
end;

Procedure LoadReg;
var Fichier : TRegistryIniFile;
    voie : indiceVoieYD;
begin
     try
     Fichier := TRegistryIniFile.create(RegistreGTI2);
     for voie := 0 to 3 do begin
         FoffsetVoie[voie,true,false] := Fichier.ReadInteger('Voie'+intToStr(voie),
                               'offsetYD',0);
         FoffsetVoie[voie,false,false] := Fichier.ReadInteger('Voie'+intToStr(voie),
                               'offsetY',0);
         FoffsetVoie[voie,true,true] := Fichier.ReadInteger('Voie'+intToStr(voie),
                               'offsetYD15',0);
         FoffsetVoie[voie,false,true] := Fichier.ReadInteger('Voie'+intToStr(voie),
                               'offsetY15',0);
     end;
     offsetSeuil[false] := Fichier.ReadFloat('Synchro','offsetM',0);
     offsetSeuil[true] := Fichier.ReadFloat('Synchro','offsetD',0);
     FnbrePoints := Fichier.ReadInteger('GTI2','Nbre',1000);
     fichier.free;
     except
     end;
end;

begin
     if fileExists(fichierConfig)
        then LoadIni
        else LoadReg;
end;

procedure TGTI2.resetOffset;
var voie : indiceVoieYD;
    k1,k2 : boolean;
begin
    for voie := 0 to 3 do
        for k1 := false to true do
            for k2 := false to true do
                FoffsetVoie[voie,k1,k2] := 0;
end;

procedure TGTI2.setNbrePoints(N : integer);
begin
  if N>NombreMaxi[FbaseDeTemps] then N := NombreMaxi[FbaseDeTemps];
  if N<100 then N := 100;
  FnbrePoints := N;
  calculBaseDeTemps;
end;

procedure TGTI2.calculBaseDeTemps;
var freq : double;
begin
     if FnbrePoints>NombreMaxi[FbaseDeTemps] then FnbrePoints := NombreMaxi[FbaseDeTemps];
     Freq := FnbrePoints/dureeBalayage[FbaseDeTemps];
     CodeUnSurN := 1;
     CodeReload := 255;
     if freq>9.95e6 then begin // 10 MHz
        CodeFreq := 2;
        CodeUnSurN := 1;
        CodeReload := 255;
        FnbrePoints := trunc(FreqEchOsc[CodeFreq]*dureeBalayage[FbaseDeTemps]);
        if FnbrePoints>10000 then FnbrePoints := 10000;
        exit
     end;
     if freq>5.95e6 then begin // 6 MHz
        CodeFreq := 0;
        CodeUnSurN := 1;
        CodeReload := 255;
        FnbrePoints := trunc(FreqEchOsc[CodeFreq]*dureeBalayage[FbaseDeTemps]);
        if FnbrePoints>10000 then FnbrePoints := 10000;
        exit
     end;
     if freq>4.95e6 then begin // 5 MHz
        CodeFreq := 2;
        CodeReload := 255;
        CodeUnSurN := 2;
        FnbrePoints := trunc(FreqEchOsc[CodeFreq]*dureeBalayage[FbaseDeTemps]/2);
        if FnbrePoints>10000 then FnbrePoints := 10000;
        exit
     end;
     if freq>2.95e6 then begin // 3 MHz
        CodeFreq := 0;
        CodeReload := 255;
        CodeUnSurN := 2;
        FnbrePoints := trunc(FreqEchOsc[CodeFreq]*dureeBalayage[FbaseDeTemps]/2);
        if FnbrePoints>10000 then FnbrePoints := 10000;
        exit
     end;
     if freq>1.95e6 then begin // 2 MHz
        CodeFreq := 1;
        CodeUnSurN := 1;
        CodeReload := 255;
        FnbrePoints := trunc(FreqEchOsc[CodeFreq]*dureeBalayage[FbaseDeTemps]);
        if FnbrePoints>10000 then FnbrePoints := 10000;
        exit
     end;
     CodeFreq := 1; // 2 MHz
     CodeReload := 256-ceil(FreqEchOsc[CodeFreq]/freq);
     if codeReload>255 then codeReload := 255;
     if codeReload<56 then begin
        codeReload := 56;
        CodeUnSurN := ceil(FreqEchOsc[CodeFreq]/freq/(256-codeReload));
     end;
     FnbrePoints := trunc(FreqEchOsc[CodeFreq]*dureeBalayage[FbaseDeTemps]/(256-CodeReload)/CodeUnSurN);
end;

Procedure TGTI2.send(commande : shortString);
var i,longueur : integer;
    ET : eventTimer;
begin
    longueur := length(commande);
    for i := 0 to pred(longueur) do
        dataGet.bufferC[i] := commande[i+1];
    dataGet.buffer[longueur] := 13; // fin de chaine
    inc(longueur); 
    _SendH8(dataGet,longueur);
    sleep(longueur);
    newTimer(ET,5);
    repeat
        sleep(1)
    until (_emptyBuffer=0) or timerExpired(ET);
end;

end.

{
Code Olivier Mecacel

    Call LanceTHREAD(HandleFile)
    EnvoiCommande ('WBIN')
    EnvoiCommande ('WPE')
    EnvoiCommande ('WGOSR')
    Call GTI2.LanceAcquisitionGTI2 'lancement synchronis
    Maxi = (LONG_GTI2LENT + 1) * 2 * nbVoie2
    Debut = GetTickCount()
    strTAB = ""
    Do
      DoEvents
      strTAB = strTAB & RecoiDonne2(500)
    While (Len(strTAB) < Maxi) And ((GetTickCount - Debut) < 1500)
    
    If GetTickCount - Tempo2 < 1500 Then
      affectePoints
    endif

Je ralenti volontairement le systeme pour eviter tout chevauchement de donnes
et surtout car je voulais faire un code rapide pour tester. Ce code est
plus que probablement perfectible, surtout concernant le cadencement du
rapatriement.
 
Pour faire simple : 
 
Mode non synchronis
configure la voie rapide 
lance acquisition rapide
configure la voie lente
lance acquisition de la voie lente
recupere toutes les donnes de la voie lente
recupere au compte goutte les donnes de la voie rapide en affichant au fur
et  mesure

Mode Synchro +
configure la voie lente
lance acquisition de la voie lente
configure la voie rapide
lance acquisition rapide
recupere toutes les donnes de la voie lente
recupere au compte goutte les donnes de la voie rapide en affichant au fur
et  mesure


En fait si vous avez des donnes dans l'une ou l'autre des modules, c'est que
le trig est arriv. Le principe est que l'on met la partie lente, en mode
"stand by" et lorsque la partie rapide detecte une synchro, il donne l'ordre
 la partie lente de commencer l'acquisition.
Il y donc un petit dcalage temporel, que vous pourrez probablement corriger
graphiquement. A moins que cela ne se voit meme pas (pour des frequence
raisonnables)

}

