unit gti2_osc_Graphe;

interface

uses winTypes, winProcs, sysUtils, graphics, classes,
     Forms, Controls, Buttons, StdCtrls, ExtCtrls, Dialogs,
     math, maths_osc, UniteKer, comctrls, fft,
     GTI2_osc_util, gti2_dll, gti2_osc_Const, gti2_osc;

const
    avecValeurSynchro = true;
    flecheSeuil : array[boolean] of char =('','');

type
    Tcurseur =(curNone,curSynchro);
    Ttrace = (tLigne,tRaie,tHisto);
    Tinterpolation = (iLin,iSinc,iFFT);

    TCourbe = class(Tunite)
                 private
                 a,b : Double; { entier = rel*a+b }
                 deltaAxe,exposant,pointDebut : Double;
                 { deltaAxe = cart entre graduations sur les axes }
                 NbreChiffre : integer;
                 courbeBottom,courbeTop : integer;
                 FMini,FMaxi : Double;
                 public
                 numero : integer;
                 active : boolean;
                 signif : shortString;
                 couleur : Tcolor;
                 exposantAff : double;
                 nbreAvantVirgule : integer;
                 vecteurAff : vecteur;
                 property Mini : double read Fmini write FMini;
                 property Maxi : double read Fmaxi write FMaxi;
                 constructor create(Anom : shortString;Aunite: strUnite;Anumero : integer);
                 procedure setDelta(grandAxe : boolean);
                 Procedure setMinMax;
                 Function FormatAxe(x : Double) : strUnite;
                 Function XWindow(r : Double) : integer;
                 Function XMonde(xi : integer) : Double;
      end;

      TCourbeOsc = class(Tcourbe)
                 private
                 protected
                 public
                 diffRG : TradioGroup;
                 shape : Tshape;
                 labelValeur : Tlabel;
                 zeroCB : TcheckBox;
                 calibreCB : TcomboBox;
                 GBvoie : TgroupBox;
                 valeurAff : double;
                 Function calibreVoie : strUnite;
                 Function AffVoltmetre : strUnite;
      end;

      TgrapheOsc = class
                 private
                 public
                 NbreMaxi : integer;
                 Canvas : Tcanvas;
                 penWidth : integer;
                 monocoup : boolean;
                 AnalyseurLogique : boolean;
                 PaintBox : TpaintBox;
                 avecAxeX,avecAxeY : boolean;
                 grandAxeX,grandAxeY : boolean;
                 NbreCourbes : integer;
                 voieOrphy : array[indiceVoieYD] of TcourbeOsc;
                 voieDB15 : array[indiceVoieYD] of TvoieDB15;
                 voieGTS2 : array[indiceVoieYD] of TcourbeOsc;
                 voieTemps : Tcourbe;
                 voieTempsGTS2 : Tcourbe;
                 FseuilSynchro : double;
                 montant,preTrig : boolean;
                 seuilAff : integer;
                 TestPente : boolean;
                 voieSynchro : indiceVoieOsc;
                 LimiteCourbe,LimiteFenetre : Trect;
                 LargCarac,HautCarac,Marge : integer;
                 Nbre,NbreGTS2,NbreAff,NbreTest : indiceVecteur;
                 idebut,ifin,idebutGTS2 : integer;
                 xAxeY : integer;
                 penteCorrecte,seuilTrouve : boolean;
                 trace : Ttrace;
                 avecGTS2 : boolean;
                 interpolation : Tinterpolation;
                 vecteurTestT,vecteurTestS : vecteur;
                 Constructor Create;
                 Procedure Draw;
                 Procedure TraceCourbes;
                 procedure TraceFragments(n1 :integer);
                 Procedure TraceSynchro(y : integer;avecFleche : boolean);
                 procedure MyLine(x1,y1,x2,y2 : integer);
                 Function GetCourbeProche(var x,y : Integer;
                                          var voie : indiceVoieYD;
                                          var isGTS2 : boolean) : boolean;
                 Procedure SetOrigineTemps;
                 Procedure TestSeuil;
                 Procedure SetOrigineGTS2;
                 Procedure TraceReticule(voie : indiceVoieYD);
                 procedure SetHauteurFonte(var H : integer);
                 property SeuilSynchro : double read FSeuilSynchro write FSeuilSynchro;
                 Function VerifPente : boolean;
                 Function isAxeX(x,y : integer) : boolean;
                 procedure changeEchelleX(xnew,xold : integer);
                 Procedure PointProche(var x,y : integer;var IndexVoie,IndexPoint : integer);
                 Procedure SetIndexPeriode(var indexPeriode : integer);
                 destructor Destroy; override;
end;

var  grapheOsc : TgrapheOsc;
     grapheFFT : TgrapheOsc;
     tension,frequence,temps : Tunite;

const
    CouleurDefaut : array[indiceVoieYD] of Tcolor =
       (clBlue,clRed,clFuchsia,clLime);
    CouleurGTS2 : array[indiceVoieYD] of Tcolor =
       (clGreen,clTeal,clPurple,clBlack);


IMPLEMENTATION

Function Tcourbe.XMonde(xi : integer) : double;
begin
     result := (xi-B)/A
end;

Function Tcourbe.XWindow(r : double) : integer;
begin
      if r=sansSignif
         then result := IntSansSignif
         else try
            result := round(A*r+B)
            except
               result := IntSansSignif
            end
end;

procedure TgrapheOsc.SetHauteurFonte(var H : integer);
begin
     H := H * 72 div canvas.font.PixelsPerInch;
     H := 2 * (H div 2);
     if H<=8 then H := 8;
     if H>=12 then H := 12;
     canvas.font.name := 'Regressi';
     canvas.font.size := H;
end;

procedure TgrapheOsc.MyLine(x1,y1,x2,y2 : integer);
begin
     canvas.moveTo(x1,y1);
     canvas.lineTo(x2,y2);
end;

procedure TgrapheOsc.Draw;

procedure ChercheMonde;
var voie : indiceVoieYD;
    N,Nc,hauteurCourbe : integer;
begin
     if AnalyseurLogique then begin
        NbreCourbes := 0;
        for voie := low(indiceVoieYD) to high(indiceVoieYD) do
            if voieOrphy[voie].active then inc(NbreCourbes);
        if (voieSynchro<>voieT) and avecGTS2 then
        for voie := low(indiceVoieYD) to high(indiceVoieYD) do
            if voieGTS2[voie].active then inc(NbreCourbes);
        N := NbreCourbes;
     end
     else N := 1;
     with limiteFenetre do begin
         avecAxeY := ((bottom-top) div 5>NbreGradMin*hautCarac*N div 4);
         avecAxeX := (right-left)>NbreGradMin*4*largCarac;
         grandAxeY := (bottom-top) div 5>NbreGraduations*HautCarac*N div 4;
         grandAxeX := (right-left)>NbreGraduations*4*largCarac;
     end;
     limiteCourbe := limiteFenetre;
     inc(limiteCourbe.left,marge);
     if avecAxeY then
        inc(limiteCourbe.left,LongNombreAxe*largCarac);
     if avecAxeX then
         dec(limiteCourbe.bottom,hautCarac+marge);
     for voie := low(indiceVoieYD) to high(indiceVoieYD) do with voieOrphy[voie] do begin
         CourbeTop := LimiteCourbe.top;
         CourbeBottom := LimiteCourbe.bottom;
     end;
     if voieSynchro<>voieT then
     for voie := low(indiceVoieYD) to high(indiceVoieYD) do with voieGTS2[voie] do begin
         CourbeTop := LimiteCourbe.top;
         CourbeBottom := LimiteCourbe.bottom;
     end;
     if AnalyseurLogique then begin
        hauteurCourbe := (LimiteCourbe.bottom-LimiteCourbe.top) div NbreCourbes - marge;
        Nc := 0;
        for voie := low(indiceVoieYD) to high(indiceVoieYD) do with voieOrphy[voie] do
            if active then begin
               CourbeTop := LimiteCourbe.top+Nc*hauteurCourbe+marge;
               inc(Nc);
               CourbeBottom := CourbeTop+hauteurCourbe;
            end;
        if (voieSynchro<>voieT) and avecGTS2 then
        for voie := low(indiceVoieYD) to high(indiceVoieYD) do with voieGTS2[voie] do
            if active then begin
               CourbeTop := LimiteCourbe.top+Nc*hauteurCourbe+marge;
               inc(Nc);
               CourbeBottom := CourbeTop+hauteurCourbe;
            end;
     end;
     with voieTemps do begin
            setMinMax;
            if (FMaxi-FMini)<1e-6 then FMaxi := FMini+1e-6;
            A := (limiteCourbe.right-limiteCourbe.left)/(Fmaxi-Fmini);
            B := limiteCourbe.left-mini*A;
            setDelta(grandAxeX);
     end;
     if voieSynchro<>voieT then
     with voieTempsGTS2 do begin
            mini := voieTemps.mini;
            maxi := voieTemps.Maxi;
            setMinMax;
            if (FMaxi-FMini)<1e-6 then FMaxi := FMini+1e-6;
            A := (limiteCourbe.right-limiteCourbe.left)/(Fmaxi-Fmini);
            B := limiteCourbe.left-mini*A;
     end;
     for voie := low(indiceVoieYD) to high(indiceVoieYD) do with voieOrphy[voie] do
         if active then begin
               setMinMax;
               A := (CourbeTop-CourbeBottom)/(FMaxi-FMini);
               B := CourbeBottom-FMini*A;
               setDelta(grandAxeY);
         end;
     if (voieSynchro<>voieT) and avecGTS2 then
     for voie := low(indiceVoieYD) to high(indiceVoieYD) do with voieGTS2[voie] do
         if active then begin
               setMinMax;
               A := (CourbeTop-CourbeBottom)/(FMaxi-FMini);
               B := CourbeBottom-FMini*A;
               setDelta(grandAxeY);
         end;
end; // ChercheMonde

procedure DrawAxis;// Trac des axes 
var yAxeX : integer;
    xNombreMax : integer;
    PointCourant : double;

Function YcorrectNombre(y : integer;voie : indiceVoieYD) : boolean;
begin with voieOrphy[voie] do begin
     YcorrectNombre := ( y>(CourbeTop+2*HautCarac)) and
                       ( y<(CourbeBottom-marge)) and
                       ( abs(y-yAxeX)>marge )
end end;

Function YcorrectNombreGTS2(y : integer;voie : indiceVoieYD) : boolean;
begin with voieGTS2[voie] do begin
     result := ( y>(CourbeTop+2*HautCarac)) and
               ( y<(CourbeBottom-marge)) and
               ( abs(y-yAxeX)>marge )
end end;

Function YcorrectGrad(y : integer;voie : indiceVoieYD) : boolean;
begin with voieOrphy[voie] do
     result := (y>(courbeTop+HautCarac)) and
                     (y<(CourbeBottom-HautCarac))
end;

Function YcorrectGradGTS2(y : integer;voie : indiceVoieYD) : boolean;
begin with voieGTS2[voie] do
     result := (y>(courbeTop+HautCarac)) and
                     (y<(CourbeBottom-HautCarac))
end;

Function XcorrectNombre(x : integer) : boolean;
begin
     XcorrectNombre := (x>(limiteCourbe.left+marge)) and
                       (x<xNombreMax) and
                       (abs(x-xAxeY)>marge )
end;

Function XcorrectGrad(x : integer) : boolean;
begin
     XcorrectGrad := (x>limiteCourbe.left) and (x<limiteCourbe.right)
end;

Procedure graduationX(x : integer);
begin
     if grille
        then MyLine(x,limiteCourbe.top,x,limiteCourbe.bottom)
        else MyLine(x,yAxeX-marge,x,yAxeX+marge)
end;

Procedure graduationY(y : integer);
begin
     if grille
        then MyLine(limiteCourbe.left,y,limiteCourbe.right,y)
        else MyLine(xAxeY-marge,y,xAxeY+marge,y)
end;

  procedure initAxeX(voie : Tcourbe);
  var x1 : integer;
  begin with voie do begin
         PointCourant := PointDebut;
         x1 := XWindow(0);
         canvas.font.Color := clBlack;
         canvas.pen.color := clBlack;
         if (x1>limiteCourbe.left) and (x1<limiteCourbe.right) then begin
             canvas.pen.style := psDash;
             Myline(x1,limiteCourbe.bottom,x1,limiteCourbe.top);
         end;
         canvas.pen.style := psSolid;
         xNombreMax := limiteFenetre.right-largCarac*longNombreAxe;
         with limiteFenetre do begin
               MyLine(left,yAxeX,right,yAxeX);
          { flches }
               MyLine(right-marge*3,yAxeX+marge,right,yAxeX);
               MyLine(right-marge*3,yAxeX-marge,right,yAxeX);
         end;
  end end; // initAxeX

  procedure traceAxeX(voie : Tcourbe);
  var xs : integer;
      NombreAff : strUnite;
  begin with voie do begin
     SetTextAlign(canvas.handle,TA_center+TA_top);
     if grille
        then begin
           canvas.pen.style := psDot;
           canvas.pen.color := clGray;
        end
        else canvas.pen.style := psSolid;
     repeat
          xs := Xwindow(PointCourant);
          if XcorrectGrad(xs) then begin
               graduationX(xs);
               if XcorrectNombre(xs) then begin
                  NombreAff := formatAxe(pointCourant/exposant);
                  canvas.TextOut(xs,yAxeX+marge,NombreAff);
             end;
          end;
          PointCourant := PointCourant+deltaAxe;
     until ( PointCourant>=maxi );
     canvas.pen.style := psSolid;
     canvas.pen.color := clBlack;
     setTextAlign(canvas.handle,TA_right+TA_top);
     canvas.TextOut(limiteFenetre.right-marge,yAxeX+marge,nomAxe(exposant));
     SetTextAlign(canvas.handle,TA_left+TA_top);
 end end; // traceAxeX

 procedure initAxeY(voie : indiceVoieYD);
 var y1 : integer;
 begin with voieOrphy[voie] do begin
        PointCourant := PointDebut;
        y1 := XWindow(0);
        canvas.Pen.color := couleur;
        if (y1>CourbeTop) and (y1<CourbeBottom)then
            Myline(limiteCourbe.left,y1,limiteCourbe.right,y1);
        MyLine(xAxeY,yAxeX,xAxeY,CourbeTop);
              { flches }
        MyLine(xAxeY-marge,courbeTop+3*marge,xAxeY,courbeTop);
        MyLine(xAxeY+marge,courbeTop+3*marge,xAxeY,courbeTop);
 end end; // InitAxeY

 procedure traceAxeY(voie : indiceVoieYD);
 var ys,margeY : integer;
     nombreAff : strUnite;
 begin with voieOrphy[voie] do begin
      canvas.font.Color := Couleur;
      if grille
         then begin
             canvas.pen.style := psDot;
             canvas.pen.color := clGray;
         end
         else canvas.pen.style := psSolid;
      SetTextAlign(canvas.handle,TA_right+TA_bottom);
      margeY := -2*marge;
      repeat
           ys := Xwindow(PointCourant); // coordonne cran de marque
         if YcorrectGrad(ys,voie) then begin
            graduationY(ys);
             if YcorrectNombre(ys,voie) then begin
              if abs(pointCourant) < (FMaxi-FMini)/1000 then pointCourant := 0;
                  NombreAff := formatAxe(pointCourant/exposant);
            	    Canvas.TextOut(xAxeY+margeY,ys+marge,NombreAff);
              end;
       end;
       pointCourant := pointCourant+deltaAxe;
      until pointCourant>=Maxi;
      canvas.pen.style := psSolid;
      SetTextAlign(canvas.handle,TA_right+TA_top);
      canvas.TextOut(xAxeY+margeY,CourbeTop,nomAxe(exposant));
 end end; // traceAxeY

  procedure initAxeGTS2(voie : indiceVoieYD);
 var y1 : integer;
 begin with voieGTS2[voie] do begin
        PointCourant := PointDebut;
        y1 := XWindow(0);
        canvas.Pen.color := couleur;
        if (y1>CourbeTop) and (y1<CourbeBottom)then
            Myline(limiteCourbe.left,y1,limiteCourbe.right,y1);
        MyLine(xAxeY,yAxeX,xAxeY,CourbeTop);
              { flches }
        MyLine(xAxeY-marge,courbeTop+3*marge,xAxeY,courbeTop);
        MyLine(xAxeY+marge,courbeTop+3*marge,xAxeY,courbeTop);
 end end; // InitAxeGTS2

 procedure traceAxeGTS2(voie : indiceVoieYD);
 var ys,margeY : integer;
     nombreAff : strUnite;
 begin with voieGTS2[voie] do begin
      canvas.font.Color := Couleur;
      if grille
         then begin
             canvas.pen.style := psDot;
             canvas.pen.color := clGray;
         end
         else canvas.pen.style := psSolid;
      SetTextAlign(canvas.handle,TA_right+TA_bottom);
      margeY := -2*marge;
      repeat
         ys := Xwindow(PointCourant); // coordonne cran de marque
         if YcorrectGradGTS2(ys,voie) then begin
            graduationY(ys);
             if YcorrectNombreGTS2(ys,voie) then begin
              if abs(pointCourant) < (FMaxi-FMini)/1000 then pointCourant := 0;
                  NombreAff := formatAxe(pointCourant/exposant);
                  Canvas.TextOut(xAxeY+margeY,ys+marge,NombreAff);
              end;
       end;
       pointCourant := pointCourant+deltaAxe;
      until pointCourant>=Maxi;
      canvas.pen.style := psSolid;
      SetTextAlign(canvas.handle,TA_right+TA_top);
      canvas.TextOut(xAxeY+margeY,CourbeTop,nomAxe(exposant));
 end end; // traceAxeYGTS2

 procedure zeroX(voie : Tcourbe);
var x1 : integer;
begin with voie do
   x1 := XWindow(0);
   if (x1>limiteCourbe.left) and (x1<limiteCourbe.right) then begin
        canvas.pen.style := psDot;
        canvas.pen.color := clBlack;
        Myline(x1,limiteCourbe.bottom,x1,limiteCourbe.top);
   end;
   canvas.pen.style := psSolid;   
end; // zeroX

procedure zeroY(voie : indiceVoieYD);
var y1 : integer;
begin with voieOrphy[voie] do begin
     y1 := Xwindow(0);
     if (y1>CourbeTop) and (y1<CourbeBottom) then begin
          canvas.pen.style := psDot;
          canvas.pen.color := voieOrphy[voie].couleur;
          Myline(limiteCourbe.left,y1,limiteCourbe.right,y1);
     end;
     canvas.pen.style := psSolid;
end end;{zeroY}

procedure zeroGTS2(voie : indiceVoieYD);
var y1 : integer;
begin with voieGTS2[voie] do begin
     y1 := Xwindow(0);
     if (y1>CourbeTop) and (y1<CourbeBottom) then begin
          canvas.pen.style := psDot;
          canvas.pen.color := voieOrphy[voie].couleur;
          Myline(limiteCourbe.left,y1,limiteCourbe.right,y1);
     end;
     canvas.pen.style := psSolid;
end end;{zeroY}

var voie : indiceVoieYD;
begin // DrawAxis
     yAxeX := limiteCourbe.bottom;
     xAxeY := limiteCourbe.left;
     if avecAxeX then begin
          initAxeX(voieTemps);
          traceAxeX(voieTemps);
          zeroX(voieTemps);
     end;
     if avecAxeY then
        if analyseurLogique or (voieSynchro=voieT) then begin
        for voie := low(indiceVoieYD) to high(indiceVoieYD) do
            if voieOrphy[voie].active then begin
                 initAxeY(voie);
                 traceAxeY(voie);
                 if not (trace in [tHisto,tRaie]) then zeroY(voie);
                 if not analyseurLogique then break;
            end;
        if analyseurLogique and avecGTS2 then
           for voie := low(indiceVoieYD) to high(indiceVoieYD) do
            if voieGTS2[voie].active then begin
                 initAxeGTS2(voie);
                 traceAxeGTS2(voie);
                 if not (trace in [tHisto,tRaie]) then zeroGTS2(voie);
            end;
        end
        else begin
             initAxeY(voieSynchro);
             traceAxeY(voieSynchro);
        end;
     if not (trace in [tHisto,tRaie]) then
        for voie := low(indiceVoieYD) to high(indiceVoieYD) do
            if voieOrphy[voie].active then zeroY(voie);
     SetTextAlign(canvas.handle,TA_left+TA_top);
end;// DrawAxis 

begin
     limiteFenetre := rect(0,0,PaintBox.width,PaintBox.height);
     with limiteFenetre do HautCarac := (bottom-top) div 50;
     setHauteurFonte(HautCarac);
     LargCarac := 2*HautCarac div 3;
     Marge := HautCarac div 3;
     canvas.Brush.color := clWindow;
     canvas.FillRect(limiteFenetre);
     canvas.pen.mode := pmCopy;
     canvas.pen.color := clBlack;
     canvas.pen.width := 1;
     setBkMode(canvas.handle,transparent);
     chercheMonde;
     drawAxis;
     if (voieSynchro<>voieT) and (trace=tLigne) then begin
        seuilAff := voieOrphy[voieSynchro].Xwindow(FseuilSynchro);
        traceSynchro(seuilAff,true);
     end;
     if Nbre>0 then TraceCourbes;
     canvas.pen.style := psSolid;
end;

procedure TgrapheOsc.traceSynchro(y : integer;avecFleche : boolean);
begin
    if (voieSynchro<>voieT) and not (trace in [tHisto,tRaie]) then
       with voieOrphy[voieSynchro] do begin
            canvas.pen.mode := pmNotXOR;
            canvas.pen.style := psDashDot;
            canvas.pen.color := couleur;
            Myline(0,y,limiteCourbe.right,y);
            if avecValeurSynchro then begin
               canvas.font.color := couleur;
               canvas.font.style := [fsBold];
               if avecFleche
                  then canvas.textOut(limiteFenetre.left+largCarac,
                           y-abs(2*canvas.font.height div 3),
                           flecheSeuil[montant])
                  else begin
                      canvas.Brush.Style := bsSolid;
                      canvas.Brush.Color := clWindow;
                      canvas.FillRect(rect(limiteFenetre.left+largCarac,y-hautCarac,
                                      limiteFenetre.left+2*largCarac,y+hautCarac));
                  end;
               canvas.font.style := [];
{
               canvas.font.size := 10;
               canvas.TextOut(limiteCourbe.left+4,
                           y-abs(canvas.font.height),
                           NomValeurUnite(Xmonde(y)));
}
            end;
            canvas.pen.style := psSolid;
            canvas.pen.mode := pmCopy;
       end;
end;

procedure TgrapheOsc.TraceCourbes;

Procedure traceLignesTemps(voie : indiceVoieYD);
var i : integer;
    xi,yi : integer;
begin
     idebut := 0;
     with voieTemps do xi := XWindow(vecteurAff[0]);
     while xi<xAxeY do begin
          inc(idebut);
          with voieTemps do xi := XWindow(vecteurAff[idebut]);
     end;
// xi>=xAxeY donc >=0 si pas de preacq     
     with voieOrphy[voie] do yi := XWindow(vecteurAff[idebut]);
     canvas.MoveTo(xi,yi);
     i := idebut+1;
     repeat
        with voieTemps do xi := XWindow(vecteurAff[i]);
        with voieOrphy[voie] do yi := XWindow(vecteurAff[i]);
        canvas.LineTo(xi,yi);
        inc(i);
     until (i>=Nbre) or (xi>=limiteFenetre.right);
     ifin := pred(i);
     if idebut>0 then dec(idebut);
end; // traceLignesTemps

Procedure traceLignesGTS2(voie : indiceVoieYD);
var i,i0 : integer;
    xi,yi : integer;
begin
     i0 := 0;
     with voieTemps do xi := XWindow(vecteurAff[i0]);
     while xi<xAxeY do begin
          inc(i0);
          with voieTemps do xi := XWindow(vecteurAff[i0]);
     end;
     with voieGTS2[voie] do yi := XWindow(vecteurAff[i0]);
     canvas.MoveTo(xi,yi);
     i := i0+1;
     repeat
        with voieTemps do xi := XWindow(vecteurAff[i]);
        with voieGTS2[voie] do yi := XWindow(vecteurAff[i]);
        canvas.LineTo(xi,yi);
        inc(i);
     until (i>=Nbre) or (xi>=limiteFenetre.right);
end; // traceLignesGTS2

Procedure traceHarmoniques(voie : indiceVoieYD);
var i : integer;
    xi,yi,y0 : integer;
begin
     with voieOrphy[voie] do y0 := XWindow(0);
     canvas.brush.color := voieOrphy[voie].couleur;
     for i := 0 to pred(Nbre) do begin
        with voieTemps do xi := XWindow(vecteurAff[i]);
        with voieOrphy[voie] do yi := XWindow(vecteurAff[i]);
        canvas.FillRect(rect(xi-penWidth,yi,xi+penWidth,y0));
     end;
end; // traceHarmoniques

Procedure traceHarmoniquesGTS2(voie : indiceVoieYD);
var i : integer;
    xi,yi,y0 : integer;
begin
     with voieGTS2[voie] do y0 := XWindow(0);
     canvas.brush.color := voieOrphy[voie].couleur;
     for i := 0 to pred(Nbre) do begin
        with voieTemps do xi := XWindow(vecteurAff[i]);
        with voieGTS2[voie] do yi := XWindow(vecteurAff[i]);
        canvas.FillRect(rect(xi-penWidth,yi,xi+penWidth,y0));
     end;
end; // traceHarmoniquesGTS2

Procedure traceSpectre(voie : indiceVoieYD);
var i : integer;
    xi,yi,y0 : integer;
begin
     with voieOrphy[voie] do y0 := XWindow(0);
     for i := 0 to pred(Nbre) do begin
        with voieTemps do xi := XWindow(vecteurAff[i]);
        with voieOrphy[voie] do yi := XWindow(vecteurAff[i]);
        canvas.moveTo(xi,y0);
        canvas.lineTo(xi,yi);
     end;
end; // traceSpectre

Procedure traceSpectreGTS2(voie : indiceVoieYD);
var i : integer;
    xi,yi,y0 : integer;
begin
     with voieGTS2[voie] do y0 := XWindow(0);
     for i := 0 to pred(Nbre) do begin
        with voieTemps do xi := XWindow(vecteurAff[i]);
        with voieGTS2[voie] do yi := XWindow(vecteurAff[i]);
        canvas.moveTo(xi,y0);
        canvas.lineTo(xi,yi);
     end;
end; // traceSpectreGTS2

var voie : indiceVoieYD;
begin
     case trace of
          tHisto : begin
             penWidth := PaintBox.Width div (3*NbreAff);
             if penWidth>5 then penWidth := 5;
          end;
          tRaie : penWidth := 1;
     end;
     canvas.pen.width := penWidth;
     for voie := low(indiceVoieYD) to high(indiceVoieYD) do
         if voieOrphy[voie].active then begin
             canvas.pen.color := voieOrphy[voie].couleur;
             case trace of
                tLigne : TraceLignesTemps(voie);
                tRaie : TraceSpectre(voie);
                tHisto : TraceHarmoniques(voie);
             end;
         end;
     if (voieSynchro<>voieT) and avecGTS2 then
     for voie := voieG to voieJ do
         if voieGTS2[voie].active then begin
             canvas.pen.color := voieGTS2[voie].couleur;
             case trace of
                tLigne : TraceLignesGTS2(voie);
                tRaie : TraceSpectreGTS2(voie);
                tHisto : TraceHarmoniquesGTS2(voie);
             end;
         end;
     canvas.pen.width := 1;
end; // raceCourbes

procedure TgrapheOsc.TraceFragments(n1 :integer);

Procedure traceLignesTemps(voie : indiceVoieYD);
var i : integer;
    xi,yi : integer;
begin
     canvas.pen.color := voieOrphy[voie].couleur;
     with voieOrphy[voie] do yi := XWindow(vecteurAff[pred(n1)]);
     with voieTemps do xi := XWindow(vecteurAff[pred(n1)]);     
     canvas.MoveTo(xi,yi);
     for i := n1 to pred(nbre) do begin
        with voieTemps do xi := XWindow(vecteurAff[i]);
        with voieOrphy[voie] do yi := XWindow(vecteurAff[i]);
        if (xi<>intSansSignif) and
           (yi<>intSansSignif) then canvas.LineTo(xi,yi);
     end;
     with voieOrphy[voie] do begin
         valeurAff := vecteurAff[pred(Nbre)];
         LabelValeur.caption := AffVoltmetre;
    end;     
end; // traceLignesTemps

var voie : indiceVoieYD;
    xi : integer;
begin
     canvas.pen.width := penWidth;
     if n1>0 then dec(n1);
     repeat
        with voieTemps do xi := XWindow(vecteurAff[n1]);
        inc(n1);
     until xi>=xAxeY;
     for voie := low(indiceVoieYD) to high(indiceVoieYD) do
         if voieOrphy[voie].active then
             TraceLignesTemps(voie);
     canvas.pen.width := 1;
end;

destructor TgrapheOsc.destroy;
var i : indiceVoieYD;
begin
     for i := low(indiceVoieYD) to high(indiceVoieYD) do voieOrphy[i].free;
     voieTemps.free;
     inherited destroy;
end;

constructor TgrapheOsc.Create;
var voie : indiceVoieYD;
begin
  inherited create;
  penWidth := 1;
  tension := Tunite.create;
  tension.nomUnite := 'V';
  frequence := Tunite.create;
  frequence.nomUnite := 'Hz';
  frequence.nom := 'f';
  temps := Tunite.create;
  temps.nomUnite := 's';
  temps.nom := 't';
  nbreMaxi := 5000;
  voieTemps := Tcourbe.create('t','s',0);
  voieTempsGTS2 := Tcourbe.create('t','s',0);  
  for voie := low(indiceVoieYD) to high(indiceVoieYD) do begin
      voieOrphy[voie] := TcourbeOsc.create('V'+intToStr(voie),'V',voie);
      voieGTS2[voie] := TcourbeOsc.create('V'+char(ord('G')+voie-voieG),'V',voie);
      voieGTS2[voie].couleur := CouleurGTS2[voie];
      voieDB15[voie] := TvoieDB15.create;
  end;    
end;

constructor Tcourbe.create(Anom : shortString;Aunite: strUnite;Anumero :integer);
begin
     inherited create;
     A := 1;
     B := 0;
     nom := Anom;
     NomUnite := Aunite;
     FMaxi := 15;
     FMini := -15;
     exposantAff := 1;
     NbreAvantVirgule := 0;
     signif := '';
     couleur := CouleurDefaut[Anumero];
     numero := Anumero;
     if couleur=clWindow then couleur := clActiveCaption;
end;

procedure Tcourbe.SetDelta(grandAxe : boolean);
{ Exposant=puissance de 10 multiple de 3 de manire  avoir entre
NbreGraduations (10) et N/2.5 (4) graduations sur l'axes.
ou NbreGradMin (5) et N/2.5 (2) graduations sur l'axes.
Delta est l'intervalle entre les graduations qui commencent  PremiereGraduation }
const coeff : array[0..5] of double = (0.1,0.2,0.5,1,2,5);
var  AbsMax,AbsMin,largeur : double;
     Exponent,index : integer;
     N : byte;
begin
      AbsMax := abs(Fmaxi);AbsMin := abs(Fmini);
      if AbsMax<AbsMin then AbsMax := AbsMin;
      Exponent := floor(Log10(AbsMax));
      largeur := (Fmaxi-Fmini)*dix(-exponent);
      index := 0;
      if grandAxe
        then while ( (largeur/coeff[index])>=NbreGraduations ) and (index<5) do inc(index)
        else while ( (largeur/coeff[index])>=NbreGradMin ) and (index<5) do inc(index);
      DeltaAxe := coeff[index]*dix(exponent);
      NbreChiffre := ceil(Log10(AbsMax))-floor(Log10(DeltaAxe));
      PointDebut := ceil(Fmini/deltaAxe)*deltaAxe;
      Index := Exponent div 3;
      if ( (Exponent mod 3) < 0) then dec(index);
  { la division euclidienne des informaticiens <> div des maths ! }
      Exposant := Dix(3*Index);
      N := ceil(Log10(AbsMax/Exposant));
      if N>NbreChiffre then NbreChiffre := N;
      if NbreChiffre<3 then NbreChiffre := 3;
      if (NbreChiffre>longNombreAxe) then
         NbreChiffre := longNombreAxe;
end;

Function Tcourbe.FormatAxe(x : double) : strUnite;
begin
     if abs(x*exposantAff)<((Fmaxi-Fmini)/1000)
         then FormatAxe := '0'
         else FormatAxe := FloatToStrF(x,ffGeneral,NbreChiffre,0)
end;

Function TgrapheOsc.GetCourbeProche(var x,y : Integer;
         var voie : indiceVoieYD;
         var isGTS2 : boolean) : boolean;
var
   i : integer;
   Xi,Yi : Integer;
   newX,newY : Integer;
   distance,distanceMin : integer;
   v : indiceVoieYD;
begin
  result := false;
  isGTS2 := false;
  distanceMin := 64;
  newX := X;
  newY := Y;
  for v := low(indiceVoieYD) to high(indiceVoieYD) do
      with voieOrphy[v] do if active then begin
      for i := 0 to pred(nbre) do begin
        Xi := voieTemps.Xwindow(voieTemps.vecteurAff[i]);
        Yi := Xwindow(vecteurAff[i]);
        distance := abs(X-Xi)+abs(Y-Yi);
        if distance<distanceMin then begin
           distanceMin := distance;
           result := true;
           voie := v;
           newX := Xi;
           newY := Yi;
        end;
      end; // for i
   end;// with voie GTI2
  for v := voieG to voieJ do
      with voieGTS2[v] do if active then begin
      for i := 0 to pred(nbre) do begin
        Xi := voieTemps.Xwindow(voieTemps.vecteurAff[i]);
        Yi := Xwindow(vecteurAff[i]);
        distance := abs(X-Xi)+abs(Y-Yi);
        if distance<distanceMin then begin
           distanceMin := distance;
           result := true;
           voie := v;
           newX := Xi;
           newY := Yi;
           isGTS2 := true;
        end;
      end; // for i
   end;// with voie GTS2
   X := newX;
   Y := newY;
end;

const
   degre = 1;
   ordre = degre+1;

type
   matrice = array[1..ordre,1..ordre+1] of double;
   matriceLigne = array[1..ordre] of double;
   EsystemeLineaireError = class(Exception);
   vectorPuiss = array[0..2*degre] of double;

Procedure Resolution(A : matrice;var Result : matriceLigne);
 { Cette procdure rsoud un systme AX = B }
 { crit sous la forme AX = A(n+1) }
 { par la mthode d'limination de Gauss (pivot partiel) }

  Procedure Elimination;

    FUNCTION Pivot(i :integer):integer;
    Var n,temp : integer;
    BEGIN
      temp := i;
      for n := i+1 to ordre do
          if abs(A[n,i]) > abs(A[i,i]) then temp:= n;
      pivot:=temp;
      if abs(A[temp,i]) <= 1e-100 then
         raise EsystemeLineaireError.Create('Erreur rsolution systme linaire')
    END;

    PROCEDURE Permute(i,j:integer);
    Var temp:double;
        k:integer;
    BEGIN
       if i<>j then For k:=i to ordre+1 do BEGIN
          temp:=A[i,k];
          A[i,k]:=A[j,k];
          A[j,k]:=temp;
       END;
    END;

  var I,J,K : integer;
      M : double;
  begin
    for k := 1 to ordre do begin
          Permute(k,pivot(k));
          for i:=k+1 to ordre do begin
              M := A[i,k]/A[k,k];
              for j:=k+1 to ordre+1 do A[i,j]:=A[i,j]-M*A[k,j];
              A[i,k]:=0.0;
          end;
       end;
    end; {Elimination}

    Procedure Substitution;
    var I,J : integer;
        S : double;
    begin
         try
         for I:=ordre downto 1 do begin
             S:=A[I,ordre+1];
             for J:=I+1 to ordre do S:=S-A[I,J]*Result[J];
             Result[I]:=S/A[I,I];
         end;
         except
            on EzeroDivide do raise EsystemeLineaireError.Create('Erreur rsolution systme linaire')
         end;
    end;

begin
  Elimination;
  Substitution;
end; // rsolution

Procedure TgrapheOsc.SetOrigineTemps;

function ChercheDroite(const Yr : vecteur;debut : integer;var origine,pente : double) : boolean;
const N = 8;
var puissX : vectorPuiss;
    A : matrice;
    Coeff : matriceLigne;
    puissMax : byte;

procedure initXY;
var j : integer;
    loc : double;
    i : byte;
begin
    for i := 1 to puissMax do puissX[i] := 0;
    puissX[0] := N;
    for i := 1 to ordre do A[i,ordre+1] := 0;
    for j := debut to pred(debut+N) do begin
        loc := j;
        for i := 1 to puissMax do begin
            puissX[i] := puissX[i]+loc;
            loc := loc*j;
        end;
        loc := yr[j];
        for i := 1 to ordre do begin
            A[i,ordre+1] := A[i,ordre+1]+loc;
            loc := loc*j;
        end;
    end;
end;

procedure AffecteCoeff;
var i,j : byte;
begin
     for i := 1 to ordre do
         for j := 1 to ordre do
             A[i,j] := PuissX[i+j-2];
     try
     Resolution(A,Coeff);
     except
         coeff[2] := 0;
         coeff[1] := 0;
     end;
end;

Function CalcCorrelation : boolean;
var i : integer;
    xsigma,ysigma,xmoy,ymoy : double;
    Sx,Sy,Sx2,Sy2,Sxx,Syy : double;
    cor : double;
begin
    result := false;
    try
    Sx := 0;Sy := 0;
    Sx2 := 0;Sy2 := 0;
    for  i := Debut to pred(Debut+N) do begin
       Sx := Sx+i;Sy := Sy+Yr[i];
       Sx2 := Sx2+i*i;Sy2 := Sy2+sqr(Yr[i]);
    end;
    Sxx := sx2-sqr(sx)/N;
    Syy := sy2-sqr(sy)/N;
    Xsigma := sqrt(Sxx/pred(N));
    Ysigma := sqrt(Syy/pred(N));
    Xmoy := Sx/N;Ymoy := Sy/N;
    Cor := 0;
    for i := Debut to pred(Debut+N) do
        Cor := Cor+(i-Xmoy)*(Yr[i]-Ymoy);
    Cor := Cor/pred(N);
    Cor := Cor/Xsigma/Ysigma;
    result := abs(Cor)>0.9;
    except
    end;
end; // Calcul stat deux variables 

begin // ChercheDroite
    result := false;
    if (debut+N)>nbre then exit; // points en dehors des donnes
    if N>((nbre-debut) div 8) then exit; // analyse non locale
    puissMax := 2*degre;
    initXY;
    affecteCoeff;
    pente := coeff[2];
    origine := coeff[1];
    result := calcCorrelation;
end; // ChercheDroite

var i,j : integer;
    frontAtteint : boolean;
    pente : double;
    Deltat : double;
    origine : double;
begin
if (voieSynchro=voieT) then exit; // or (NbreVoiesGTS2>0)
with voieOrphy[voieSynchro] do begin
         i := 0;
         frontAtteint := abs(vecteurAff[i]-FseuilSynchro)<(maxi/100);
         if frontAtteint then if montant
               then frontAtteint := (vecteurAff[i]>FseuilSynchro)
               else frontAtteint := (vecteurAff[i]<FseuilSynchro);
         while not frontAtteint and (i<NbrePreAcqGTI2) do begin
              // on attend d'tre en dessous
              if montant
                 then frontAtteint := vecteurAff[i]<FseuilSynchro
                 else frontAtteint := vecteurAff[i]>FseuilSynchro;
              inc(i);
         end;
         if i>0 then // on attend la traverse
           repeat
              inc(i);
              if montant
                 then frontAtteint := vecteurAff[i]>FseuilSynchro
                 else frontAtteint := vecteurAff[i]<FseuilSynchro;
           until (i>NbrePreAcqGTI2) or frontAtteint;
         if i>NbrePreAcqGTI2
            then begin
               i := 0;
               seuilTrouve := true
            end
            else begin
               pente := 0;
               for j := i+4 to i+8 do
                   pente := pente + vecteurAff[j];
               pente := pente-vecteurAff[i]*4;
               if montant
                   then seuilTrouve := pente>0
                   else seuilTrouve := pente<0;
               if i>0 then dec(i);
            end;
         if (seuilTrouve or not(testPente)) and
            (abs(FseuilSynchro)<maxi/20) and
            monocoup and
            chercheDroite(vecteurAff,i,origine,pente)
            then begin // faire comme si FseuilSynchro=0
            try
            j := ceil(-origine/pente);
            if (j<i) and (j>0) then i := j;
            except
            end;
         end;
         with voieTemps do begin
              deltat := vecteurAff[1]-vecteurAff[0];
              for j := 0 to nbre do
                  vecteurAff[j] := deltat*(j-i); // t=0 pour passage du seuil
         end;
end end; // setOrigineTemps

Procedure TgrapheOsc.TestSeuil;
var i,j : integer;
    frontAtteint : boolean;
    pente : double;
    Deltat : double;
begin
seuilTrouve := true;
if (voieSynchro=voieT) then exit;
seuilTrouve := false;
         i := 0;
         frontAtteint := abs(vecteurTestS[i]-FseuilSynchro)<(voieOrphy[voieSynchro].maxi/100);
         if montant
            then frontAtteint := frontAtteint and (vecteurTestS[i]>FseuilSynchro)
            else frontAtteint := frontAtteint and (vecteurTestS[i]<FseuilSynchro);
          while not frontAtteint and (i<NbrePreAcqGTI2) do begin
              // on attend d'tre en dessous
              if montant
                 then frontAtteint := vecteurTestS[i]<FseuilSynchro
                 else frontAtteint := vecteurTestS[i]>FseuilSynchro;
              inc(i);
           end;
           if i>0 then // on attend la traverse
           repeat
              inc(i);
              if montant
                 then frontAtteint := vecteurTestS[i]>FseuilSynchro
                 else frontAtteint := vecteurTestS[i]<FseuilSynchro;
           until (i>NbrePreAcqGTI2) or frontAtteint;
         if i>NbrePreAcqGTI2
            then begin
               i := 0;
               seuilTrouve := true; // on fait comme si !
            end
            else begin
               pente := 0;
               for j := i+4 to i+8 do
                   pente := pente + vecteurTestS[j];
               pente := pente-vecteurTestS[i]*4;
               if montant
                   then seuilTrouve := pente>0
                   else seuilTrouve := pente<0;
               if i>0 then dec(i);
            end;
         with voieTemps do begin
              deltat := vecteurTestT[1]-vecteurTestT[0];
              for j := 0 to nbreTest do
                  vecteurTestT[j] := deltat*(j-i); // t=0 pour passage du seuil
         end;
end; // TestSeuil

Procedure TgrapheOsc.SetIndexPeriode(var indexPeriode : integer);
var i,i0,debut,fin : integer;
    ref,ecart,ecartMax : double;
    Deltat : double;
begin
         deltat := voieTemps.vecteurAff[1]-voieTemps.vecteurAff[0];
         i0 := round(-voieTemps.vecteurAff[0]/deltat); // index de t=0
         ref := voieOrphy[voieSynchro].vecteurAff[i0];
         if abs(ref-seuilSynchro)>voieOrphy[voieSynchro].maxi/10 then exit;
         debut := i0+indexPeriode-(indexPeriode div 20);
         fin := i0+indexPeriode+(indexPeriode div 20);
         ecartMax := voieOrphy[voieSynchro].maxi/10;
         for i := debut to fin do begin
             ecart := abs(voieOrphy[voieSynchro].vecteurAff[i]-ref);
             if ecart<ecartMax then begin
                ecartMax := ecart;
                indexPeriode := i-i0;
             end;
         end;
end;

Function TgrapheOsc.VerifPente : boolean;
var i,j : integer;
    frontAtteint : boolean;
    pente : double;
begin
         penteCorrecte := true;
         result := voieSynchro=voieT;
         if result then exit;
with voieOrphy[voieSynchro] do begin
         result := nbre>NbrePreAcqGTI2;
         if not result or not testPente then exit;
         i := 0;
         frontAtteint := abs(vecteurAff[i]-FseuilSynchro)<(maxi/100);
         if montant
            then frontAtteint := frontAtteint and (vecteurAff[i]>FseuilSynchro)
            else frontAtteint := frontAtteint and (vecteurAff[i]<FseuilSynchro);
          while not frontAtteint and (i<NbrePreAcqGTI2) do begin
         // on attend d'tre en dessous
              if montant
                 then frontAtteint := vecteurAff[i]<FseuilSynchro
                 else frontAtteint := vecteurAff[i]>FseuilSynchro;
              inc(i);
           end;
           if i>0 then // on attend la traverse
           repeat
              inc(i);
              if montant
                 then frontAtteint := vecteurAff[i]>FseuilSynchro
                 else frontAtteint := vecteurAff[i]<FseuilSynchro;
           until (i>NbrePreAcqGTI2) or frontAtteint;
         if i>NbrePreAcqGTI2
            then result := false
            else begin
               pente := 0;
               for j := i+4 to i+8 do
                   pente := pente + vecteurAff[j];
               pente := pente-vecteurAff[i]*4;
               if montant
                  then penteCorrecte := pente>0
                  else penteCorrecte := pente<0;
           end;
end end; // verifPente

Function TcourbeOsc.CalibreVoie : strUnite;
begin
   if Fmaxi*Fmini<0
      then if (Fmaxi+Fmini)<(Fmaxi-Fmini)/10
            then result := ''+formatUnite(Fmaxi)
            else result := formatUniteDouble(Fmini,Fmaxi)
      else if maxi>0
           then result := formatUnite(Fmaxi)
           else result := formatUnite(Fmini)
end;

procedure TgrapheOsc.TraceReticule(voie : indiceVoieYD);
var xi,yi : integer;
begin
      with voieTemps do xi := XWindow(vecteurAff[grapheOsc.nbre]);
      with voieOrphy[voie] do yi := XWindow(valeurAff);
      canvas.pen.color := voieOrphy[voie].couleur;
      canvas.brush.color := canvas.pen.color;
      canvas.pen.width := penWidth;
      MyLine(xi-scaleCurseur,yi,xi+scaleCurseur,yi);
      MyLine(xi,yi-scaleCurseur,xi,yi+scaleCurseur);
end;// TraceReticule 

Function TcourbeOsc.AffVoltmetre : strUnite;
var posVirgule,i : integer;
begin
    if valeurAff=sansSignif
    then result := ' Erreur'
    else begin
       result := FormatUniteExp(valeurAff,ExposantAff,NbreAvantVirgule);
       posVirgule := pos('.',result);
       if posVirgule=0 then posVirgule := pos(',',result);
       for i := 0 to (NbreAvantVirgule-PosVirgule) do
           result := ' '+result;
    end;    
end;

Procedure Tcourbe.setMinMax;
var maxmax : double;
    exponent : integer;
begin
   VerifMinMaxDouble(Fmini,Fmaxi);
   MaxMax := abs(Fmaxi);
   if MaxMax<abs(Fmini) then MaxMax := abs(Fmini);
   Exponent := floor(Log10(MaxMax));
   if (Exponent mod 3) < 0
      then Exponent := (Exponent div 3)-1
      else Exponent := Exponent div 3;
{ la division euclidienne des informaticiens <> div des maths ! }
   ExposantAff := Dix(3*Exponent);
   NbreAvantVirgule := floor(Log10(MaxMax/ExposantAff))+2;
end; // setMinMax

Function TgrapheOsc.isAxeX(x,y : integer) : boolean;
begin
    result := (y>limiteCourbe.bottom) and
              (y<(limiteCourbe.bottom+hautCarac)) and
              (x>limiteCourbe.left) and
              (x<(limiteCourbe.right-8*largCarac))
end;

procedure TgrapheOsc.changeEchelleX(xnew,xold : integer);
begin with voieTemps do begin
     xnew := xnew-limiteCourbe.left;
     if xnew<16 then exit;
     xold := xold-limiteCourbe.left;
     A := A*xnew/xold;
     Maxi := Mini+(limiteCourbe.right-limiteCourbe.left)/A;
end end;

procedure TgrapheOsc.setOrigineGTS2;
var tampon : array[indiceVoieYD] of vecteur;

Procedure SurEchantillonFFT(var valeur : vecteur);
var  j,k : integer;
     oldNmes,newNmes,pas : integer;
     nbreFFT : integer;
     fftReel,fftImag : vecteurFFT;
begin
    oldNmes := nbreGTS2;
    newNmes := nbre;
    pas := newNmes div oldNmes;
    for k := oldNmes to pred(newNmes) do valeur[k] := 0;
    Fourier(newNmes,valeur,fftReel,fftImag,cZero,NbreFFT);
    j := NbreFFT div 2 div pas;
    for k := 0 to pred(j) do begin
        fftReel[k] := fftReel[k]*pas;
        fftImag[k] := fftImag[k]*pas;
        fftReel[NbreFFT-k] := fftReel[NbreFFT-k]*pas;
        fftImag[NbreFFT-k] := fftImag[NbreFFT-k]*pas;
    end;
    for k := j to NbreFFT-j do begin
        fftReel[k] := 0;
        fftImag[k] := 0;
    end;
    FourierInverse(NewNmes,NbreFFT,fftReel,fftImag,valeur);
end; // SurEchantillonFFT

procedure RemoveGlitche(var Y : vecteur);
const ordreG = 2;
var
   j : integer;
   ecart : double;
   dY : vecteur;
begin
   dY[0] := 0;
   for j := 1 to pred(nbreGTS2) do dY[j] := y[j]-y[j-1];
   ecart := 0;
   for j := 1 to 2*ordreG do ecart := ecart+dY[j];
   for j := ordreG to pred(nbreGTS2-ordreG) do begin
       // on enlve un point au dbut pour remettre un  la fin
       ecart := ecart+dy[j+ordreG]-dy[j-ordreG]-dy[j]-dy[j+1];
       // on enlve les carts tudis
       if (dy[j]*dy[j+1]<0) and
          (abs(dy[j])>ecart) and
          (abs(dy[j+1])>ecart) then begin
              Y[j] := (Y[j+1]+Y[j-1])/2;
              dy[j] := Y[j]-Y[j-1];
              dy[j+1] := y[j+1]-Y[j];
       end;
       ecart := ecart+dy[j]+dy[j+1];
   end;
end; // RemoveGlitche

Procedure SurEchantillonSinc;
var i,j,k,jk : integer;
    pas,coeff : double;
    valeurCourante : array[indiceVoieYD] of double;
    v : indiceVoieYD;
begin
     pas := (voieTemps.vecteurAff[1]-voieTemps.vecteurAff[0])/
            (voieTempsGTS2.vecteurAff[1]-voieTempsGTS2.vecteurAff[0]);
     for i := 0 to pred(nbre) do begin
         j := round(i*pas);
         for v := voieG to voieJ do
             valeurCourante[v] := 0;
         for k := -20 to 20 do begin
             jk := j+k;
             if jk<0 then continue;
             if jk>=nbreGTS2 then break;             
             coeff := sinc(i*pas-jk);
             for v := voieG to voieJ do
                 valeurCourante[v] := valeurCourante[v]+tampon[v,jk]*coeff;
         end;
         for v := voieG to voieJ do
             voieGTS2[v].vecteurAff[i] := valeurCourante[v];
     end;
end; // surEchantillonSinc

Procedure SurEchantillonLin;
var i,j,i0 : integer;
    pas : double;
    valeurCourante : array[indiceVoieYD] of double;
    ecart : array[indiceVoieYD] of double;
    v : indiceVoieYD;
begin
     pas := (voieTemps.vecteurAff[1]-voieTemps.vecteurAff[0])/
            (voieTempsGTS2.vecteurAff[1]-voieTempsGTS2.vecteurAff[0]);
     for v := voieG to voieJ do begin
         valeurCourante[v] := 0;
         ecart[v] := 0;
     end;
     j := 0;
     i0 := 0;
     for i := 0 to pred(nbre) do begin
         if (j<NbreGTS2) and
            (voieTempsGTS2.vecteurAff[j]<=voieTemps.vecteurAff[i]) then begin
            for v := voieG to voieJ do begin
                valeurCourante[v] := tampon[v,j];
                ecart[v] := (tampon[v,j+1]-tampon[v,j])*pas;
            end;
            inc(j);
            i0 := i;
         end;
         for v := voieG to voieJ do 
             voieGTS2[v].vecteurAff[i] := valeurCourante[v]+ecart[v]*(i-i0);
     end;
end; // surEchantillonLin

var v : indiceVoieYD;
begin
     for v := voieG to voieJ do begin
         tampon[v] := voieGTS2[v].vecteurAff;
         tampon[v,nbreGTS2] := tampon[v,pred(nbreGTS2)];
         if voieGTS2[v].active then removeGlitche(tampon[v]);
     end;
     case interpolation of
         iLin: surEchantillonLin;
         iSinc: surEchantillonSinc;
         iFFT: for v := voieG to voieJ do
               if voieGTS2[v].active then surEchantillonFFT(tampon[v]);
     end;
end;

Procedure TgrapheOsc.PointProche(var x,y : Integer;var indexVoie,indexPoint : integer);
var
   i : integer;
   Xi,Yi,newX,newY : Integer;
   distance,distanceMin : integer;
   voie : indiceVoieYD;
begin
  indexPoint := -1;
  newX := x;
  newY := y;
  distanceMin := 32;
  indexVoie := voieT;
  if y>(limiteCourbe.bottom-4) then exit;
  for i := 0 to pred(NbreAff) do begin
        with voieTemps do xi := Xwindow(vecteurAff[i]);
        for voie := low(indiceVoieYD) to high(indiceVoieYD) do
            with voieOrphy[voie] do if active then begin
            yi := Xwindow(vecteurAff[i]);
            distance := abs(X-Xi)+abs(Y-Yi);
            if (distance<distanceMin) then begin
               distanceMin := distance;
               indexPoint := i;
               newX := Xi;
               newY := Yi;
               indexVoie := voie;
            end;
            end;
      end;
      if indexPoint>=0 then begin
         x := newX;
         y := newY;
      end;
end;

end.

